/*
*
* written by Roland Mnsson, Lund University Computing Center, Sweden
* roland_m@ldc.lu.se
* put by the author into the public domain
*
*/



#include <types.h>
#include <memory.h>
#include <errors.h>
#include <resources.h>

#include	"::vs:wind.h"
#include	"::vs:vsdata.h"

/* ConfigRec */
#include "configrec.h"
/* putln */
#include <Dialogs.h>	/* Maclook.h needs it */
#include "maclook.h"
/* VSmaxwidth */
#include "::vs:vsinterf.h"


/* Local things: */
#include "LUU.h"


/* Even more local things: */
static int get_trsl (short int id, BYTE **table);
static void setup_def_table (BYTE **table);



/*************** external variables ***************/


extern WindRec *screens;			/* The screen array from maclook.c */
extern int scrn;						/* The current screen from maclook.c */
extern VSscrn *VSwhereis(i);


/*************** #defines ***************/

#define	TRSL			'TRSL'	/* resource type for translation */
#define	US82US7		257		/* Resource ids for 'TRSL' resources */
#define	US2NAT		258		/* Resource ids for 'TRSL' resources */
#define	NAT2US		259		/* Resource ids for 'TRSL' resources */

/*************** global variables ***************/

BYTE	*us82us7table,*us2nattable,*nat2ustable;

char	tmp[100];							/* only used for debugging */


/*************** translation routines ***************/

/***** trInit *****/

/* 
*	Be very careful with calling putln from this procedure, since
*	putln uses the translation tables. If the tables are not setup
*	garbage output will appear. This is not harmful, but very
*	annoying.
*/

void trInit (void)
{
	if (get_trsl ((short int) US82US7, &us82us7table))
		setup_def_table (&us82us7table);
	if (get_trsl ((short int) US2NAT, &us2nattable))
		setup_def_table (&us2nattable);
	if (get_trsl ((short int) NAT2US, &nat2ustable))
		setup_def_table (&nat2ustable);
	putln ("past initTrans()");
}

static int get_trsl (short int id, BYTE **table)
{
	Handle h;
	int size;

	h = GetResource (TRSL,id);
	if ((h==NULL) || (ResError()!=noErr)) {
		sprintf (tmp,"Trouble getting translation resource id %d", id);
		putln (tmp);
		return (1);
	}
	size = GetHandleSize(h);
	if (size!=256) {
		sprintf (tmp,"Translation resource #%d is not of correct length (is %d bytes)",id,size);
		putln(tmp);
		return (1);
	}
	HLock (h);
	*table = (BYTE *) *h;
	return (0);
}


static void setup_def_table (BYTE **table)
{
	Handle h;
	int i;
	BYTE b,*bptr;
	

	putln ("Setting up default table");
	h=NewHandle (256);
	for (bptr=*h,i=0; i<256; i++,bptr++) {
		b=(BYTE) i;
		*bptr=b;
	}
	HLock (h);
	*table = (BYTE *) *h;	
}


/*
*
* Converts a char from 8-bit national to 7-bit national if National Chars
* is ON, otherwise converts to 7-bit US (default mapping).
*
*/

void tr8to7 (int *ascii)
{
	BYTE b;
	
	b = (BYTE) *ascii;
	if (screens[scrn].national)
		*ascii = nat2ustable[b];			/* national chars ON */
	else
		*ascii = us82us7table[b];			/* national chars OFF */
}


/***** tr7to8 *****/
void tr7to8 (int *ascii)
{
	BYTE b;
	
	if (screens[scrn].national) {
		b = (BYTE) *ascii;
		*ascii = us2nattable[b];
	}
}

/***** trbuf8to7 *****/
void trbuf8to7 (BYTE *buf, int len)
{
	int i;
	BYTE *p;

	if (screens[scrn].national)
		for (i=0,p=buf; i<len; i++,p++)
			*p = nat2ustable[*p];						/* national chars ON */
	else
		for (i=0,p=buf; i<len; i++,p++)
			*p = us82us7table[*p];						/* national chars OFF */
			
}


/***** trbuf7to8 *****/
void trbuf7to8 (BYTE *buf, int len)
{
	int i;
	BYTE *p;

	if (screens[scrn].national)
		for (i=0,p=buf; i<len; i++,p++)
			*p = us2nattable[*p];
		
}


int transBuffer (void)		/* translate entire buffer */
{
	VSscrn	*vsscreen;
	VSline	*vslin,*p;
	int		lineNo,maxLineNo;
	int		width;				/* allocated witdth of window (80/132) */
	char		tmp[80];				/* only for debugging */
	
	vsscreen = VSwhereis (screens[scrn].vs);
	vslin = vsscreen->buftop;
	putln ("in transBuffer, well and alive");
/*	sprintf (tmp,"VSgetlines(screens[scrn].vs):%d", VSgetlines(screens[scrn].vs)); putln (tmp);
	sprintf (tmp,"VSmaxwidth(screens[scrn].vs):%d", VSmaxwidth(screens[scrn].vs)); putln (tmp);
	sprintf (tmp,"vsscreen->lines:%d\n",vsscreen->lines); putln (tmp);
	sprintf (tmp,"vsscreen->maxlines:%d\n",vsscreen->maxlines); putln (tmp);
	sprintf (tmp,"vsscreen->numlines:%d\n",vsscreen->numlines); putln (tmp);
	sprintf (tmp,"vsscreen->allwidth:%d\n",vsscreen->allwidth); putln (tmp);
	sprintf (tmp,"vsscreen->maxwidth:%d\n",vsscreen->maxwidth); putln (tmp);
	sprintf (tmp,"vsscreen:%08x\n",vsscreen);  putln (tmp);
	sprintf (tmp,"vslin:%08x\n",vslin);  putln (tmp);
	sprintf (tmp,"next:%08x\n",vslin->next);  putln (tmp);
	sprintf (tmp,"prev:%08x\n",vslin->prev);  putln (tmp);
	sprintf (tmp,"text:%08x\n\n",vslin->text);  putln (tmp);
*/
	width = VSmaxwidth(screens[scrn].vs)+1;	/* VSmaxwidth returns 79 or 131 */
	p = vslin;
	maxLineNo = vsscreen->numlines+VSgetlines(screens[scrn].vs); /* VSgetlines returns 24 or whatever */
	for (lineNo=1; lineNo<=maxLineNo; lineNo++) {
		/*sprintf(tmp,"lineNo:%d, p:%08x, starts with:%c%c%c%c",lineNo,p,*(p->text),*(p->text+1),*(p->text+2),*(p->text+3)); putln(tmp);*/
		if (p==NULL) { putln ("p is NULL"); return (-1); }
		if (p->text==NULL) { putln ("p->text is NULL"); return (-1); }
		if (screens[scrn].national)
			trbuf7to8 (p->text,width);		/* to national chars */
		else {
			screens[scrn].national=1;
			trbuf8to7 (p->text,width);		/* to US chars */
			screens[scrn].national=0;
		}
		p = p->next;
	}
	sprintf (tmp, "transBuffer:did convert %d lines", lineNo-1); putln (tmp);
	return 0;
}
