/*
 * Present dialog allowing user to select color for various parts of game.
 */


# include	"TransSkel.h"

# include	"Boxes.h"


# define	nColors		7
# define	noColor		(-1)
# define	frameWidth	3	/* width of color hiliting frame */


typedef enum		/* Color dialog item numbers */
{
	iOK = 1,
	iCancel,
	iPalette,			/* color palette user item */
	iDot,				/* these are radio buttons */
	iSide,
	iScoreboard,
	iPlayer1,
	iPlayer2,
	iPlayer3,
	iPlayer4,
	iDotColor,			/* these are user items for drawing current color */
	iSideColor,			/* for each thing associated with radio buttons */
	iScoreboardColor,	/* (note that radio button and color box are */
	iPlayer1Color,		/* separated by iDotColor - iDot) */
	iPlayer2Color,
	iPlayer3Color,
	iPlayer4Color,
	iOutline
};


static Rect		palRect;
static short	colorHSize, colorVSize;	/* dimensions of color rectangles */
static short	curRadio;				/* current radio button */
static short	curColor;				/* currently selected color */
static short	colors[nColors] =
{
	blackColor,
	redColor,
	greenColor,
	blueColor,
	cyanColor,
	magentaColor,
	yellowColor
};


static short
ColorIndex (short color)
{
short	i;

	for (i = 0; i < nColors; i++)
	{
		if (color == colors[i])
			return (i);
	}
	return (noColor);
}


static void
PalColorRect (short i, Rect *rp)
{
	i = ColorIndex (i);
	SetRect (rp, 0, 0, colorHSize, colorVSize);
	OffsetRect (rp, palRect.left + frameWidth,
					palRect.top + i * (colorVSize + frameWidth) + frameWidth);
}


/*
 * This selects the color even if the color is already selected, which is handy
 * for update events.
 */

static void
SelectColor (short color)
{
Rect	r;

	if (curColor != noColor)
	{
		PalColorRect (curColor, &r);
		InsetRect (&r, -frameWidth + 1, -frameWidth + 1);
		PenMode (patBic);
		FrameRect (&r);
		PenNormal ();
	}
	PalColorRect (curColor = color, &r);
	ForeColor (curColor);
	InsetRect (&r, -frameWidth + 1, -frameWidth + 1);
	FrameRect (&r);
	ForeColor (blackColor);
}


static short
ColorHit (Point pt)
{
short	i;
Rect	r;

	if (PtInRect (pt, &palRect))
	{
		for (i = 0; i < nColors; i++)
		{
			PalColorRect (colors[i], &r);
			InsetRect (&r, -frameWidth + 1, -frameWidth + 1);
			if (PtInRect (pt, &r))		/* maybe should track it */
				return (i);
		}
	}
	return (-1);
}


/*
 * Draw color box associated with a radio button.  The radio button is
 * the item number - (iDotColor - iDot).  This is used for update events
 * and when a color is selected for a button in the filter procedure.
 */

static pascal void
DrawColorBox (DialogPtr dlog, short item)
{
Rect		r;
short		color;

	color = (short) SkelGetDlogCtlRefCon (dlog, item - (iDotColor - iDot));
	ForeColor (color);
	SkelGetDlogRect (dlog, item, &r);
	PaintRect (&r);
	ForeColor (blackColor);
}


/*
 * Draw the palette.  The item is ignored because it has to be iPalette
 */

static pascal void
DrawPalette (DialogPtr dlog, short item)
{
Rect	r;
short	i;

	for (i = 0; i < nColors; i++)
	{
		PalColorRect (colors[i], &r);
		ForeColor (colors[i]);
		PaintRect (&r);
	}
	ForeColor (blackColor);
	SelectColor (curColor);
}


/*
 * Filter function for board dialog.  Checks for hits in the board
 * size rectangle and follow the mouse as long as it's held down.
 */

static pascal Boolean
ColorFilter (DialogPtr dlog, EventRecord *evt, short *item)
{
Point	pt;
short	i, j;

	if (evt->what == mouseDown)
	{
		pt = evt->where;
		GlobalToLocal (&pt);
		if ((j = ColorHit (pt)) >= 0)	/* mouse down in color rect */
		{
			i = -1;
			for (;;)
			{
				if (j >= 0 && j != i)	/* new color selected? */
				{
					i = j;
					SelectColor (colors[i]);
					SkelSetDlogCtlRefCon (dlog, curRadio, (long) colors [i]);
					DrawColorBox (dlog, curRadio + (iDotColor - iDot));
				}
				if (!StillDown ())				/* now track mouse */
					break;
				GetMouse (&pt);
				j = ColorHit (pt);
			}
			*item = iPalette;
			return (true);
		}
	}
	return (false);
}


void
ColorDialog (void)
{
ModalFilterProcPtr	filter;
DialogPtr	dlog;
GrafPtr		savePort;
short		item = 0;
short		i;
long		color;

	dlog = GetNewDialog (colorDlog, nil, (WindowPtr) -1L);

	SkelPositionWindow (dlog, skelPositionOnParentDevice, horizRatio, vertRatio);

	SetCursor (&arrow);
	GetPort (&savePort);
	SetPort (dlog);

	/* install OK button outliner */
	SkelSetDlogButtonOutliner (dlog, iOutline);

	/* get palette rect and set draw proc for palette userItem */
	SkelGetDlogRect (dlog, iPalette, &palRect);
	SkelSetDlogProc (dlog, iPalette, DrawPalette);

	/* set draw proc for color boxes associated with radio buttons */
	for (i = iDotColor; i <= iPlayer4Color; i++)
		SkelSetDlogProc (dlog, i, DrawColorBox);

	/* these must be set before dialog is drawn */
	/* so that palette is drawn correctly */
	colorHSize = palRect.right - palRect.left - 2 * frameWidth;
	colorVSize = (palRect.bottom - palRect.top - (nColors + 1) * frameWidth) / nColors;

	SkelSetDlogCtlRefCon (dlog, iDot, (long) GetDotColor ());
	SkelSetDlogCtlRefCon (dlog, iSide, (long) GetSideColor ());
	SkelSetDlogCtlRefCon (dlog, iScoreboard, (long) GetScoreboardColor ());
	SkelSetDlogCtlRefCon (dlog, iPlayer1, (long) GetPlayerColor (0));
	SkelSetDlogCtlRefCon (dlog, iPlayer2, (long) GetPlayerColor (1));
	SkelSetDlogCtlRefCon (dlog, iPlayer3, (long) GetPlayerColor (2));
	SkelSetDlogCtlRefCon (dlog, iPlayer4, (long) GetPlayerColor (3));
	
	/* select dot button and set palette for it */

	SkelSetDlogRadioButtonSet (dlog, iDot, iPlayer4, curRadio = iDot);
	curColor = noColor;		/* this has to be done before first SelectColor() */
	SelectColor (GetDotColor ());

	ShowWindow (dlog);
	
	for (;;)
	{
		filter = SkelDlogFilter (ColorFilter, false);
		SkelDlogDefaultItem (iOK);
		SkelDlogCancelItem (iCancel);
		ModalDialog (filter, &item);
		SkelRmveDlogFilter ();
		if (item == iOK || item == iCancel)
			break;
		if (iDot <= item && item <= iPlayer4)
		{
			SkelSetDlogRadioButtonSet (dlog, iDot, iPlayer4, item);
			SelectColor ((short) SkelGetDlogCtlRefCon (dlog, curRadio = item));
		}
	}

	SetPort (savePort);
	if (item == iOK)
	{
		/* set color values in here and invalidate board window to force redraw */
		SetDotColor ((short) SkelGetDlogCtlRefCon (dlog, iDot));
		SetSideColor ((short) SkelGetDlogCtlRefCon (dlog, iSide));
		SetScoreboardColor ((short) SkelGetDlogCtlRefCon (dlog, iScoreboard));
		SetPlayerColor (0, (short) SkelGetDlogCtlRefCon (dlog, iPlayer1));
		SetPlayerColor (1, (short) SkelGetDlogCtlRefCon (dlog, iPlayer2));
		SetPlayerColor (2, (short) SkelGetDlogCtlRefCon (dlog, iPlayer3));
		SetPlayerColor (3, (short) SkelGetDlogCtlRefCon (dlog, iPlayer4));
		ForceWindowUpdate ();
	}
	DisposeDialog (dlog);
}
