/*
** This is the TApp class definition.  This class is an adaptation of 
** the class of the same name in the book, Elements of C++ Macintosh 
** Programming, by Dan Weston.
**
** Copyright  1991 Mark Gross 
** (RR2, Box 84, Clayville, NY 13322); 		
** this file may be published everywhere, but no charge
** may be made for its use.  Please contact me about any bugs found.
** I'm also looking for Macintosh development work, so drop me a line
** if you think I could help.  
**
** If you're trying to learn Macintosh programming, then this
** file will not be of much value.
** DON'T READ THIS FILE!!!  You'll go blind if you do.  
** Look only at the class declaration file (that's the .h file).  
** The only thing you need to know is that it works (as far as 
** I know). It isn't commented; for comments read Dan Weston's book.
** See also Macintosh Programming Primer, vol I & II
** Inside Macintosh I-VI, THINK Reference, THINK C User's Manual, and
** Tech notes.
*/

#include "TApp.h"
#include <oops.h>
#include "magics.h"

/*	clipboard support		*/

OSType	TApp::CanAcceptClipType(void)
{
	return '????';
}/*end of function*/

void	TApp::GetClipFromSystem(void)
{
	long	offset, result;
	Handle 	newData;
	OSType	newType;
	
	newData = NewHandle(0);
	newType = CanAcceptClipType();
	result = GetScrap(newData, newType, &offset);
	if (result > 0)
	{
		if(fClipData != NULL)
			DisposHandle(fClipData);
		fClipData = newData;
		fClipType = newType;
	}/*end if*/
}/*end of function*/

	
void	TApp::GiveClipToSystem(void)
{
	long	result, size;
	PScrapStuff 	scrapInfo;
	
	if(fClipData != NULL)
	{
		result = ZeroScrap();
		if(result != noErr)
			return;
		
		size = GetHandleSize(fClipData);
		HLock(fClipData);
		result = PutScrap(size, fClipType, *fClipData);
		HUnlock(fClipData);
	}
	/*
	** update our scrapcount fiedl so we can tell 
	** if scrap has changed later on
	*/
	scrapInfo = InfoScrap();
	fLastScrapCount = scrapInfo->scrapCount;
}/*end of function*/

	
Boolean	TApp::ClipHasChanged(void)
{
	PScrapStuff	scrapInfo;
	
	scrapInfo = InfoScrap();
	return (scrapInfo->scrapCount != fLastScrapCount);
}/*end offunction*/


void	TApp::CheckForDASwitch(WindowPtr theFrontWindow)
{
	Boolean	DAWindowOnTop;
	
	if(theFrontWindow == NULL)
		return;
	
	DAWindowOnTop = ((WindowPeek)theFrontWindow)->windowKind < 0;
	/* if the state has changed since we last check it, then
	** do clipboard conversion
	*/
	
	if(DAWindowOnTop != fDAonTop)
	{
		fDAonTop = DAWindowOnTop;
		
		if(DAWindowOnTop)
			GiveClipToSystem();
		else
		{
			if(ClipHasChanged())
				GetClipFromSystem();
		}
	}/*end outer if*/
}
	
	
	
/*	constructor and destructor		*/
TApp*	TApp::Init(void)
{
	Handle	menuBar;
	
	/*initialize the class members*/
	fCurDoc = NULL;
	fDone = false;
	fInBackground = false;
	fClipData = NULL;
	fClipType = '????';
	fDAonTop = false;
	fLastScrapCount = 0;
	
	/*INITIALIZE MAC TOOLBOX COMPONTENTS*/
	InitGraf(&thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs( NULL);
	InitCursor();
	
	SysEnvirons(curSysEnvVers, &fenvRec);
	MaxApplZone();

	fDocList  = ((TList *)new(TList))->Init();
	fHaveWaitNextEvent = TrapAvailable();
	menuBar = GetNewMBar(rMenuBarID);
	SetMenuBar(menuBar);
	DisposHandle(menuBar);
	AddResMenu(GetMHandle(rAppleMenu), 'DRVR');
	DrawMenuBar();
	
	return this;
}/*end of function*/


Boolean	TApp::TrapAvailable(void)
{
	return NGetTrapAddress(0x60,ToolTrap) != 
				NGetTrapAddress(0x9f,ToolTrap);
}/*end of function*/


void	TApp::Delete(void)
{
	delete(fDocList);
}/*end of function*/

	
/*	initializing the application	*/
Boolean		TApp::InitApp(void)
{
	GetClipFromSystem();
	return TRUE;
}/*end of function*/


/*cleaning up after the application	*/
void	TApp::CleanUp(void)
{
	GiveClipToSystem();
}/*end of function*/


/*	making documents			*/
TDoc*	TApp::MakeDoc(SFReply *reply)
{
	TDoc *New;
	
	New = (TDoc *)new(TDoc);
	return (New->Init(GetCreator(), reply));
}/*end of function*/


OSType	TApp::GetCreator(void)
{
	return '????';
}/*end of function*/


void	TApp::AddDocument(TDoc *theDoc)
{
	fDocList->AddItem(theDoc);
	fCurDoc = theDoc;
}/*end of function*/

	
/*	opening new documents		*/
void	TApp::OpenNewDoc(void)
{
	TDoc	*newDoc;
	short	numDocs;
	
	newDoc = MakeDoc(NULL);
	if(newDoc)
	{
		if( (newDoc->MakeWindow(fenvRec.hasColorQD) ) &&
							(newDoc->InitDoc()) )
		{
			numDocs = fDocList->NumItems();
			newDoc->MoveDocWindow(kHPos + (numDocs*kStagger), kVPos + (numDocs*kStagger) );
			newDoc->ShowDocWindow();
			AddDocument(newDoc);
		}
		else
		{
			newDoc->Delete();
			delete(newDoc);
		}
	}/*end if newDoc*/
}/*end of function*/
	

/*	opening old documents		*/
int	TApp::GetNumFileTypes(void)
{
	return ZERO;
}/*end of function*/


void	TApp::GetFileTypeList(void)
{
	fFileTypeList[0]=0L;
}/*end of function*/


void	TApp::OpenOldDoc(void)
{
	SFReply		reply;
	Point		p;
	int	i;
	SFTypeList		mySFList;
	
	
	GetFileTypeList();
	for(i=0;i<4;i++)
		mySFList[0] = fFileTypeList[0];
	p.h = 100; 
	p.v = 100;

	SFGetFile( p, "\p", NULL, GetNumFileTypes(), mySFList, NULL, &reply);
	
	/*don't go on if user cancels dialog*/
	if (!reply.good)
		return;
	
	InitOldDoc(&reply);
}/*end of function*/


Boolean		TApp::InitOldDoc(SFReply *reply)
{
	TDoc	*newDoc;
	short	numDocs, refNum;
	
	newDoc = MakeDoc(reply);	
	if(newDoc)
	{
		if( (newDoc->MakeWindow(fenvRec.hasColorQD)) &&
			(newDoc->InitDoc()) )
		{
			numDocs = fDocList->NumItems();
			newDoc->MoveDocWindow(kHPos + (numDocs*kStagger), 
					kVPos + (numDocs*kStagger) );
			newDoc->SetDocWindowTitle( reply->fName);
			refNum = newDoc->OpenDocFile(reply);
			
			if(refNum != 0)
			{
				if( newDoc->ReadDocFile(refNum) )
				{
					newDoc->ShowDocWindow();
					AddDocument(newDoc);
				}
				else
				{
					newDoc->CloseDocFile(refNum);
					newDoc->Delete();
					delete(newDoc);
					return false;
				}
			}
			else
			{
				/*file not opened successfully, but doc created*/
				newDoc->Delete();
				delete(newDoc);
				return false;
			}
		}
		else
		{
			/*make window or init failed*/
			newDoc->Delete();
			delete(newDoc);
			return false;
		}
	}
	else
	{
		/*document never created*/
		return false;
	}/*end nested if's*/
	
	return TRUE; 
}/*end of function*/

	
/*	opening documents form finder	*/
Boolean		TApp::OpenDocFromFinder(void)
{
	short	message, count, i, j;
	unsigned char	strLen;
	AppFile	theApp;
	SFReply	reply;
	Boolean	fileOpened = false;
	
	CountAppFiles( &message, &count);
	if(count == 0)
		return false;
	
	for(i=count;i; i--)
	{
		GetAppFiles(i,&theApp);/*then convert theApp to SFReply*/
		reply.good = TRUE;
		reply.fType = theApp.fType;
		reply.vRefNum = theApp.vRefNum;
		reply.version = theApp.versNum;
		strLen = theApp.fName[0];
		for(j=0; j<=strLen; j++) reply.fName[j] = theApp.fName[j];
		
		/*check here to see if file is acceptable*/
		if(AcceptableFileType(reply.fType) )
		{
			if(InitOldDoc(&reply) )
				fileOpened = TRUE;
		}
	}/*end for loop*/
	
	return fileOpened;
}/*end of function*/



Boolean		TApp::AcceptableFileType(OSType theType)
{
	int	numTypes,i;
	OSType *theTypeList;
	
	numTypes = GetNumFileTypes();
	GetFileTypeList();
	theTypeList = (OSType *)fFileTypeList;
	
	if( (numTypes == 0) || (theTypeList == NULL) )
		return TRUE;
	
	for(i=0;i<numTypes; i++)
	{
		if(theType == *theTypeList++)
			return TRUE;
	}/*end for loop*/
	
	return false;
}/*end of function*/



/*	deleting documents		*/
Boolean 	TApp::CloseADoc(TDoc	*theDoc)
{
	if(theDoc != NULL)
	{
		if(theDoc->DoClose() )
		{
			fDocList->RemoveItem(theDoc);
			if(theDoc == fCurDoc)
				fCurDoc = NULL;
			theDoc->Delete();
			delete(theDoc);
			return TRUE;
		}
	}
	return false;
}/*end of function*/


	
/*	handling events		*/
void	TApp::EventLoop(void)
{
	long	gotEvent;
	EventRecord	theEvent;
	WindowPtr	theFrontWindow;
	
	while(fDone == false)
	{
		theFrontWindow = FrontWindow();
		CheckForDASwitch( theFrontWindow);
		fCurDoc = fDocList->FindDoc(theFrontWindow);
		
		if( fHaveWaitNextEvent)
			gotEvent = WaitNextEvent( everyEvent,
									&theEvent,
									SLEEPVAL,
									NULL);
		else
		{
			SystemTask();
			gotEvent = GetNextEvent( everyEvent, &theEvent);
		}
		
		if( gotEvent == false)
		{
			AppIdle();
			continue;
		}
	
		switch (theEvent.what)
		{
			case nullEvent:
				AppIdle();
				break;
			case mouseDown:
				MouseDown(&theEvent);
				break;
			case mouseUp:
				MouseUp(&theEvent);
				break;
			case keyDown:
			case autoKey:
				KeyDown(&theEvent);
				break;
			case updateEvt:
				UpdateEvt(&theEvent);
				break;
			case diskEvt:
				DiskEvt(&theEvent);
				break;
			case activateEvt:
				ActivateEvt(&theEvent);
				break;
			case kOSEvent:
				OSEvent(&theEvent);
				break;
			default:
				break;
		}/*end switch*/
	}/*end while*/
}/*end function*/

		
void	TApp::AppIdle(void)
{
	if( fCurDoc != NULL)
		fCurDoc->DoIdle();
}/*end function*/


void	TApp::MouseUp(EventRecord	 *theEvent)
{
	;
}

void	TApp::DiskEvt(EventRecord	*theEvent)
{
	;
}

void	TApp::MouseDown(EventRecord	*theEvent)
{
	WindowPtr	theWindow;
	short		partCode;
	long		mResult;
	TDoc *tempDoc;
	
	partCode = FindWindow( theEvent->where, &theWindow);
	tempDoc = fDocList->FindDoc( theWindow);
	switch( partCode)
	{
		case	inSysWindow:
			SystemClick(theEvent, theWindow);
			break;
		
		case	inMenuBar:
			AdjustMenus();
			mResult = MenuSelect(theEvent->where);
			if(HiWord(mResult) != 0)
			{
				DoMenuCommand(HiWord(mResult), LoWord(mResult) );
				HiliteMenu(0);
			}
			break;
		
		case 	inGoAway:
			if (TrackGoAway(theWindow, theEvent->where) )
				CloseADoc(tempDoc);
			break;
		
		case	inDrag:
			if(tempDoc != NULL)
				tempDoc->DoDrag(theEvent);
			break;
		
		case	inGrow:
			if(tempDoc != 0)
				tempDoc->DoGrow(theEvent);
			break;
		
		case	inZoomIn:
		case	inZoomOut:
			if( (TrackBox( theWindow, theEvent->where, partCode) ) &&
				(tempDoc != NULL) )
				tempDoc->DoZoom(partCode);
			break;
		
		case	inContent:
			if (theWindow != FrontWindow() )
				SelectWindow( theWindow);
			else
				if(tempDoc != NULL)
					tempDoc->DoContent(theEvent);
				
			break;
	}/*end switch*/
}/*if of function*/


void	TApp::KeyDown(EventRecord *theEvent)
{
	char	key;
	long	mResult;
	
	key = (char) (theEvent->message & charCodeMask);
	if ( (theEvent->modifiers & cmdKey) && (theEvent->what == keyDown) )
	{
		AdjustMenus();
		mResult = MenuKey( key);
		if(HiWord(mResult) != 0)
		{
			DoMenuCommand(HiWord(mResult), LoWord(mResult) );
			HiliteMenu(0);
			return;
		}
	}
	else
		if (fCurDoc != NULL)
			fCurDoc->DoKeyDown(theEvent);
}/*end of function*/



void	TApp::ActivateEvt(EventRecord *theEvent)
{
	WindowPtr	theWindow;
	TDoc	*tempDoc;
	
	theWindow = (WindowPtr) theEvent->message;
	tempDoc = fDocList->FindDoc( theWindow);
	
	if(tempDoc != NULL)
		tempDoc->DoActivate(theEvent);
}/*end of function*/



void	TApp::UpdateEvt(EventRecord *theEvent)
{
	WindowPtr	theWindow;
	TDoc	*tempDoc;
	
	theWindow = (WindowPtr)(theEvent->message);
	tempDoc = fDocList->FindDoc(theWindow);
	
	if(tempDoc != NULL)
		tempDoc->DoTheUpdate(theEvent);
}/*end of function*/

	
/*	Multifinder support		*/

void	TApp::OSEvent(EventRecord *theEvent)
{
	Boolean	doConvert;
	unsigned char	evType;
	
	evType = (unsigned char) (theEvent->message >>24) &0x00ff;
	switch(evType)
	{
		case kMouseMovedMessage:
			AppIdle();
			break;
		case kSuspendResumeMessage:
			doConvert = ((theEvent->message & kClipConvertMask) != 0 );
			fInBackground = (theEvent->message &kResumeMask) == 0;
			if (fInBackground)
				DoSuspend( theEvent, doConvert);
			else
				DoResume(theEvent, doConvert);
			break;
	}/*end switch*/
}/*end of function*/



void	TApp::DoSuspend(EventRecord *theEvent, Boolean convertClip)
{
	if (convertClip)
		GiveClipToSystem();
	if(fCurDoc != NULL)
	{
		theEvent->modifiers &= (!activeFlag);
		fCurDoc->DoActivate(theEvent);
	}
}/*end of function*/



void	TApp::DoResume(EventRecord *theEvent, Boolean convertClip)
{
	if (convertClip)
		GetClipFromSystem();
	if(fCurDoc != NULL)
	{
		theEvent->modifiers |= activeFlag;
		fCurDoc->DoActivate(theEvent);
	}
}/*end of function*/	
	
	
void		TApp::SetMenuAbility(MenuHandle menu, short item, Boolean enable)
{

	if (enable) 
		EnableItem(menu, item);
	else
		DisableItem(menu, item);
	
}/*end set menu ability member function*/

	
/*	handling menus			*/
void	TApp::AdjustMenus(void)
{
	MenuHandle	menu;
	
	if (fCurDoc != NULL)
		fCurDoc->AdjustDocMenus();
		
	menu = GetMHandle(rFileMenu);
	SetMenuAbility( menu, iNew, TRUE);
	SetMenuAbility( menu, iOpen, CanOpen());
	SetMenuAbility( menu, iQuit, TRUE);
	
	if (fCurDoc == NULL)
	{
		SetMenuAbility( menu, iClose, false);
		SetMenuAbility( menu, iSave, false);
		SetMenuAbility( menu, iSaveAs, false);
		SetMenuAbility( menu, iPageSetup, false);
		SetMenuAbility( menu, iPrint, false);
	}
	
	menu = GetMHandle(rEdit);
	if(fCurDoc == NULL)
	{
		SetMenuAbility( menu, iUndo, fDAonTop);
		SetMenuAbility( menu, iCut, fDAonTop);
		SetMenuAbility( menu, iCopy, fDAonTop);
		SetMenuAbility( menu, iPaste, fDAonTop);
		SetMenuAbility( menu, iClear, fDAonTop);
		SetMenuAbility( menu, iSelectAll, fDAonTop);
	}
	else
	{
		SetMenuAbility(menu, iPaste, (fClipData != NULL) &&
				 (fCurDoc->CanPaste(fClipType) ) );
	}
}/*end of function*/


Boolean 	TApp::CanOpen(void)
{
	return false;
}

void	TApp::DoMenuCommand(short menuID, short menuItem)
{
	short	itemHit;
	Str255	daName;
	short	daRefNum;
	
	if ((fCurDoc != NULL) &&
		(fCurDoc->DoDocMenuCommand(menuID, menuItem)) )
		return;
	
	switch (menuID)
	{
		case	rAppleMenu:
			switch (menuItem)
			{
				case iAbout:
					itemHit = Alert(rAboutID, NULL);
					break;
				default:
					GetItem(GetMHandle(rAppleMenu), menuItem, daName);
					daRefNum = OpenDeskAcc( daName);
					break;
			}/*end inner switch*/
			break;
		
		case	rFileMenu:
			switch (menuItem)
			{
				case iNew:
					OpenNewDoc();
					break;
				case	iOpen:
					OpenOldDoc();
					break;
				case 	iClose:
					CloseADoc(fCurDoc);
					break;
				case iQuit:
					Quit();
					break;
			}/*end inner switch*/
			break;

		case	rEdit:
			if(!SystemEdit(menuItem-1) )
			{
				switch(menuItem)
				{
					case	iUndo:
						DoUndoCmd( fCurDoc);
						break;
					case	iCut:
						DoCutCmd(fCurDoc);
						break;
					case	iCopy:
						DoCopyCmd(fCurDoc);
						break;
					case	iPaste:
						DoPasteCmd(fCurDoc);
						break;
				}/*end inner switch*/
			}/*end if*/
			break;
	}/*end outter switch*/
}/*end of function*/

	
void	TApp::Quit(void)
{
	TIterator	*iter;
	TDoc *nextDoc;
	Boolean OKToQuit = TRUE;
	
	iter = ((TIterator *) new(TIterator))->Init(fDocList);
	
	while( nextDoc = (TDoc*)iter->Next() )
	{
		if(!CloseADoc(nextDoc) )
		{
			OKToQuit = false;
			break;
		}
	}
	if (OKToQuit)
		ExitLoop();
}/*end of function*/


void	TApp::ExitLoop(void)
{
	fDone = TRUE;
}

void	TApp::DoUndoCmd(TDoc *theDoc)
{
	if (theDoc != NULL)
		theDoc->DoUndo();
}/*end of function*/


void	TApp::DoPasteCmd(TDoc *theDoc)
{
	if (theDoc != NULL)
		theDoc->DoPaste(fClipData, &fClipType);
}/*end of function*/


void	TApp::DoCutCmd(TDoc *theDoc)
{
	Handle	newData;
	OSType	newType;
	
	if(theDoc != NULL)
		if(theDoc->DoCut( &newData, &newType) )
		{
			if(fClipData !=NULL)
				DisposHandle(fClipData);
				
			fClipData = newData;
			fClipType = newType;
		}
}/*end of function*/



void	TApp::DoCopyCmd(TDoc *theDoc)
{
	Handle	newData;
	OSType	newType;
	
	if(theDoc != NULL)
		if(theDoc->DoCopy(&newData, &newType) )
		{
			if(fClipData !=NULL)
				DisposHandle(fClipData);
				
			fClipData = newData;
			fClipType = newType;
		}
	
	
	
}/*end of class definition of TDoc class*/

