/*
** This is the TScrollDoc class definition.  This class is an adaptation of 
** the class of the same name in the book, Elements of C++ Macintosh 
** Programming, by Dan Weston.
**
** Copyright  1991 Mark Gross 
** (RR2, Box 84, Clayville, NY 13322); 		
** this file may be published everywhere, but no charge
** may be made for its use.  Please contact me about any bugs found.
** I'm also looking for Macintosh development work, so drop me a line
** if you think I could help.  
**
** If you're trying to learn Macintosh programming, then this
** file will not be of much value.
** DON'T READ THIS FILE!!!  You'll go blind if you do.  
** Look only at the class declaration file (that's the .h file).  
** The only thing you need to know is that it works (as far as 
** I know). It isn't commented; for comments read Dan Weston's book.
** See also Macintosh Programming Primer, vol I & II
** Inside Macintosh I-VI, THINK Reference, THINK C User's Manual, and
** Tech notes.
*/

#include "TScrollDoc.h"
#include "magics.h"


ControlHandle	TScrollDoc::GetVScroll(void)
{
	return fVertScrollBar;
}

ControlHandle	TScrollDoc::GetHScroll(void)
{
	return fHorizScrollBar;
}
	


TDoc* 	TScrollDoc::Init(OSType theCreator, SFReply *SFInfo)
{
	inherited::Init(theCreator, SFInfo);
	fHorizScrollBar = NULL;
	fVertScrollBar = NULL;
	fVOffset = 0;
	fHOffset = 0;
	return this;
}/*end of function*/
	

void		TScrollDoc::Delete(void)
{
	inherited::Delete();
}/*end of function*/

		
Boolean		TScrollDoc::InitDoc(void)
{
	Rect r;
	
	SetRect(&r, 0,0,0,0);
	
	if(!inherited::InitDoc())
		return false;
	
	if(fDocWindow != NULL)
	{
		SetPort(fDocWindow);
		fHorizScrollBar = NewControl(fDocWindow,&r, "",TRUE,
								0, 0, 0, scrollBarProc, 0);
		fVertScrollBar = NewControl(fDocWindow,&r, "",TRUE,
								0, 0, 0, scrollBarProc, 0);
		SizeScrollBars();
		SynchScrollBars();
	}
	return ( (fHorizScrollBar != NULL)&&(fVertScrollBar != NULL) );
}/*end of function*/


void	TScrollDoc::GetContentRect(Rect *r)
{
	*r = fDocWindow->portRect;
	if(fVertScrollBar != NULL)
		r->right -= kScrollBarPos;
	if(fHorizScrollBar != NULL)
		r->bottom -= kScrollBarPos;
}/*end of function*/

	
short	TScrollDoc::GetVertSize(void)
{
	return 0;
}/*end of function*/

short	TScrollDoc::GetHorizSize(void)
{
	return 0;
}/*end of function*/
	
short	TScrollDoc::GetVertLineScrollAmount(void)
{
	return 0;
}/*end of function*/


short	TScrollDoc::GetHorizLineScrollAmount(void)
{
	return 0;
}/*end of function*/

	
short	TScrollDoc::GetVertPageScrollAmount(void)
{
	Rect r;
	
	GetContentRect(&r);
	return r.bottom - r.top - kScrollOverlap;
}/*end of function*/

	
short	TScrollDoc::GetHorizPageScrollAmount(void)
{
	Rect r;
	
	GetContentRect(&r);
	return r.right - r.left - kScrollOverlap;
}/*end of function*/

	
void	TScrollDoc::FocusOnContent(void)
{
	Rect r;
	
	SetPort(fDocWindow);
	SetOrigin(fHOffset, fVOffset);
	GetContentRect(&r);
	ClipRect(&r);
}/*end of function*/


void	TScrollDoc::FocusOnWindow(void)
{
	Rect r;
	
	SetPort(fDocWindow);
	SetOrigin(0,0);
	r = fDocWindow->portRect;
	ClipRect(&r);
}/*end of function*/
	
	
	
void	TScrollDoc::SizeScrollBars()
{
	Rect r, r_temp;
	
	if(fDocWindow != NULL)
	{
		FocusOnWindow();
		r = fDocWindow->portRect;

		if(fVertScrollBar != NULL)
		{
			HideControl(fVertScrollBar);
			SizeControl(fVertScrollBar, kScrollBarWidth,
						(r.bottom - r.top - kScrollBarPos) + 2);
			MoveControl(fVertScrollBar, r.right - kScrollBarPos, -1);
			ShowControl(fVertScrollBar);
			r_temp = (**fVertScrollBar).contrlRect;
			ValidRect(&r_temp);
		}
		
		if(fHorizScrollBar != NULL)
		{
			HideControl(fHorizScrollBar);
			SizeControl(fHorizScrollBar, (r.right - r.left - kScrollBarPos) +2, 
											kScrollBarWidth);
			MoveControl(fHorizScrollBar, -1, r.bottom - r.top - kScrollBarPos);
			ShowControl(fHorizScrollBar);
			r_temp = (**fHorizScrollBar).contrlRect;
			ValidRect(&r_temp);
		}
	}
}/*end of function*/



void	TScrollDoc::AdjustScrollBars(void)
{
	Rect r;
	short dh, dv;
	short currentValue;
	short newMax;
	RgnHandle oldClip;
	
	GetContentRect(&r);
	dh = 0; dv = 0;
	if(fVertScrollBar != NULL)
	{
		currentValue = GetCtlValue(fVertScrollBar);
		newMax = GetVertSize() - (r.bottom - r.top);
		if(newMax < 0)
			newMax = 0;
		if(currentValue > newMax)
			dv = currentValue - newMax;
		SetCtlMax(fVertScrollBar, newMax);
	}
	
	if(fHorizScrollBar != NULL)
	{
		currentValue = GetCtlValue(fHorizScrollBar);
		newMax = GetHorizSize() - (r.right - r.left);
		if(newMax < 0)
			newMax = 0;
		if(currentValue > newMax)
			dh = currentValue - newMax;
		SetCtlMax(fHorizScrollBar, newMax);
	}
	
	if(dh | dv)
	{
		FocusOnContent();
		
		GetContentRect(&r);
		InvalRect(&r);
		
		oldClip = NewRgn();
		GetClip(oldClip);
		SetRect(&r, 0, 0, 0, 0);
		ClipRect(&r);
		ScrollContents(dh,dv);
		
		SetClip(oldClip);
		DisposeRgn(oldClip);
	}
}/*end of function*/


void	TScrollDoc::SetScrollBarValues()
{
	FocusOnWindow();
	if(fHorizScrollBar != NULL)
		SetCtlValue(fHorizScrollBar, fHOffset);
	if(fVertScrollBar != NULL)
		SetCtlValue(fVertScrollBar, fVOffset);
}/*end of function*/



void	TScrollDoc::SynchScrollBars(void)
{
	AdjustScrollBars();
	SetScrollBarValues();
}/*end of function*/



void	TScrollDoc::Activate(void)
{
	FocusOnWindow();
	if(fVertScrollBar != NULL)
		ShowControl(fVertScrollBar);
	if(fHorizScrollBar != NULL)
		ShowControl(fHorizScrollBar);
	gCurrScrollDoc = this;
}/*end of function*/



void	TScrollDoc::Deactivate(void)
{
	FocusOnWindow();
	if(fVertScrollBar != NULL)
		HideControl(fVertScrollBar);
	if(fHorizScrollBar != NULL)
		HideControl(fHorizScrollBar);
}/*end of function*/



void	TScrollDoc::DoTheUpdate(EventRecord *theEvent)
{
	Rect r;
	
	if(fDocWindow != NULL)
	{
		FocusOnContent();
		BeginUpdate(fDocWindow);
		r = (**(fDocWindow->visRgn)).rgnBBox;
		Draw(&r);
		FocusOnWindow();
		DrawControls(fDocWindow);
		DrawGrowIcon(fDocWindow);
		EndUpdate(fDocWindow);
	}
}/*end of function*/

	

void	TScrollDoc::DoContent(EventRecord *theEvent)
{
	Rect contents;
	
	FocusOnWindow();
	GlobalToLocal(&theEvent->where);
	GetContentRect(&contents);
	if(PtInRect(theEvent->where, &contents))
	{
		FocusOnContent();
		ContentClick(theEvent);
	}
	else
		ScrollClick(theEvent);
}/*end of function*/

	

void	TScrollDoc::ScrollClick(EventRecord *theEvent)
{
	ControlHandle whichControl;
	short part;
	
	FocusOnWindow();
	if(part = FindControl(theEvent->where, fDocWindow, &whichControl) )
	{
		switch(part)
		{
			case inThumb:
				DoThumbScroll(whichControl, theEvent->where);
				break;
			case inUpButton:
			case inDownButton:
				DoButtonScroll(whichControl, theEvent->where);
				break;
			case inPageUp:
			case inPageDown:
				DoPageScroll(whichControl,part);
				break;
		}/*end of switch*/
	}
}/*end of function*/

	

void	TScrollDoc::DoGrow(EventRecord *theEvent)
{
	FocusOnWindow();
	inherited::DoGrow(theEvent);
	SizeScrollBars();
	SynchScrollBars();
}/*end of function*/




void	TScrollDoc::DoZoom(short partCode)
{
	FocusOnWindow();
	inherited::DoZoom(partCode);
	SizeScrollBars();
	SynchScrollBars();
}/*end of function*/



	
void	TScrollDoc::ScrollContents(short dh, short dv)
{
	Rect r;
	RgnHandle updateRgn;
	
	GetContentRect(&r);
	updateRgn = NewRgn();
	
	ScrollRect(&r,dh,dv,updateRgn);
	
	fVOffset -= dv;
	fHOffset -= dh;
	
	InvalRgn(updateRgn);
	DoTheUpdate(NULL);
	DisposeRgn(updateRgn);
}/*end of function*/



void	TScrollDoc::Scroll(ControlHandle theControl, short change)
{
	RgnHandle oldClip;
	short diff;
	short oldValue, newValue, minValue, maxValue;
	
	oldClip = NewRgn();
	GetClip(oldClip);
	diff = 0;
	oldValue = GetCtlValue(theControl);
	newValue = oldValue + change;
	
	if(change<0)
	{
		minValue = GetCtlMin(theControl);
		if(newValue < minValue)
			newValue = minValue;
	}
	else
	{
		maxValue = GetCtlMax(theControl);
		if(newValue > maxValue)
			newValue = maxValue;
	}
	diff = oldValue - newValue;
	
	FocusOnContent();
	if(theControl == fHorizScrollBar)
		ScrollContents(diff,0);
	if(theControl == fVertScrollBar)
		ScrollContents(0, diff);
	
	FocusOnWindow();
	SetScrollBarValues();
	
	SetClip(oldClip);
	DisposeRgn(oldClip);
}/*end of function*/



void	TScrollDoc::DoThumbScroll(ControlHandle theControl, Point localPt)
{
	short oldValue, trackResult, newValue, diff;
	
	oldValue = GetCtlValue(theControl);
	trackResult = TrackControl(theControl, localPt, NULL);
	if(trackResult != 0)
	{
		newValue = GetCtlValue(theControl);
		diff = oldValue - newValue;
		FocusOnContent();
		if(theControl == fHorizScrollBar)
			ScrollContents(diff, 0);
		if(theControl == fVertScrollBar)
			ScrollContents(0, diff);
		
		FocusOnWindow();
	}
}/*end of function*/

	


void	TScrollDoc::DoPageScroll(ControlHandle theControl, short part)
{
	short scrollAmount, currentPart;
	Point thePt;
	
	if(theControl == fVertScrollBar)
		scrollAmount = GetVertPageScrollAmount();
	else
		scrollAmount = GetHorizPageScrollAmount();
	
	do
	{
		GetMouse(&thePt);
		currentPart = TestControl(theControl, thePt);
		if(currentPart == part)
		{
			if(currentPart == inPageUp)
				Scroll(theControl, - scrollAmount);
			if(currentPart == inPageDown)
				Scroll(theControl, scrollAmount);
		}
	}while(Button());
}/*end of function*/



void	TScrollDoc::DoButtonScroll(ControlHandle theControl, Point localPt)
{
	short result;
	
	result = TrackControl(theControl, localPt, (ProcPtr) ActionProc) ;
}/*end of function*/


void	TScrollDoc::ContentClick(EventRecord *theEvent)
{

}

