/*
*********************************************************************
*	
*	cd.c
*	Initialization of CD-ROM
*
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*
*/

/* #define DEBUG */

#include <Packages.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "cd.h"

#include "environment.h"
#include "events.h"
#include "main.h"
#include "menus.h"
#include "pnx.h"
#include "pref.h"
#include "pstr.h"
#include "util.h"
#include "window.h"

/*
******************************* Prototypes ***************************
*/

static long GetDirIDFromPath(StringPtr longName,short vRefNum,StringPtr volName);
static OSErr CheckDir(DirSpec *dirSpec, short *vRefNum, long *dirID);
static Boolean ProcessDBFiles(short dbcode);
static Boolean CheckHeader(StringPtr fName, short fd, short dbcode, long rec_size, u_long *nrec, u_short *recsize);
static short InitUserBreak(DialogPtr myDialog);

/*
******************************** Global variables *****************
*/


/**************************************
*	Startup check for an attached EMBL CD-ROM.
*	We compare the volume name to the value stored in a STR# resource
*	(variable gVolLabel).
*/

void InitCD()
{
	register short	i;
	HParamBlockRec	myPB;
	Str255			volName;
	Boolean			foundCD=FALSE, ret;
	OSErr				err;
	DialogPtr		myDialog;
	DialogTHndl		myHandle;
	short				which;
	
	while(!foundCD) {
		/* now check every attached device */
		myPB.volumeParam.ioNamePtr=volName;
		for(i=1,err=noErr;err == noErr;++i) {
			myPB.volumeParam.ioVolIndex=i;
			err=PBHGetVInfo(&myPB,FALSE);					/* get volume information		*/
			if(err == noErr) {								/* error or no more volumes	*/
				/* Check volume name */
				if(!pstrncmp(volName,gVolLabel,*gVolLabel)) {
					/* it looks like an EMBL CD-ROM, now try to initialize it */
					StartWaitCursor();
					ret=InitDB(myPB.volumeParam.ioVRefNum,volName);
				}
			}
		}
		InitCursor();
				
		/* In case no EMBL CD-ROM is attached or something went wrong, we show insertcd
			dialog */
		
		for(i = 0; i < DB_NUM; ++i) {
			if( gDBInfo[i].ndiv != 0 )
				break;
		}
		
		if(i != DB_NUM)
			foundCD = TRUE;
			
		if(! foundCD) {
			SysBeep(10);
			
			if(gSystem.sysVersion >= 0x0700) {
				myHandle = (DialogTHndl)GetResource('DLOG',INSERTCD_DLG);
				(**myHandle).procID = movableDBoxProc;
			}
			myDialog = GetNewCenteredDialog(INSERTCD_DLG,50);
			DisableMenuBar();
			ShowWindow(myDialog);
			DrawDialog(myDialog);
			
			while((which = InitUserBreak(myDialog)) == 0)
				;
			DisposeDialog(myDialog);
			EnableMenuBar();
			InitCursor();
			
			if(which == Cancel)
				ExitToShell();
		}
	}
}


/**************************************
*	Check files on CD for correctness
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

Boolean InitDB(short vRefNum,StringPtr volName)
{
	Str255 name;
	short i;
	Boolean foundOne = FALSE;
	DialogPtr		myDialog;
	OSErr err;

	ShowWindow(myDialog=GetNewCenteredDialog(INITWAIT_DLG,50));

	for( i = 0; i < DB_NUM; ++i) {
		if(gDBInfo[i].ndiv != 0) /* already initialised */
			continue;
			
		RotateWaitCursor();
		GetItem(gMenu[DBNAMES],i+1,name);
		ParamText(name,"\p","\p","\p");
		DrawDialog(myDialog);
		
		gDBInfo[i].DataVRefNum = gDBInfo[i].DataDirID = 0;
		gDBInfo[i].InxVRefNum = gDBInfo[i].InxDirID = 0;
	
		/* Try to use data directory as stored in Prefs file */
		if(*gPrefs.dataDirSpec[i].volName != EOS) {
			if((err = CheckDir(&gPrefs.dataDirSpec[i],
									&gDBInfo[i].DataVRefNum,&gDBInfo[i].DataDirID)) != noErr) {
				sprintf(gError,LoadErrorStr(ERR_PNF,FALSE),gDBInfo[i].DBName);
				ErrorMsg(0);
				*gPrefs.dataDirSpec[i].volName = EOS;	/* reset name on error */
			}
		}
		
		/* get dirID of default data directory */
		if(*gPrefs.dataDirSpec[i].volName == EOS) {
			pstrcpy(name,gDBInfo[i].DataDName);
			if(!(gDBInfo[i].DataDirID = GetDirIDFromPath(name,vRefNum,volName))) {
				gDBInfo[i].ndiv = 0;
				continue;
			}
			else
				gDBInfo[i].DataVRefNum = vRefNum;
		}
				
		/* Try to use index directory as stored in Prefs file */
		if(*gPrefs.inxDirSpec[i].volName != EOS) {
			if((err = CheckDir(&gPrefs.inxDirSpec[i],
									&gDBInfo[i].InxVRefNum,&gDBInfo[i].InxDirID)) != noErr) {
				sprintf(gError,LoadErrorStr(ERR_PNF,FALSE),gDBInfo[i].DBName);
				ErrorMsg(0);
				*gPrefs.inxDirSpec[i].volName = EOS;	/* reset name on error */
			}
		}
	
		/* get dirID of default index directory */
		if(*gPrefs.inxDirSpec[i].volName == EOS) {
			pstrcpy(name,volName);
			pstrcat(name,"\p:");
			pstrcat(name,gDBInfo[i].InxDName);
			if(!(gDBInfo[i].InxDirID  = GetDirIDFromPath(name,vRefNum,volName))) {
				gDBInfo[i].ndiv = 0;
				continue;
			}
			else
				gDBInfo[i].InxVRefNum = vRefNum;
		}
	
		if(!ProcessDBFiles(i)) {
			gDBInfo[i].ndiv = 0;
			continue;
		}
		
		foundOne = TRUE;
	}
		
	DisposDialog(myDialog);
	
	return(foundOne);
}


/**************************************
*	Get a directory ID from a full path specification
*	Returns directory ID, or 0 if an error occurred
*/

static long GetDirIDFromPath(StringPtr longName,short vRefNum,StringPtr volName)
{
	CInfoPBRec	myPB;
	OSErr			err;

	/* Get DirID */
	myPB.dirInfo.ioNamePtr=longName;
	myPB.dirInfo.ioVRefNum=vRefNum;
	myPB.dirInfo.ioFDirIndex=0;
	myPB.dirInfo.ioDrDirID=0;
	if((err = PBGetCatInfo(&myPB,FALSE)) != noErr) {
		return(0);
	}
	
	return(myPB.dirInfo.ioDrDirID);
}

/**************************************
*	Check validity of a DirSpec
*	Returns OSErr
*/

static OSErr CheckDir(DirSpec *dirSpec, short *vRefNum, long *dirID)
{
	CInfoPBRec	myPB;
	short			vrn;
	OSErr			err;
	
	if(!GetVRefNumFromName(dirSpec->volName,&vrn))
		return(fnfErr);
			
	
	/* Get DirID */
	myPB.dirInfo.ioNamePtr = NULL;
	myPB.dirInfo.ioVRefNum = vrn;
	myPB.dirInfo.ioFDirIndex = -1;
	myPB.dirInfo.ioDrDirID = dirSpec->dirID;
	
	if((err=PBGetCatInfo(&myPB,FALSE)) == noErr) {
		*vRefNum = vrn;
		*dirID = dirSpec->dirID;
	}
	
	return(err);
}

/**************************************
*	Analyse index files on CD-ROM
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

static Boolean ProcessDBFiles(short dbcode)
{
	short				fd;
	short				i;
	Header			hdr;
	DivisionRec		divRec;
	long				size;
	unsigned long	secs;
	DateTimeRec		date;
	Boolean			ret;
	SignedByte		oldHState;
	u_long			temp;
	u_short			recsize;
	short				rel;
	FSSpec			theFSS;
	
	/* Process division information file */	
	MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.divFName,gDBInfo[dbcode].InxDirID,&theFSS);
	if( OpenMacFileReadOnly(&theFSS,&fd, TRUE) != noErr )
		return(FALSE);
	
	/* check header */
	size=sizeof(Header);
	if ( ReadMacFile(fd,&size,&hdr,theFSS.name, TRUE) ) {
		FSClose(fd);
		return(FALSE);
	}

	/* store db name and release number */
	strncpy(gDBInfo[dbcode].DBName,hdr.db_name,DBNAMELEN);
	gDBInfo[dbcode].DBName[DBNAMELEN]=EOS;
	rtrim(gDBInfo[dbcode].DBName);

	strncpy(gDBInfo[dbcode].DBRelNum,hdr.db_relnum,DBRELNUMLEN);
	gDBInfo[dbcode].DBRelNum[DBRELNUMLEN]=EOS;
	rtrim(gDBInfo[dbcode].DBRelNum);
	
	/* check for compatibility of software with CD-ROM
	if(dbcode == DB_EMBL) {
		rel=atoi( gDBInfo[dbcode].DBRelNum );
		if (rel < 29) {
			sprintf(gError,LoadErrorStr(ERR_TOOOLD,FALSE));
			return(ErrorMsg(0));
		}
	}
	
	/* get release date and convert it to a string */
	date.year=1900+hdr.db_reldate.year;
	date.month=hdr.db_reldate.month;
	date.day=hdr.db_reldate.day;
	date.hour=date.minute=date.second=0;
	Date2Secs(&date,&secs);
	IUDateString(secs,shortDate,(StringPtr)gDBInfo[dbcode].DBRelDate);
	p2cstr((StringPtr)gDBInfo[dbcode].DBRelDate);
	
	/* check record size */
	if(ConvertShort(&hdr.record_size) != sizeof(DivisionRec)){
		FSClose(fd);
		sprintf(gError,LoadErrorStr(ERR_INVRECSIZE,FALSE),p2cstr(theFSS.name));
		c2pstr((char *)theFSS.name);
		return(ErrorMsg(0));
	}
	
	/* throw away old division information, if any */
	if(gDBInfo[dbcode].gDivNames != NULL)
		DisposHandle((Handle)gDBInfo[dbcode].gDivNames);
		
	/* allocate memory for division information */ 
	gDBInfo[dbcode].ndiv = (short)ConvertLong(&hdr.nrecords);
	size= (sizeof(Str255) * (gDBInfo[dbcode].ndiv+1));
	if( !(gDBInfo[dbcode].gDivNames = 
		(CStringDivHdl)NewHandleClear((Size)sizeof(CStringDiv) * (gDBInfo[dbcode].ndiv+1))) ) {
		FSClose(fd);
		return(ErrorMsg(ERR_MEMORY));
	}
	
	RotateWaitCursor();
	
	/* read data file names and division codes*/
	oldHState = LockHandleHigh((Handle)gDBInfo[dbcode].gDivNames);
	for(i=1;i<=gDBInfo[dbcode].ndiv;++i) {
		size=sizeof(DivisionRec);
		if( ReadMacFile(fd,&size,&divRec,theFSS.name,TRUE) ) {
			FSClose(fd);
			DisposHandle((Handle)gDBInfo[dbcode].gDivNames);
			gDBInfo[dbcode].gDivNames = NULL;
			return(FALSE);
		}
		if(ConvertShort(&divRec.div_code) != (u_short)i) {
			FSClose(fd);
			DisposHandle((Handle)gDBInfo[dbcode].gDivNames);
			gDBInfo[dbcode].gDivNames = NULL;
			sprintf(gError,LoadErrorStr(ERR_DIVCODEORDER,FALSE),p2cstr(theFSS.name));
			c2pstr((char *)theFSS.name);
			return(ErrorMsg(0));
		}
		else {
			strncpy((char *)(*gDBInfo[dbcode].gDivNames+i),divRec.filename,
						DIVNAMELEN);
			((char *)(*gDBInfo[dbcode].gDivNames+i))[DIVNAMELEN]=EOS;
			rtrim((char *)(*gDBInfo[dbcode].gDivNames+i));
			strcat((char *)(*gDBInfo[dbcode].gDivNames+i),";1");
			c2pstr((char *)(*gDBInfo[dbcode].gDivNames+i));
		}
	}
	HSetState((Handle)gDBInfo[dbcode].gDivNames,oldHState);
	FSClose(fd);
	
	RotateWaitCursor();

	/* Check short description file */
	MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.briefIdxFName,gDBInfo[dbcode].InxDirID,
				&theFSS);
	if((ret = (OpenMacFileReadOnly(&theFSS, &fd, TRUE) == noErr))  == TRUE) {
		ret = CheckHeader(theFSS.name,fd,dbcode,0L,&temp,&recsize);
		FSClose(fd);
	}
	if(ret)
		gDBInfo[dbcode].brief_recsize = recsize;	/* we assume entry name len is same as for
																	entryname index */
	else {
		return(FALSE);
	}

	RotateWaitCursor();

	/* Check entry name index */
	MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.enameIdxFName,gDBInfo[dbcode].InxDirID,
			&theFSS);
	if((ret = (OpenMacFileReadOnly(&theFSS,&fd,TRUE) == noErr)) == TRUE) {
		ret = CheckHeader(theFSS.name,fd,dbcode,0L,&gDBInfo[dbcode].ename_nrec, &recsize);
		FSClose(fd);
	}
	if(ret) {
		gDBInfo[dbcode].ename_recsize = recsize;
		gDBInfo[dbcode].ename_len = recsize - 2*sizeof(u_long) - sizeof(u_short);
	}
	else {
		return(FALSE);
	}
	
	/* Compare nrec of entryname and shortdir indexes */
	if (gDBInfo[dbcode].ename_nrec != temp)
		return(ErrorMsg(ERR_WRONGNREC));
	
	RotateWaitCursor();

	/* Check accession number hit file */
	MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.inxFNames[FLD_ACCNUM].hitFName,
				gDBInfo[dbcode].InxDirID,
				&theFSS);
	if ((ret = (OpenMacFileReadOnly(&theFSS,&fd,FALSE) == noErr)) == TRUE) {
		ret = CheckHeader(theFSS.name,fd,dbcode,sizeof(u_long),
								&gDBInfo[dbcode].inxInfo[FLD_ACCNUM].hit_nrec,
								&recsize);
		FSClose(fd);
	}
	if(ret == FALSE)
		gDBInfo[dbcode].inxInfo[FLD_ACCNUM].trg_recsize=0;
	else {
		/* Check accession number target list */
		MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.inxFNames[FLD_ACCNUM].trgFName,
				gDBInfo[dbcode].InxDirID,
				&theFSS);
		if((ret = (OpenMacFileReadOnly(&theFSS,&fd,FALSE) == noErr)) == TRUE) {
			ret = CheckHeader(theFSS.name,fd,dbcode,0L,
									&gDBInfo[dbcode].inxInfo[FLD_ACCNUM].trg_nrec,&recsize);
			FSClose(fd);
		}
		if(ret == FALSE)
			gDBInfo[dbcode].inxInfo[FLD_ACCNUM].trg_recsize=0;
		else
			gDBInfo[dbcode].inxInfo[FLD_ACCNUM].trg_recsize=recsize;
	}
	
	/* Load entryname preindex */
	if(gDBInfo[dbcode].ename_pnxHdl != NULL) {
		DisposHandle(gDBInfo[dbcode].ename_pnxHdl);
		gDBInfo[dbcode].ename_pnxHdl = NULL;
	}
	MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.enamePnxFName,
			gDBInfo[dbcode].InxDirID,
			&theFSS);
	if ((ret = ( OpenMacFileReadOnly(&theFSS,&fd,FALSE) == noErr)) == TRUE) {
		ret = CheckHeader(theFSS.name,fd,dbcode,0L,
								&gDBInfo[dbcode].ename_pnxNrec, &recsize);
		if(ret == TRUE) {
			gDBInfo[dbcode].ename_pnxRecsize = recsize;
			gDBInfo[dbcode].ename_pnxHdl = LoadPreIndex(fd);
		}
		FSClose(fd);
	}

		
	for(i=FLD_KEYWORD; i < FLD_NUM; ++i) {
		RotateWaitCursor();
		
		/* Check hit file */
		MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.inxFNames[i].hitFName,
				gDBInfo[dbcode].InxDirID,
				&theFSS);
		if ((ret = ( OpenMacFileReadOnly(&theFSS,&fd,FALSE) == noErr)) == TRUE) {
			ret = CheckHeader(theFSS.name,fd,dbcode,sizeof(u_long),
									&gDBInfo[dbcode].inxInfo[i].hit_nrec, &recsize);
			FSClose(fd);
		}
		if(ret == FALSE) /* no error, we simply disable free text queries
									(might be an old CD-ROM */
			gDBInfo[dbcode].inxInfo[i].trg_recsize=0;
		else {
			/* Check target file */
			MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.inxFNames[i].trgFName,
				gDBInfo[dbcode].InxDirID,
				&theFSS);
			if((ret = (OpenMacFileReadOnly(&theFSS,&fd,FALSE) == noErr))  == TRUE) {
				ret = CheckHeader(theFSS.name,fd,dbcode,0L,&gDBInfo[dbcode].inxInfo[i].trg_nrec,
						&recsize);
				FSClose(fd);
			}
			if(ret == FALSE) 		/* no error, we simply disable free text queries
											(might be an old CD-ROM */
				gDBInfo[dbcode].inxInfo[i].trg_recsize=0;
			else
				gDBInfo[dbcode].inxInfo[i].trg_recsize=recsize;
		}
		
		if(ret != FALSE) {
			/* Load preindex */
			if(gDBInfo[dbcode].inxInfo[i].pnxHdl != NULL) {
				DisposHandle(gDBInfo[dbcode].inxInfo[i].pnxHdl);
				gDBInfo[dbcode].inxInfo[i].pnxHdl = NULL;
			}
			MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.inxFNames[i].pnxFName,
					gDBInfo[dbcode].InxDirID,
					&theFSS);
			if ((ret = (OpenMacFileReadOnly(&theFSS,&fd,FALSE) == noErr)) == TRUE) {
				ret = CheckHeader(theFSS.name,fd,dbcode,0L,
										&gDBInfo[dbcode].inxInfo[i].pnx_nrec, &recsize);
				if(ret == TRUE) {
					gDBInfo[dbcode].inxInfo[i].pnx_recsize = recsize;
					gDBInfo[dbcode].inxInfo[i].pnxHdl = LoadPreIndex(fd);
				}
				FSClose(fd);
			}
		}
	}

	return(TRUE);
}


/**************************************
*	Check index file header
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*	Side-effect:	*nrec contains number of records, *recsize the size of a record
*/

static Boolean CheckHeader(StringPtr fName, short fd, short dbcode,
									long rec_size, u_long *nrec, u_short *recsize)
{
	long		size;
	Header	hdr;
	char		dbname[DBNAMELEN+1];
	char		dbrelnum[DBRELNUMLEN+1];
	
	*nrec=0L;
	
	/* rewind file */
	if(SetFPos(fd,fsFromStart,0L) != noErr)
		return(FALSE);
	
	/* read header */
	size=sizeof(Header);
	if ( ReadMacFile(fd,&size,&hdr,fName,TRUE) )
		return(FALSE);

	/* check record size; for generic record structures we pass rec_size of 0 and
		do not check */
	if(rec_size != 0L) {
		if(ConvertShort(&hdr.record_size) != rec_size){
			sprintf(gError,LoadErrorStr(ERR_INVRECSIZE,FALSE),fName);
			return(ErrorMsg(0));
		}
	}

	/* check dbcode name and version (we don't check the date) */
	strncpy(dbname,hdr.db_name,DBNAMELEN);
	dbname[DBNAMELEN]=EOS;
	rtrim(dbname);
	
	strncpy(dbrelnum,hdr.db_relnum,DBRELNUMLEN);
	dbrelnum[DBRELNUMLEN]=EOS;
	rtrim(dbrelnum);
	
	if(strcmp(dbname,gDBInfo[dbcode].DBName) ||
		strcmp(dbrelnum,gDBInfo[dbcode].DBRelNum)) {
		sprintf(gError,LoadErrorStr(ERR_HEADINC,FALSE),fName);
		return(ErrorMsg(0));
	}	

	*nrec=ConvertLong(&hdr.nrecords);
	*recsize = ConvertShort(&hdr.record_size);
	return(TRUE);
}

static short InitUserBreak(DialogPtr myDialog)
{
	EventRecord theEvent;
	DialogPtr	whichDialog;
	short			itemHit;
	WindowPtr	whichWindow;
	static long	ticks = 0;
	
	if(gInBackground || TickCount() - ticks > 30) {
		ticks = TickCount();
		if(WaitNextEvent(everyEvent,&theEvent,
								gInBackground ? 25 : 0,NULL)) {
			if(theEvent.what == osEvt) /* see "Zen and..." (bug in IsDialogEvent) */
				DoEvent(theEvent);
			else if(theEvent.what == keyDown || theEvent.what == autoKey) {
				if ( CmdPeriod(&theEvent)  ||
					( (theEvent.message & keyCodeMask) >> 8 == ESCAPE))
					return(Cancel);
			}
			else {
				if(IsDialogEvent(&theEvent)) {
					if(DialogSelect(&theEvent,&whichDialog,&itemHit))
						if(whichDialog)
							if(itemHit == OK || itemHit == Cancel)
								return(itemHit);
				}
				else {
					switch(theEvent.what) {
						case updateEvt:
						case activateEvt:
							DoEvent(theEvent);
							break;
						case mouseDown:
							switch (FindWindow( theEvent.where, &whichWindow )) {
								case inSysWindow:
									SystemClick( &theEvent, whichWindow );
									break;
								case inDrag:
									if(whichWindow == myDialog)
										DragWindow(whichWindow,theEvent.where,&gDragRect);
									break;
									case inMenuBar:
								MenuSelect(theEvent.where);
									HiliteMenu(0);
									break;
								default:
									SysBeep(10);
									break;
							}
							break;
						default:
							break;
					}
				}
			}
		}
	}
	
	return(0);
}