/*
*********************************************************************
*	
*	CheckApp.c
*	If EMBL-Search is started by opening an EMBL-Search document from
*	the Finder
*	
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*
*********************************************************************
*	
*/ 

/*
****************************** Include files *************************
*/

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "checkapp.h"

#include "environment.h"
#include "print.h"
#include "query.h"
#include "results.h"
#include "util.h"
#include "window.h"

/*
******************************* Prototypes ***************************
*/

/*
******************************** Global variables ******************
*/


/**************************************
*	Handle information passed by Finder of Systems < 7.0
*	Return value:	none
*/

void CheckAppFiles()
{
	short		message,count;
	short		index;
	AppFile	theFile;
	WDPtr		dummy;
	short		vRefNum;
	long		parID;
	long		procID;
	FSSpec	theFSS;

	if(gSystem.bHasAppleEvents)					/* Apple Events do everything for us */
		return;
		
	CountAppFiles(&message,&count);	/* check startup information */
	if(message == appPrint)	{			/* print files */
		for(index=1;index<=count;index++) {	/* print files, one by one */
			GetAppFiles(index,&theFile);
			GetWDInfo(theFile.vRefNum, &vRefNum,&parID, &procID);
			MakeFSS(vRefNum,theFile.fName,parID,&theFSS);
			if(!DoDocPrint(theFile.fType,&theFSS))
				break;
			ClrAppFiles(index);
		}
  	}
	else {
		for(index=1;index<=count;index++) {	/* open files, one by one */
			GetAppFiles(index,&theFile);
			GetWDInfo(theFile.vRefNum, &vRefNum,&parID, &procID);
			MakeFSS(vRefNum,theFile.fName,parID,&theFSS);
			if(!DoDocOpen(theFile.fType,&theFSS,&dummy))
				break;
			ClrAppFiles(index);
   	}
   }
	/* should we ExitToShell if application was started from the Finder to
		print a document!? Couldn't find any clear interface guidelines */
}


/**************************************
*	Open document
*	Return value:	TRUE, if sucessful
*						FALSE, if an error occurred
*	By side-effect a WDPtr in wdpPtr, if a window was opened
*/

Boolean DoDocOpen(OSType fType,FSSpecPtr theFSS, WDPtr *wdpPtr)
{
	char		fullPath[256];
	short		w;
	Boolean	ret = TRUE;

	*wdpPtr = NULL;
	if( (w=GetFreeWindow()) == -1) {		/* find a free window */
		return(ErrorMsg(ERR_MAXWIN));
	}

  	StartWaitCursor();
	switch(fType) {
		case kQryFileType:
   		if ( (ret=LoadQuery(w,theFSS)) )
   			*wdpPtr = &gWindows[w];
	   	break;
		case kPrefFileType:		/* no special action required */
			break;
		case kResFileType:
   		if( (ret=Load1Results(w,theFSS)) )
   			*wdpPtr = &gWindows[w];
	   	break;
	}
	InitCursor();
	
	return(ret);
}

/**************************************
*	Print document
*	Return value:	TRUE, if sucessful
*						FALSE, if an error occurred
*/

Boolean DoDocPrint(OSType fType, FSSpecPtr theFSS)
{
	WDPtr wdp = NULL;
	
	if(fType == kQryFileType)
		return(TRUE);

	if(DoDocOpen(fType,theFSS,&wdp)) {	/* open window */
		if(wdp) {
			PrintIt(wdp);
			CloseMyWindow(wdp,TRUE);
		}
	
		/* See above: should we set gQuitApplication to TRUE if application was
		started from the Finder to print a document */
		return(TRUE);
	}
	else return(FALSE);
}