/*
*********************************************************************
*	
*	Click.c
*	Mouseclicks in window content region
*
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/ 

#include <ctype.h>
#include <string.h>
#include <stdio.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "click.h"

#include "entry.h"
#include "events.h"
#include "hitstorage.h"
#include "main.h"
#include "pstr.h"
#include "results.h"
#include "util.h"
#include "xref.h"

/*
******************************* Prototypes ****************************
*/

static void ResDragScroll(WindowPtr wPtr, ControlHandle theControl, short scrollAmt);
static void GetClickPos(WDPtr wdp, Rect *viewRectPtr, Point *where,
								short *row, short *col);
static Boolean IsResDoubleClick(EventRecord *eventPtr, Rect *viewRectPtr, WDPtr wdp);


/*
********************************* Globals *****************************
*/


/**************************************
*	Handle clicks in content region of entry window
*/

void DoEntryContentClick(EventRecord *eventPtr, Rect *viewRectPtr, WDPtr wdp)
{
	short				xrefCode, keyType;
	char				xrefPrimaryID[PROSITEENTRYNAMELEN+1];
	EntryRecHdl		entryRecHdl;
	GrafPtr			savePort;
	Point				where;
	short				bufPos, col;
	CString80Hdl	bufHdl;
	CString80		line;
	short				dbcode;
	char				*pos;
	register short i;
	
	if( wdp == NULL || ((WindowPeek)wdp)->windowKind != entryW )
		return;
		
	if((entryRecHdl = (EntryRecHdl)(wdp->userHandle)) == NULL)
		return;

	if( IsResDoubleClick(eventPtr, viewRectPtr, wdp) ) {
		bufHdl = (**entryRecHdl).lineBufHdl;
		dbcode = (**entryRecHdl).dbcode;
	
		savePort = ChangePort((GrafPtr)wdp);
		where=eventPtr->where;
		GlobalToLocal(&where);
		
		/* Double click ? */
	
		/* find current row */
		GetClickPos(wdp,viewRectPtr,&where, &bufPos,&col);
		bufPos -= (**entryRecHdl).buftop;
		
		if(bufPos >= (**entryRecHdl).buftop) {
			pstrcpy((StringPtr)line,(StringPtr)(*bufHdl)[bufPos]);
			p2cstr((StringPtr)line);
			
			if( linetype(line,"DR") ) {
				switch( dbcode ) {
					/* SWISS-PROT: Pointers to EMBL and Prosite.dat */
					case DB_SWISS:
						if ( gDBInfo[DB_EMBL].ndiv > 0 && (pos = strstr(line,"EMBL")) != NULL ) {
							while(*pos++ != ';') ;	/* skip db name */
							while(isspace(*pos)) ++pos;	/* skip white space */
							for( i=0; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
								xrefPrimaryID[i] = *pos;
							xrefPrimaryID[i] = EOS;
							
							OpenXRef(DB_EMBL,xrefPrimaryID,FLD_ACCNUM);
						}
						else if ( gDBInfo[DB_PROSITEDAT].ndiv > 0 && (pos = strstr(line,"PROSITE")) != NULL ) {
							while(*pos++ != ';') ;	/* skip db name */
							while(isspace(*pos)) ++pos;	/* skip white space */
							for( i=0; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
								xrefPrimaryID[i] = *pos;
							xrefPrimaryID[i] = EOS;
			
							OpenXRef(DB_PROSITEDAT,xrefPrimaryID,FLD_ACCNUM);
						}
						break;
						
					/* EMBL: Pointers to SWISS-PROT */
					case DB_EMBL:
						if ( gDBInfo[DB_SWISS].ndiv > 0 && (pos = strstr(line,"SWISS-PROT")) != NULL ) {				
							while(*pos++ != ';') ;	/* skip db name */
							while(isspace(*pos)) ++pos;	/* skip white space */
							for( i=0; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
								xrefPrimaryID[i] = *pos;
							xrefPrimaryID[i] = EOS;
							
							OpenXRef(DB_SWISS,xrefPrimaryID,FLD_ACCNUM);
						}
						else if ( gDBInfo[DB_EPD].ndiv > 0 && (pos = strstr(line,"EPD")) != NULL ) {
							while(*pos++ != ';') ;	/* skip db name */
							while(isspace(*pos)) ++pos;	/* skip white space */
							for( i=0; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
								xrefPrimaryID[i] = *pos;
							xrefPrimaryID[i] = EOS;
							
							OpenXRef(DB_EPD,xrefPrimaryID,FLD_ACCNUM);
						}
						break;
						
					/* Prosite.dat: Pointers to SWISS-PROT. All DR lines are SWISS-PROT xrefs */
					case DB_PROSITEDAT:
						/* multiple xrefs per line: we start after the three blanks and
							extract xref for xref, then we simply check whether the click
							occurred before the end of this xref */
						
						if(gDBInfo[DB_SWISS].ndiv > 0) {
							pos = line + 5;
							while(*pos && *pos != '\r') {
								while(isspace(*pos)) ++pos;	/* skip white space */
								for( i=0; *pos != ',' && i < PROSITEENTRYNAMELEN; ++pos,++i )
									xrefPrimaryID[i] = *pos;
								xrefPrimaryID[i] = EOS;
								while(*pos++ != ';') ; /* go to end of xref, skip T, P */
								
								if(line + col <= pos) {
									OpenXRef(DB_SWISS,xrefPrimaryID,FLD_ACCNUM);
									break;
								}
							}
						}
						break;
						
					/* Enzyme: Pointers to SWISS-PROT. All DR lines are SWISS-PROT xrefs */
					case DB_ENZYME:
						if(gDBInfo[DB_SWISS].ndiv > 0) {
							pos = line + 5;
							while(*pos && *pos != '\r') {
								while(isspace(*pos)) ++pos;	/* skip white space */
								for( i=0; *pos != ',' && i < PROSITEENTRYNAMELEN; ++pos,++i )
									xrefPrimaryID[i] = *pos;
								xrefPrimaryID[i] = EOS;
								while(*pos++ != ';') ; /* go to end of xref */

								if(line + col <= pos) {
									OpenXRef(DB_SWISS,xrefPrimaryID,FLD_ACCNUM);
									break;
								}
							}
						}
						break;
					default:
						SysBeep(10);
						break;
				}
			}
			else if ( dbcode == DB_PROSITEDAT && linetype(line,"DO")) {
				if(gDBInfo[DB_PROSITEDOC].ndiv > 0) {
					for( i=0, pos = line + 5; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
						xrefPrimaryID[i] = *pos;
					xrefPrimaryID[i] = EOS;
			
					OpenXRef(DB_PROSITEDOC,xrefPrimaryID,FLD_ENAME);
				}
				else SysBeep(10);
			}
			else if ( dbcode == DB_PROSITEDOC && linetype(line,"{PS")) {
				if(gDBInfo[DB_PROSITEDAT].ndiv > 0) {
					for( i=0, pos = line + 1; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
						xrefPrimaryID[i] = *pos;
					xrefPrimaryID[i] = EOS;
			
					OpenXRef(DB_PROSITEDAT,xrefPrimaryID,FLD_ACCNUM);
				}
				else SysBeep(10);
			}
			else if( dbcode == DB_EPD && (linetype(line,"FP") || linetype(line,"F-"))) {
				if(gDBInfo[DB_EMBL].ndiv > 0) {
					for( i=0, pos = line + 34; !isspace(*pos) && i < SEQENTRYNAMELEN; ++pos,++i )
						xrefPrimaryID[i] = *pos;
					xrefPrimaryID[i] = EOS;
			
					OpenXRef(DB_EMBL,xrefPrimaryID,FLD_ENAME);
				}
				else SysBeep(10);
			}
			else SysBeep(10);	
		}
		else SysBeep(10);
	}
}


/**************************************
*	Handle clicks in content region of result window
*/

void DoResContentClick(EventRecord *eventPtr,Rect *viewRectPtr,WDPtr wdp)
{
	register 	short	i;
	short			bufPos,newBufPos,anchor,dummy;
	short			firstSelectRow,lastSelectRow,sel;
	ResultHdl	resHdl;
	HitlistHdl	hlHdl;
	short 		oldStatus,newStatus;
	Point			where;
	SignedByte	oldRState;
	short			screenTop,visLines;
	GrafPtr		savePort;

	if( wdp == NULL || ((WindowPeek)wdp)->windowKind != resW )
		return;
		
	savePort = ChangePort((GrafPtr)wdp);
	where=eventPtr->where;
	GlobalToLocal(&where);
	
	resHdl = (ResultHdl)(wdp->userHandle);
	hlHdl = (**resHdl).hlHdl;

	visLines = (viewRectPtr->bottom - viewRectPtr->top) / fMonaco.height;
	
	/* find current row */
	GetClickPos(wdp,viewRectPtr,&where, &bufPos, &dummy);
	if( bufPos >= (**resHdl).nhits ) {
	
		/* deselect all */
		oldRState = MyHLock((Handle)resHdl);
		for( i = 0; i < (**resHdl).nhits; i++)
			SelectDraw(wdp,i,FALSE);
		HSetState((Handle)resHdl,oldRState);
		
		/* update header */
		DrawResHeader(wdp, resHdl);
		SetPort(savePort);
		return;
	}

	/* Double click ? */
	if( IsResDoubleClick(eventPtr, viewRectPtr, wdp) ) {
		OpenEntries(wdp);
		SetPort(savePort);
		return;
	}

	/* Handle clicks differently depending on modifier keys pressed */
	
	/* no modifiers */
	if( !(eventPtr->modifiers & shiftKey) &&
		!(eventPtr->modifiers & cmdKey) ) { 
		
		/* deselect all */
		oldRState = MyHLock((Handle)resHdl);
		for( i = 0; i < (**resHdl).nhits; i++)
			SelectDraw(wdp,i,FALSE);
		HSetState((Handle)resHdl,oldRState);
			
		/* select and update header */
		SelectDraw(wdp, bufPos, TRUE);
		DrawResHeader(wdp, resHdl);
	
/*		while( StillDown() ) {*/
/*			GetMouse(&where);	*//* GetMouse returns local coordinates *//**/
/*			if (PtInRect(where,viewRectPtr) ) {*/
/*				newBufPos = GetClickRow(wdp,viewRectPtr,&where);*/
/*				if ( newBufPos != bufPos ) {	*//* if moved *//**/
/*					*//* deselect old row and select new row *//**/
/*					if( bufPos < (**resHdl).nhits && newBufPos < (**resHdl).nhits ) {*/
/*						SelectDraw(wdp,bufPos, FALSE);*/
/*						SelectDraw(wdp,newBufPos, TRUE);*/
/*						DrawResHeader(wdp, resHdl);*/
/*					}*/
/*					bufPos = newBufPos;*/
/*				}*/
/*			}*/
/*		}*/
	}
	
	/* command key */
	else if ( eventPtr->modifiers & cmdKey ) {
		/* Get status of selected cell */
		oldStatus = GetSelectState(hlHdl,bufPos);
		newStatus = !oldStatus;
		
		/* select and update header */
		SelectDraw(wdp, bufPos, newStatus);
		DrawResHeader(wdp, resHdl);
		
		while( StillDown() ) {
			screenTop = GetCtlValue(wdp->vScroll);	/* first hit on screen */
			GetMouse(&where);	/* GetMouse returns local coordinates */
			GetClickPos(wdp,viewRectPtr,&where, &newBufPos,&dummy);
			if(newBufPos < screenTop)
				newBufPos = screenTop;
			else if(newBufPos >= screenTop + visLines)
				newBufPos = screenTop + visLines -1;
				
			if(newBufPos >= (**resHdl).nhits)
				newBufPos = (**resHdl).nhits - 1;
			else if(newBufPos < 0)
				newBufPos = 0;
				
			/* select new row(s) */
			if(newBufPos < bufPos) {
				for(i = newBufPos; i<= bufPos; ++i) {
					SelectDraw(wdp,i, newStatus);
				}
			
				DrawResHeader(wdp, resHdl);
				bufPos = newBufPos;
			}
			if(newBufPos > bufPos) {
				for(i = bufPos; i<= newBufPos; ++i) {
					SelectDraw(wdp,i, newStatus);
				}
			
				DrawResHeader(wdp, resHdl);
				bufPos = newBufPos;
			}
			
			if(newBufPos < (**resHdl).nhits - 1 && where.v > viewRectPtr->bottom) {
				SelectDraw(wdp,bufPos = newBufPos + 1,newStatus);
				ResDragScroll((WindowPtr)wdp,(ControlHandle)wdp->vScroll,1);
				DrawResHeader(wdp, resHdl);
			}
			else if(newBufPos > 0 && where.v < viewRectPtr->top) {
				SelectDraw(wdp,bufPos = newBufPos - 1,newStatus);
				ResDragScroll((WindowPtr)wdp,(ControlHandle)wdp->vScroll,-1);
				DrawResHeader(wdp, resHdl);
			}
		}
	}
	
	/* shift key */
	else if ( eventPtr->modifiers & shiftKey ) {
		/* find first and last selected row */
		firstSelectRow = lastSelectRow = -1;
		oldRState = MyHLock((Handle)resHdl);
		for( i = sel = 0; i < (**resHdl).nhits && sel < (**resHdl).nsel; ++i)
			if( GetSelectState(hlHdl,i) ) {
				sel++;
				if (sel == 1)
					firstSelectRow = i;
				if (sel == (**resHdl).nsel)
					lastSelectRow = i;
			}
		HSetState((Handle)resHdl,oldRState);
		
		if( firstSelectRow == -1)
			firstSelectRow = bufPos;	
		if( lastSelectRow == -1) {
			if(bufPos > firstSelectRow)
				lastSelectRow = bufPos;
			else
				lastSelectRow = firstSelectRow;
		}
			
		if( bufPos >= firstSelectRow ) {
			for(i = firstSelectRow; i <= bufPos; ++i)
				SelectDraw(wdp,i,TRUE);
			for(i = bufPos+1; i<=lastSelectRow; ++i)
				SelectDraw(wdp,i,FALSE);
			anchor = firstSelectRow;
		}
		else {
			for(i = bufPos; i <= lastSelectRow; ++i)
				SelectDraw(wdp,i,TRUE);
			anchor = lastSelectRow;
		}
		DrawResHeader(wdp, resHdl);

	/* I wonder whether there is some bug in StillDown() ?!? If I hold the mouse button
		down and move up and down for a while, sometimes this procedure breaks out of the
		while (StillDown()) loop...
	*/
		while( StillDown() ) {
			screenTop = GetCtlValue(wdp->vScroll);	/* first hit on screen */
			GetMouse(&where);	/* GetMouse returns local coordinates */
			GetClickPos(wdp,viewRectPtr,&where, &newBufPos,&dummy);
			if(newBufPos < screenTop)
				newBufPos = screenTop;
			else if(newBufPos >= screenTop + visLines)
				newBufPos = screenTop + visLines -1;
				
			if(newBufPos >= (**resHdl).nhits)
				newBufPos = (**resHdl).nhits - 1;
			else if(newBufPos < 0)
				newBufPos = 0;
				
			if( newBufPos < bufPos ) {
				for(i = bufPos; i>= newBufPos ; --i) {
					if( i <= anchor || i == newBufPos )
						SelectDraw(wdp,i,TRUE);
					else
						SelectDraw(wdp,i, FALSE);
				}

				DrawResHeader(wdp, resHdl);
				bufPos = newBufPos;
			}
			
			if( newBufPos > bufPos ) {
				for(i = bufPos; i<= newBufPos; ++i) {
					if( i >= anchor || i == newBufPos )
						SelectDraw(wdp,i,TRUE);
					else
						SelectDraw(wdp,i, FALSE);
				}
				
				DrawResHeader(wdp, resHdl);
				bufPos = newBufPos;
			}
			
			if(newBufPos < (**resHdl).nhits - 1 && where.v > viewRectPtr->bottom) {
				SelectDraw(wdp,bufPos = newBufPos + 1,TRUE);
				ResDragScroll((WindowPtr)wdp,(ControlHandle)wdp->vScroll,1);
				DrawResHeader(wdp, resHdl);
			}
			else if(newBufPos > 0 && where.v < viewRectPtr->top) {
				SelectDraw(wdp,bufPos = newBufPos - 1,TRUE);
				ResDragScroll((WindowPtr)wdp,(ControlHandle)wdp->vScroll,-1);
				DrawResHeader(wdp, resHdl);
			}
		}
	}
	SetPort(savePort);
}

/**************************************
*	Update result window when user drag-scrolls
*/

static void ResDragScroll(WindowPtr wPtr, ControlHandle theControl, short scrollAmt)
{
	short			oldValue,newValue;
	short			max=GetCtlMax(theControl);
	
	oldValue=GetCtlValue(theControl);
	newValue=oldValue+scrollAmt;
	/* limit check */
	if(newValue < 0) newValue=0;
	if(newValue > max) newValue=max;
			
	SetCtlValue( theControl, newValue );
	AdjustResText(wPtr,oldValue,newValue,vertBar);
}


/**************************************
*	Find line in result buffer corresponding to screen position where click occurred
*	Return value:	line offset in result buffer
*/

static void GetClickPos(WDPtr wdp, Rect *viewRectPtr,Point *where,
								short *row, short *col)
{
	short screenRow,hitPos;
	
	*row = *col = 0;
	
	hitPos = GetCtlValue(wdp->vScroll);	/* first hit on screen */
	screenRow = (where->v - viewRectPtr->top) / fMonaco.height;
	*row = hitPos + screenRow;
	
	*col = (where->h - viewRectPtr->left) / fMonaco.finfo.widMax;
}


static Boolean IsResDoubleClick(EventRecord *eventPtr, Rect *viewRectPtr, WDPtr wdp)
{
	Point where1,where2;
	short row1,row2,col1,col2;
	GrafPtr savePort;
	
	if((WindowPtr)wdp != gLastClick.wPtr)
		return(FALSE);
		
	if(eventPtr->when-gLastClick.when > GetDblTime())
		return(FALSE);

	savePort = ChangePort((GrafPtr)wdp);
	where1 = eventPtr->where;
	GlobalToLocal(&where1);
	where2 = gLastClick.where;
	GlobalToLocal(&where2);
	SetPort(savePort);
	
	GetClickPos(wdp,viewRectPtr,&where1, &row1,&col1);
	GetClickPos(wdp,viewRectPtr,&where2, &row2,&col2);
	
	return (row1 == row2);
}

