/*
*********************************************************************
*	
*	Entry.c
*	Handling of entry window
*
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/ 

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "entry.h"

#include "click.h"
#include "environment.h"
#include "events.h"
#include "export.h"
#include "main.h"
#include "pref.h"
#include "pstr.h"
#include "results.h"
#include "util.h"
#include "window.h"
#include "xref.h"

/*
******************************* Prototypes ***************************
*/

static Boolean OpenEntryWindow(short w, StringPtr title, EntryRecHdl entryRecHdl);
static Boolean NewEntryRec(EntryRecHdl *new);
static Boolean NewEntryLineBuf(CString80Hdl *new);
static Boolean InitEntryRec(EntryRecHdl *new, short dbcode);
static void DisposeEntryRec(EntryRecHdl entryRecHdl);
static Boolean FillEntryRec(ResultHdl resHdl, short bufPos, EntryRecHdl entryRecHdl);
static void DrawEntryWin1(WDPtr wdp, short updown);
static pascal void ScrollEntryProc(ControlHandle theControl, short theCode);
static void DrawStringItalic(StringPtr str);


/*
******************************** Global variables *****************
*/


/**************************************
*	Create and open a new entry window
*/

Boolean NewEntryWindow(ResultHdl resHdl,short bufPos)
{
	short				w;
	EntryRecHdl		entryRecHdl;
	CString80Hdl	descBufHdl;
	Str255			title;

	if( resHdl == NULL )
		return(FALSE);
		
	if( (descBufHdl = (**resHdl).descBufHdl) == NULL )
		return(FALSE);
		
	if( (w=GetFreeWindow()) == -1 )
		return(ErrorMsg(ERR_MAXWIN));

	if( !InitEntryRec(&entryRecHdl,(**resHdl).dbcode) )
		return(ErrorMsg(ERR_MEMORY));
	
	if( !FillEntryRec(resHdl,bufPos,entryRecHdl) ) {
		DisposeEntryRec(entryRecHdl);
		DisposHandle((Handle)entryRecHdl);
		return(FALSE);
	}
	else {
		GetEnameFromDesc(descBufHdl,bufPos-(**resHdl).buftop,(**entryRecHdl).dbcode,title);
		return(OpenEntryWindow(w,title,entryRecHdl));
	}
}


/**************************************
*	Open a new entry window
*/

static Boolean OpenEntryWindow(short w,StringPtr title,EntryRecHdl entryRecHdl)
{
	Rect			tempRect;
	WStateData	**stateHdl;
	WDPtr			wdp;
	WindowPtr	wPtr;
	long			maxHeight;
	short			nlines;
	
	tempRect = gEntryWinRect;
	AdjustPosition(w,&tempRect);
	nlines = (**entryRecHdl).nlines > MIN_LINES ? (**entryRecHdl).nlines : MIN_LINES;
	maxHeight = (long)nlines*(long)fMonaco.height + SBarWidth;
	if((long)(tempRect.bottom - tempRect.top) > maxHeight)
		tempRect.bottom = tempRect.top + (short)maxHeight;
	

	wdp = &gWindows[w];
	SetPort(NewWindow(wdp,&tempRect,"\p",FALSE,documentProc + 8,(void *)-1,TRUE,0L));

	wPtr = (WindowPtr)wdp;

	/* anchor entry record */
	wdp->userHandle = (Handle)entryRecHdl;
			
	SetWTitle(wPtr,title);
	TextFont(fMonaco.num);
	TextSize(9);

	((WindowPeek)wPtr)->windowKind = entryW;
	pstrcpy(wdp->theFSS.name,title);
	wdp->theFSS.vRefNum = 0;
	wdp->dirty = TRUE;
	wdp->inited = FALSE;
	wdp->inUse = TRUE;
	
	/* adjust zoom standard state */
	stateHdl = (WStateData **)((WindowPeek)wPtr)->dataHandle;
	AdjustWSize(entryW,&(**stateHdl).stdState,fMonaco.height,fMonaco.finfo.widMax);
	
	/* draw vertical scroll bar */
	tempRect = wPtr->portRect;
	tempRect.left = tempRect.right-SBarWidth;
	tempRect.right += 1;
	tempRect.top -= 1;
	tempRect.bottom -= (SBarWidth-1);
	wdp->vScroll = NewControl(wPtr,&tempRect,"\p",TRUE,0,0,10,scrollBarProc,0L);
	
	/* draw horizontal scroll bar */
	tempRect = wPtr->portRect;
	tempRect.top = tempRect.bottom-SBarWidth;
	tempRect.bottom += 1;
	tempRect.right -= (SBarWidth-1);
	tempRect.left -= 1;
	wdp->hScroll = NewControl(wPtr,&tempRect,"\p",TRUE,0,0,10,scrollBarProc,0L);
	
	/* Set scroll bars */
	SetVScroll(wdp);
	SetHScroll(wdp);
	
	AddWindowToMenu(title, wPtr);
	
	ShowWindow(wPtr);
	return(TRUE);
}

/**************************************
*	Transform entry name record by moving "value" to the end
*	Return values:
*		None
*		By side effect: Converted EnameRec
*/

void TransformEnameRec(EnameRec *enameRec, short ename_len)
{
	char		ename[255];
	Ptr		ptr = (Ptr)enameRec;
	u_long	annotation_offset;
	u_long	sequence_offset;
	u_short	div_code;
		
	strncpy(ename,ptr,ename_len);
	ename[ename_len] = EOS;
	ptr += ename_len;
	
	annotation_offset = *(u_long *)ptr;
	ptr += sizeof(u_long);
	sequence_offset = *(u_long *)ptr;
	ptr += sizeof(u_long);
	div_code = *(u_short *)ptr;
	
	enameRec->annotation_offset = annotation_offset;
	enameRec->sequence_offset = sequence_offset;
	enameRec->div_code = div_code;
	strncpy(enameRec->value,ename,ename_len);
	/* no '\0' copied !!! */
}

/**************************************
*	Allocates movable memory block for entry record
*	Return values:
*		TRUE if successful, FALSE on failure
*		By side effect: Handle to this block
*/

static Boolean NewEntryRec(EntryRecHdl *new)
{
	*new = (EntryRecHdl)NewHandleClear(sizeof(EntryRec));
	return( *new != NULL );
}


/**************************************
*	Allocates movable memory block for entry line buffer
*	Return values:
*		TRUE if successful, FALSE on failure
*		By side effect: Handle to this block
*/

static Boolean NewEntryLineBuf(CString80Hdl *new)
{
	*new = (CString80Hdl)NewHandleClear((Size)ENTRYBUFLINES * sizeof(CString80));
	return( *new != NULL );
}


/**************************************
*	Gets a new entry record, incl. line buffer, and initialises it
*	Return values:
*		TRUE if successful, FALSE on failure
*		By side effect: Handle to new entry record
*/

static Boolean InitEntryRec(EntryRecHdl *new, short dbcode)
{
	EntryRecPtr entryPtr;
	short		i;
	
	if( new == NULL )
		return(FALSE);
		
	/* Allocate memory for entry record */
	if ( !NewEntryRec(new) )
		return( FALSE );
	
	/* Get a pointer to it */
	LockHandleHigh((Handle)*new);
	entryPtr = (EntryRecPtr)**new;
	
	/* Allocate memory for entry line buffer */
	if ( !NewEntryLineBuf(&entryPtr->lineBufHdl) ) {
		DisposHandle((Handle)*new);
		*new = NULL;
		return(FALSE);
	}
	
	/* Init other fields */
	entryPtr->dbcode=dbcode;
	entryPtr->nlines = 0;
	entryPtr->buftop = 0;
	entryPtr->seqline_offset = 0L;
	entryPtr->seqline_no = 0;
	for(i=0;i<MAXBUFPARTS;entryPtr->bufpart[i++]=0L)
		;
	entryPtr->xrefHdl = NULL;
	entryPtr->firstSel = entryPtr->lastSel = -1;
	
	HUnlock((Handle)*new);
	return(TRUE);
}


/**************************************
*	Dispose of contents of an existing entry record
*	Return values:
*		none
*/

static void DisposeEntryRec(EntryRecHdl entryRecHdl)
{
	EntryRecPtr entryPtr;
	
	if(entryRecHdl == NULL)
		return;
	
	/* Get a pointer to the entry record */
	LockHandleHigh((Handle)entryRecHdl);
	entryPtr = *entryRecHdl;
	
	/* Free memory linked to this record */
	if( entryPtr->lineBufHdl != NULL)
		DisposHandle((Handle)entryPtr->lineBufHdl);
	if( entryPtr->xrefHdl != NULL)
		DisposHandle((Handle)entryPtr->xrefHdl);
		
	HUnlock((Handle)entryRecHdl);
}



/**************************************
*	Close entry window and dispose of entry record
*/

void DisposeEntry(WDPtr wdp)
{
	if(wdp == NULL || ((WindowPeek)wdp)->windowKind != entryW)
		return;
		
	if (wdp->inUse)
		CloseWindow((WindowPtr)wdp);
		
	if(wdp->userHandle) {
		DisposeEntryRec((EntryRecHdl)wdp->userHandle);
		DisposHandle(wdp->userHandle);
	}
		
	ClearWindowRec(wdp);
}


/**************************************
*	Fill entry line buffer from disk so that "from" is in buffer.
*	To avoid continous loading when a user browses through the entry,
*	the buffer is filled (if possible) such that "from" is in the middle of
*	the buffer.
*	Return values:
*		TRUE, if successful,
*		FALSE, if an error occurred
*/

#define MAXCHARPERLINE 80

Boolean FillLineBuffer(EntryRecHdl entryRecHdl,short first)
{
	CString80Ptr	buf;
	Str255			fName;
	short				input;
	SignedByte		oldState;
	short				i,part,cnt;
	u_long			pos;
	long				size,count;
	OSErr				err = noErr;
	char				line[MAXCHARPERLINE + 2],*s,*t;
	char				*endOfEntry;
	FSSpec			theFSS;
	
	if(entryRecHdl == NULL)
		return(FALSE);
		
	if( (**entryRecHdl).lineBufHdl == NULL )
		return(FALSE);
		
	switch((**entryRecHdl).dbcode) {
		case DB_EMBL:
		case DB_SWISS:
		case DB_PROSITEDAT:
		case DB_ENZYME:
		case DB_EPD:
			endOfEntry = "//";
			break;
		case DB_PROSITEDOC:
			endOfEntry = "{E";
			break;
	}
		
	/* calculate first hit to be loaded. The size of the buffer guarantees
		that we always load at least two screen fulls */
	if (first >= ENTRYBUFLINES/2)
		part = (first - ENTRYBUFLINES/4) / (ENTRYBUFLINES/2);
	else part = 0;
	
	/* Open data file */
	pstrcpy(fName,(**entryRecHdl).theFSS.name);
	if(OpenMacFileReadOnly(&(**entryRecHdl).theFSS,&input, TRUE) != noErr)
		return(FALSE);
	
	/* Lock block */
	oldState = LockHandleHigh((Handle)(**entryRecHdl).lineBufHdl);
	buf = *((**entryRecHdl).lineBufHdl);
	
	if( (err = SetFPos(input,fsFromStart,(**entryRecHdl).bufpart[part])) != noErr) {
		sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
						p2cstr(fName),err );
		ErrorMsg(0);
	}
	else {
		for(i=0; i<ENTRYBUFLINES; ++i) { 
			/* read line */
			count= MAXCHARPERLINE + 2;		/* plus \r\n */
			if( (err=ReadLine(input,&count,line,fName)) != noErr )
				break;
			
			for(cnt=0,s=line,t=buf[i];cnt < 80 && *s != '\r';*t++ = *s++)
				++cnt;
			*t = EOS;
	
			c2pstr((char *)buf[i]);
			if( linetype(line,endOfEntry))
				break;
		}
	}

	FSClose(input);
	HSetState((Handle)(**entryRecHdl).lineBufHdl,oldState);
	(**entryRecHdl).buftop = part*(ENTRYBUFLINES/2);

	return(err == noErr);
}



/**************************************
*	Reads entry line by line and stores information in EntryRec and lineBuf
*	Return values:
*		TRUE if successful, FALSE on failure
*/

static Boolean FillEntryRec(ResultHdl resHdl,short bufPos,EntryRecHdl entryRecHdl)
{
	HitlistHdl		hlHdl;
	short				dbcode;
	Str255			indexFName,divFName;
	short				input;
	u_long			rec;
	OSErr				err;
	long				size,count,offset;
	EnameRec			*enameRec;
	CString80Ptr	buf;
	SignedByte		oldState;
	short				nlines,part;
	char				line[MAXCHARPERLINE + 2];
	char				*s,*t;
	short				maxlines,cnt;
	char				*endOfEntry;
	FSSpec			theFSS;
	
	if(resHdl == NULL || entryRecHdl == NULL)
		return(FALSE);
				
	if( (hlHdl = (**resHdl).hlHdl) == NULL )
		return(FALSE);
		
	dbcode = (**entryRecHdl).dbcode;
	
	if((enameRec = (EnameRec *) NewPtrClear((Size)gDBInfo[dbcode].ename_recsize)) == NULL)
		return(ErrorMsg(ERR_MEMORY));
		
	switch(dbcode) {
		case DB_EMBL:
		case DB_SWISS:
		case DB_PROSITEDAT:
		case DB_ENZYME:
		case DB_EPD:
			endOfEntry = "//";
			break;
		case DB_PROSITEDOC:
			endOfEntry = "{E";
			break;
	}
	
	/* Open entryname index  */
	pstrcpy(indexFName,gFileList.enameIdxFName);
	MakeFSS(gDBInfo[dbcode].InxVRefNum,gFileList.enameIdxFName,gDBInfo[dbcode].InxDirID,
				&theFSS);
	if( OpenMacFileReadOnly(&theFSS,&input, TRUE) != noErr ) {
		DisposePtr((Ptr)enameRec);
		return(FALSE);
	}
	
	/* goto appropriate position */
	size = gDBInfo[dbcode].ename_recsize;
	rec = (*hlHdl)[bufPos].ename_rec;
	
	if( (err=SetFPos(input,fsFromStart,rec*size+sizeof(Header))) != noErr) {
		sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
					p2cstr(indexFName),err );
		FSClose(input);
		DisposePtr((Ptr)enameRec);
		return(ErrorMsg(0));
	}

	/* read record */
	count=size;
	if( ReadMacFile(input,&count,(Ptr)enameRec,indexFName,TRUE) ) {
		FSClose(input);
		DisposePtr((Ptr)enameRec);
		return(FALSE);
	}
	else
		FSClose(input);
	
	/* convert values */
	TransformEnameRec(enameRec,gDBInfo[dbcode].ename_len);
	ConvertLong(&enameRec->annotation_offset);
	ConvertLong(&enameRec->sequence_offset);
	ConvertShort(&enameRec->div_code);
	
	/* Store data file name and vRefNum */	
	pstrcpy(divFName,
		(StringPtr)(*gDBInfo[dbcode].gDivNames + (enameRec->div_code)));
	MakeFSS(gDBInfo[dbcode].DataVRefNum,
			(StringPtr)(*gDBInfo[dbcode].gDivNames + (enameRec->div_code)),
			gDBInfo[dbcode].DataDirID,
			&(**entryRecHdl).theFSS);
	
	/* Open data file */
	if( OpenMacFileReadOnly(&(**entryRecHdl).theFSS,&input,TRUE) != noErr ) {
		DisposePtr((Ptr)enameRec);
		return(FALSE);
	}
	
	/* Move to entry */
	offset=enameRec->annotation_offset;
		
	if( (err=SetFPos(input,fsFromStart,offset)) != noErr ) {
		FSClose(input);
		sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),p2cstr(divFName),err );
		DisposePtr((Ptr)enameRec);
		return(ErrorMsg(0));
	}

	oldState=LockHandleHigh((Handle)(**entryRecHdl).lineBufHdl);
	buf = *((**entryRecHdl).lineBufHdl);
	nlines=0;
	part = 0;
	
	err=noErr;
	maxlines = MAXBUFPARTS * ENTRYBUFLINES;
	do {
		RotateWaitCursor();
		
		/* store byte offset for every ENTRYBUFLINES/2th line */
		if( nlines % (ENTRYBUFLINES/2) == 0 ) {
			if( (err=GetFPos(input,&((**entryRecHdl).bufpart[part]))) != noErr ) {
				sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
						p2cstr(divFName),err );
				ErrorMsg(0);
				break;
			}
			else
				++part;
		}
		
		/* read line */
		count=MAXCHARPERLINE + 2;
		if( (err=ReadLine(input,&count,line,divFName)) != noErr )
			break;
		
		if(nlines < ENTRYBUFLINES) {
			for(cnt=0,s=line,t=buf[nlines];cnt < 80 && *s != '\r';*t++ = *s++)
				++cnt;
			*t = EOS;
			c2pstr((char *)buf[nlines]);
		}
		
		if( linetype(line,"DR") ) {
			if(!ParseDRLine(dbcode,entryRecHdl,line)) {
				err = notEnoughMemoryErr;
				break;
			}
		}
		else if ( dbcode == DB_PROSITEDAT && linetype(line,"DO")) {
			if(!ParseDOLine(entryRecHdl,line)) {
				err = notEnoughMemoryErr;
				break;
			}
		}
		else if ( dbcode == DB_PROSITEDOC && linetype(line,"{PS")) {
			if(!ParsePSLine(entryRecHdl,line)) {
				err = notEnoughMemoryErr;
				break;
			}
		}
		else if ( dbcode == DB_EPD && (linetype(line,"FP") || linetype(line,"F-")) ) {
			if(!ParseFPLine(entryRecHdl,line)) {
				err = notEnoughMemoryErr;
				break;
			}
		}
		else if( linetype(line,"SQ") ) {
			(**entryRecHdl).seqline_no = nlines + 1;
			if( (err=GetFPos(input,&((**entryRecHdl).seqline_offset))) != noErr ) {
				sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
						p2cstr(divFName),err );
				ErrorMsg(0);
				break;
			}
		}
		
		++nlines;
	} while( nlines < maxlines && !linetype(line,endOfEntry) );
	
	(**entryRecHdl).nlines = nlines;
	(**entryRecHdl).buftop = 0;
	
	HSetState((Handle)(**entryRecHdl).lineBufHdl,oldState);
	FSClose(input);
	DisposePtr((Ptr)enameRec);

	return(err == noErr);
}

/**************************************
*	Reads database entry line by line
*	Return values:
*		OSErr
*/

OSErr ReadLine(short refNum, long *count, Ptr buf, StringPtr fName)
{
	ParamBlockRec	pb;			
	OSErr			err;
	
	pb.ioParam.ioRefNum		= refNum;
	pb.ioParam.ioCompletion	= NULL;
	pb.ioParam.ioBuffer		= buf;
	pb.ioParam.ioReqCount	= *count;
	pb.ioParam.ioPosMode	=  ('\n' << 8) | 128 ;
	pb.ioParam.ioPosOffset	= 0;
	
	if( (err = PBRead((ParmBlkPtr) &pb, FALSE)) == noErr ) {
		*count = pb.ioParam.ioActCount;
		buf[*count] = EOS;
	}
	else {
		sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),p2cstr(fName),err);
		c2pstr((char *)fName);
		ErrorMsg(0);
 	}
		
	return(err);
}


/**************************************
*	Completely redraws a entry window
*/

void DrawEntryWinAll(WDPtr wdp,short dummy)
{
	GrafPtr			savePort;
	Rect				area,linebox,viewRect;
	short				i,visLines;
	EntryRecHdl		entryRecHdl;
	CString80Hdl	bufHdl;
	WindowPtr		topWindow = FrontWindow();
	short 			vert,horiz;
	short				bufPos,linePos,horizPos;
	SignedByte		oldState;
	RgnHandle		oldClipRgn,newClipRgn;
	
	if(wdp == NULL || ((WindowPeek)wdp)->windowKind != entryW)
		return;
			
	/* Locate and lock down entry data */
	entryRecHdl = (EntryRecHdl)(wdp->userHandle);
	if(entryRecHdl == NULL)
		return;
	bufHdl = (**entryRecHdl).lineBufHdl;
	oldState=LockHandleHigh((Handle)bufHdl);
	
	savePort = ChangePort((GrafPtr)wdp);
		
	/* Calculate text area without small margin (for redrawing) */
	GetViewRect((WindowPtr)wdp,&area);
	viewRect = area;
	InsetRect(&viewRect,ENTRY_MARGIN,0);
	
	/* How many lines to be shown */
	visLines = (area.bottom - area.top) / fMonaco.height;

	linePos = GetCtlValue(wdp->vScroll);	/* first hit on screen */
	/* Check whether buffer contains enough lines */
	if(	(linePos + visLines > (**entryRecHdl).buftop +ENTRYBUFLINES) ||
			(linePos < (**entryRecHdl).buftop) ) {
		/* Refill buffer with new chunk of data */
		if( !FillLineBuffer(entryRecHdl,linePos) )
			goto err;
	}

	bufPos = linePos - (**entryRecHdl).buftop;	/* where in buffer is first line? */

	/* empty drawing area */
	EraseRect(&area);
	
	/* store old clip region */
	oldClipRgn = NewRgn();
	GetClip(oldClipRgn);
	
	/* calculate horizontal and vertical drawing positions */
	horizPos = GetCtlValue(wdp->hScroll);
	horiz = area.left + ENTRY_MARGIN - horizPos*fMonaco.finfo.widMax;
					/* this takes horizontal scrolling into account */
	vert = area.top + fMonaco.finfo.ascent + fMonaco.finfo.leading;
	
	/* initialise area for "selected" box (hilighted hits) */
	linebox = area;
	linebox.bottom = linebox.top + fMonaco.height;
	
	/* Replace clip region by section of clip region and text view area */
	newClipRgn = NewRgn();
	RectRgn(newClipRgn,&viewRect);
	SectRgn(oldClipRgn,newClipRgn,newClipRgn);

	/* now draw line by line and advance printing position */
	for(	i = 0;
			i < visLines && bufPos < ENTRYBUFLINES && linePos < (**entryRecHdl).nlines;
			++i, ++bufPos, ++linePos, vert += fMonaco.height ) {

		SetClip(newClipRgn);
		MoveTo(horiz,vert);
		if(gPrefs.markHyperLink && IsXref((**entryRecHdl).dbcode,(StringPtr)(*bufHdl)[bufPos]))
			DrawStringItalic( (StringPtr)(*bufHdl)[bufPos]);
		else
			DrawString( (StringPtr)(*bufHdl)[bufPos] );
		
		/* Reset old clip region */
		SetClip(oldClipRgn);
		
		/* Hilite if selected */
		if( !gInBackground &&
				(WindowPtr)wdp == topWindow &&
				linePos >= (**entryRecHdl).firstSel && linePos <= (**entryRecHdl).lastSel )
			DoInvertRect(&linebox);
		OffsetRect(&linebox,0,fMonaco.height);	/* next line */
	}
	
	DisposeRgn(newClipRgn);		
	DisposeRgn(oldClipRgn);
	
err:
	SetPort(savePort);
	HSetState((Handle)bufHdl,oldState);
}


static void DrawEntryWin1(WDPtr wdp, short updown)
{
	GrafPtr			savePort;
	Rect				area,linebox,viewRect;
	short				visLines;
	EntryRecHdl		entryRecHdl;
	CString80Hdl	bufHdl;
	WindowPtr		topWindow = FrontWindow();
	short				vert,horiz;
	short				linePos,bufPos,horizPos;
	SignedByte 		oldState;
	RgnHandle		oldClipRgn,newClipRgn;

	if(wdp == NULL || ((WindowPeek)wdp)->windowKind != entryW)
		return;
		
	/* Locate and lock down entry data */
	entryRecHdl = (EntryRecHdl)(wdp->userHandle);
	if(entryRecHdl == NULL)
		return;
	bufHdl = (**entryRecHdl).lineBufHdl;
	oldState=LockHandleHigh((Handle)bufHdl);
			
	savePort = ChangePort((GrafPtr)wdp);
	
	/* Calculate text area without small margin (for redrawing) */
	GetViewRect((WindowPtr)wdp,&area);
	viewRect = area;
	InsetRect(&viewRect,ENTRY_MARGIN,0);

	/* How many lines on screen ? */
	visLines = (area.bottom - area.top) / fMonaco.height;
	if (visLines < 1)
		return;

	/* first hit on page = first line on screen */
	linePos = GetCtlValue(wdp->vScroll);
	/* Check whether buffer contains enough lines */
	if(	(linePos + visLines > (**entryRecHdl).buftop +ENTRYBUFLINES) ||
			(linePos < (**entryRecHdl).buftop) ) {
		/* Refill buffer with new chunk of data */
		if( !FillLineBuffer(entryRecHdl,linePos) )
			goto err;
	}
	
	/* calculate horizontal and vertical drawing positions */
	horizPos = GetCtlValue(wdp->hScroll);
	horiz = area.left + ENTRY_MARGIN - horizPos*fMonaco.finfo.widMax;
	vert = area.top + fMonaco.finfo.ascent + fMonaco.finfo.leading;

	/* Draw first line or last line ? */
	if( updown > 0) { /* Down arrown/page => move upwards */
		linePos += visLines-1;	/* we must redraw last line on screen */
		vert += fMonaco.height * (visLines-1);
	}

	/* Is last line blank? Then we can exit */
	if(linePos > (**entryRecHdl).nlines)
		goto err;

	bufPos = linePos - (**entryRecHdl).buftop; /* where in buffer */

	/* store old clip region */
	oldClipRgn = NewRgn();
	GetClip(oldClipRgn);
	
	/* Replace clip region by section of clip region and text view area */
	newClipRgn = NewRgn();
	RectRgn(newClipRgn,&viewRect);
	SectRgn(oldClipRgn,newClipRgn,newClipRgn);
	SetClip(newClipRgn);
	DisposeRgn(newClipRgn);	
		
	/* Draw that line */
	MoveTo(horiz,vert);
	if(gPrefs.markHyperLink && IsXref((**entryRecHdl).dbcode,(StringPtr)(*bufHdl)[bufPos]))
		DrawStringItalic( (StringPtr)(*bufHdl)[bufPos] );
	else
		DrawString( (StringPtr)(*bufHdl)[bufPos] );
		
	/* Reset old clip region */
	SetClip(oldClipRgn);
	DisposeRgn(oldClipRgn);
	
	/* Is it selected ? */
	if( (WindowPtr)wdp == topWindow &&
		 linePos >= (**entryRecHdl).firstSel && linePos <= (**entryRecHdl).lastSel ) {
		linebox = area;
		linebox.bottom = vert + fMonaco.finfo.descent;
		linebox.top = linebox.bottom - fMonaco.height;
		DoInvertRect(&linebox);
	}

err:
	SetPort(savePort);
	HSetState((Handle)bufHdl,oldState);
}


/**************************************
*	Redrawing of screen after scrolling or sizing 
*/

void AdjustEntryText(WindowPtr wPtr,short oldvalue,short newvalue,
						scrollBarType which)
{
	void			(*drawFct)(WDPtr, short);
	Rect			r;
	RgnHandle	updateRgn, clip, eraseRgn;
	EntryRecHdl	entryRecHdl;
	short			delta = oldvalue-newvalue;
	WDPtr			wdp = FindMyWindow(wPtr);
	
	if(wPtr == NULL || ((WindowPeek)wPtr)->windowKind != entryW)
		return;
		
	entryRecHdl = (EntryRecHdl)wdp->userHandle;
	if(entryRecHdl == NULL)
		return;
	
	/* calculate drawing area */
	GetViewRect(wPtr,&r);
	
	/* We need a new region for ScrollRect */
	updateRgn  = NewRgn();
	
	if(which == vertBar) {
		/* Scroll screen vertically */
		ScrollRect(&r,0,delta * fMonaco.height,updateRgn);
		/* Redraw complete screen or only one line (top or bottom) ? */
		if(delta == 1 || delta == -1)
			drawFct = DrawEntryWin1;
		else
			drawFct = DrawEntryWinAll;
	}
	else {
		/* scroll screen horizontally */
		ScrollRect(&r,delta * fMonaco.finfo.widMax, 0,updateRgn);
		drawFct = DrawEntryWinAll;	/* redraw complete screen */
	}
	
	/* clip drawing area and do drawing */
	clip = wPtr->clipRgn;
	wPtr->clipRgn = updateRgn;
	(*drawFct)(wdp,-delta);
	wPtr->clipRgn = clip;
	DisposeRgn(updateRgn);
}


void DoEntryClicks(WDPtr wdp, EventRecord *eventPtr)
{
	ControlHandle	theControl;
	short				cntlCode;
	short				oldvalue,newvalue;
	Rect				viewRect;
	short				pagesize;
	Point				where;
	GrafPtr			savePort;
	
	if(wdp == NULL || ((WindowPeek)wdp)->windowKind != entryW)
		return;
		
	savePort = ChangePort((GrafPtr)wdp);
	
	where = eventPtr->where;
	GlobalToLocal(&where);	/* convert to local coordinates 	*/
	GetViewRect((WindowPtr)wdp,&viewRect);
	
	if (!(cntlCode = FindControl(where, (WindowPtr)wdp, &theControl))) {
		if(PtInRect(where,&viewRect))
			DoEntryContentClick(eventPtr,&viewRect,wdp);
	}
	else {
	   if( theControl == wdp->vScroll ) {	/* click in scroll bar */
			gScrollDir = vertBar;
			pagesize = (viewRect.bottom - viewRect.top) / fMonaco.height;
		}
		else if (theControl == wdp->hScroll) { 
			gScrollDir = horizBar;
			pagesize = (viewRect.right - viewRect.left - 2*ENTRY_MARGIN) /
							fMonaco.finfo.widMax;;
		}
		
		if(cntlCode == inThumb) {
			oldvalue=GetCtlValue(theControl);
			TrackControl(theControl, where, NULL);
			AdjustEntryText((WindowPtr)wdp,oldvalue,GetCtlValue(theControl),gScrollDir);
		}
		else {
			switch(cntlCode) {
				case inUpButton: gScrollAmt = -1;
					break;
				case inDownButton: gScrollAmt = 1;
					break;
				case inPageUp: gScrollAmt = -pagesize;
					break;
				case inPageDown: gScrollAmt = pagesize;
					break;
			}
			gScrollCode=cntlCode;
			TrackControl(theControl, where, (ProcPtr)ScrollEntryProc);
		} /* end else (cntlCode == inThumb) */
	} /* end else FindControl */
	SetPort(savePort);
}


/**************************************
*	Callback procedure called by TrackControl 
*/

static pascal void	ScrollEntryProc(ControlHandle theControl, short theCode)
{
	short			oldValue,newValue;
	WindowPtr	ctlWindow=(*theControl)->contrlOwner;
	short			max=GetCtlMax(theControl);
	
	if(theCode == gScrollCode) {
		oldValue=GetCtlValue(theControl);	
		newValue=oldValue+gScrollAmt;
		/* limit check */
		if(newValue < 0) newValue=0;
		if(newValue > max) newValue=max;
			
		SetCtlValue( theControl, newValue );
		AdjustEntryText(ctlWindow,oldValue,newValue,gScrollDir);
	}
}


/**************************************
*	(De)Selects whole entry and redraws screen
*/

void SelectAllEntry(WDPtr wdp,Boolean what)
{
	short			i;
	EntryRecHdl	entryHdl;
	Rect			r;
	GrafPtr		savePort;
	
	if(wdp == NULL || ((WindowPeek)wdp)->windowKind != entryW)
		return;
		
	entryHdl = (EntryRecHdl)(wdp->userHandle);
	if(entryHdl == NULL)
		return;

	savePort = ChangePort((GrafPtr)wdp);
	if(what) {
		(**entryHdl).firstSel = 0;
		(**entryHdl).lastSel = (**entryHdl).nlines - 1;
	}
	else {
		(**entryHdl).firstSel = (**entryHdl).lastSel = -1;
	}
	
	InvalRect(&((WindowPtr)wdp)->portRect);
	SetPort(savePort);
}


/**************************************
*	Show hiliting or remove it (eg if window becomes active/inactive)
*/

void HideShowEntrySelections(WDPtr wdp, Boolean show)
{
	EntryRecHdl		entryRecHdl;
	SignedByte 		oldState;
	CString80Hdl	bufHdl;
	Rect 				viewRect,linebox;
	short 			screenTop,visLines;
	short 			i;
	short 			top,horizPos,horiz,vert;
	RgnHandle		oldClipRgn,newClipRgn;
	GrafPtr			savePort;
	
	if(wdp == NULL || ((WindowPeek)wdp)->windowKind != entryW)
		return;
		
	/* Locate and lock down entry data */
	entryRecHdl = (EntryRecHdl)(wdp->userHandle);
	if(entryRecHdl == NULL)
		return;
	bufHdl = (**entryRecHdl).lineBufHdl;
	oldState=LockHandleHigh((Handle)bufHdl);
		
	savePort = ChangePort((GrafPtr)wdp);
		
	GetViewRect((WindowPtr)wdp,&viewRect);
	top = viewRect.top;
	
	/* store old clip region */
	oldClipRgn = NewRgn();
	GetClip(oldClipRgn);

	/* Replace clip region by section of clip region and text view area */
	newClipRgn = NewRgn();
	RectRgn(newClipRgn,&viewRect);
	SectRgn(oldClipRgn,newClipRgn,newClipRgn);

	/* calculate horizontal and vertical drawing positions */
	horizPos = GetCtlValue(wdp->hScroll);
	horiz = viewRect.left + ENTRY_MARGIN - horizPos*fMonaco.finfo.widMax;
					/* this takes horizontal scrolling into account */
	vert = viewRect.top + fMonaco.finfo.ascent + fMonaco.finfo.leading;
	
	/* initialise area for "selected" box (hilighted hits) */
	linebox = viewRect;
	linebox.bottom = linebox.top + fMonaco.height;

	screenTop = GetCtlValue(wdp->vScroll);	/* first hit on screen */
	visLines = (viewRect.bottom - viewRect.top) / fMonaco.height;

	/* Check whether buffer contains enough lines */
	if(	(screenTop + visLines > (**entryRecHdl).buftop +ENTRYBUFLINES) ||
			(screenTop < (**entryRecHdl).buftop) ) {
		/* Refill buffer with new chunk of data */
		if( !FillLineBuffer(entryRecHdl,screenTop) )
			goto err;
	}

	for(i = screenTop; i < screenTop + visLines && i <(**entryRecHdl).nlines; ++i) {
		if( i >= (**entryRecHdl).firstSel && i <= (**entryRecHdl).lastSel ) {
			viewRect.top = top + (i-screenTop) * fMonaco.height;
			viewRect.bottom = viewRect.top + fMonaco.height;
			EraseRect(&viewRect);
			vert = viewRect.top + fMonaco.finfo.ascent + fMonaco.finfo.leading;
			MoveTo(horiz,vert);
			if(gPrefs.markHyperLink && IsXref((**entryRecHdl).dbcode,(StringPtr)(*bufHdl)[i - (**entryRecHdl).buftop]))
				DrawStringItalic( (StringPtr)(*bufHdl)[i - (**entryRecHdl).buftop] );
			else
				DrawString( (StringPtr)(*bufHdl)[i - (**entryRecHdl).buftop] );

			if(show)
				DoInvertRect(&viewRect);
			ValidRect(&viewRect);
		}
	}
	
err:
	/* Reset old clip region */
	SetClip(oldClipRgn);
	DisposeRgn(newClipRgn);		
	DisposeRgn(oldClipRgn);
	SetPort(savePort);
	HSetState((Handle)bufHdl,oldState);
}


Boolean OpenEntries(WDPtr wdp)
{
	register short i;
	SignedByte		oldRState;
	ResultHdl		resHdl;
	HitlistHdl		hlHdl;
	Boolean			ret;
	
	if( wdp == NULL || ((WindowPeek)wdp)->windowKind != resW )
		return;
	
	resHdl = (ResultHdl)(wdp->userHandle);
	oldRState = MyHLock((Handle)resHdl);
	hlHdl = (**resHdl).hlHdl;
	ret = TRUE;
	
	for(i=0;i < (**resHdl).nhits; i++) {
		if( GetSelectState(hlHdl,i) ) {
			if( !(ret=NewEntryWindow(resHdl,i)) )
				break;
		}
	}
	HSetState((Handle)resHdl,oldRState);
	return(ret);
}

static void DrawStringItalic(StringPtr str)
{
	TextFace(italic);
	DrawString(str);
	TextFace(normal);
}