/*
*********************************************************************
*	
*	Export.c
*	Export of database entries
*
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/ 

#include <ctype.h>
#include <string.h>
#include <stdio.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "export.h"

#include "entry.h"
#include "environment.h"
#include "events.h"
#include "results.h"
#include "main.h"
#include "pref.h"
#include "pstr.h"
#include "util.h"

/*
******************************* Prototypes ***************************
*/

static Boolean Export1Entry(HitlistHdl hlHdl, short pos, short output, short dbcode, StringPtr outFName, char *inpBuf, char *outBuf, DialogPtr myDialog);
static Boolean WriteWholeEntry(char *inpBuf,
										char *outBuf, short output, StringPtr outFName,
										long count, char *endOfEntry);
static Boolean WriteSeqOnly(char *inpBuf, char *outBuf, short output, StringPtr outFName, long count);
static short GetExportFilename(StringPtr oldFName, FSSpecPtr theFSS);
static pascal short myExportDlgHook(short theItem, DialogPtr myDialog);



/*
********************************* Globals *****************************
*/


static Boolean lbSkipButton,lbSaveAllButton;


/**************************************
*	Export entries
*/

void ExportEntries(WDPtr wdp)
{
	ResultHdl		resHdl;
	HitlistHdl		hlHdl;
	CString80Hdl	bufHdl;
	Str255			ename,oldFName;
	short				i, found;
	short				output=0;
	SignedByte		oldState;
	Boolean			ret,ret2;
	char 				*inpBuf,*outBuf;
	DialogPtr		myDialog = NULL;
	DialogTHndl		myHandle;
	Boolean			bUserBreak = FALSE, bUserNotified = FALSE;
	NMRec				myNotification;
	FSSpec			theFSS;
	short				dbcode;
	OSType			fileType, fileCreator;
	
	if( wdp == NULL || ((WindowPeek)wdp)->windowKind != resW )
		return;
		
	resHdl = (ResultHdl)(wdp->userHandle);
	if( !resHdl )
		return;
		
	dbcode = (**resHdl).dbcode;
		
	hlHdl = (**resHdl).hlHdl;
	if( !hlHdl )
		return;
	oldState = LockHandleHigh((Handle)hlHdl);
	
	bufHdl = (**resHdl).descBufHdl;
	
	/* Allocate file buffers */
	inpBuf=NewPtr((Size)FILEBUF);
	if(inpBuf)
		if( !(outBuf=NewPtr((Size)FILEBUF)) )
			DisposPtr((Ptr)inpBuf);
	if ( !inpBuf || !outBuf ) {
		HSetState((Handle)hlHdl,oldState);
		ErrorMsg(ERR_MEMORY);
		return;
	}

	StartWaitCursor();
	
	/* Loop for all hits */
	for(i=0,found=0,lbSaveAllButton = bUserBreak = FALSE;
		 i < (**resHdl).nhits && bUserBreak == FALSE;
		 ++i) {
		/* find those which are selected */
		if( GetSelectState(hlHdl,i) ) {
			if( i < (**resHdl).buftop || i >= (**resHdl).buftop + MAXBUFLINES) {
				ret2 = FillDEBuffer(resHdl,i,FALSE);
				if( !ret2 )
					break;
			}

			ret = OK;
			
			GetEnameFromDesc(bufHdl,i-(**resHdl).buftop,dbcode,ename);
			
			/* if necessary, ask user for filename */
			if( lbSaveAllButton == FALSE ) {
				InitCursor();
				BuildFName(dbcode,ename,oldFName);
				ret=GetExportFilename(oldFName,&theFSS);
				if( ret == OK ) {
					if( CreateMacFile(&theFSS,gPrefs.creatorSig,kEntryFileType,TRUE) != noErr )
						break;
					if( OpenMacFile(&theFSS, &output, TRUE) != noErr)
						break;
				}
			}
			
			/* check whether user cancelled or wants to skip an entry */
			if( ret == Cancel ) {
				if(lbSkipButton)
					continue;
				else
					break;
			}
			else {
				RotateWaitCursor();
				
				if(myDialog == NULL) {
					if(gSystem.sysVersion >= 0x0700) {
						myHandle = (DialogTHndl)GetResource('DLOG',EXPORT_DLG);
						(**myHandle).procID = movableDBoxProc;
					}
					myDialog = GetNewCenteredDialog(EXPORT_DLG,50);
					DisableMenuBar();
				}
				
				ParamText(ename,"\p","\p","\p");
				ShowWindow(myDialog);
				DrawDialog(myDialog);
				if(UserBreak(myDialog))
					bUserBreak = TRUE;
				else {
					/* copy entry to disk */
					if( !Export1Entry(hlHdl,i,output,dbcode,
									theFSS.name,inpBuf,outBuf,myDialog) )
						break;
									
					++found;
					if( found > (**resHdl).nsel )
						break;
						
					/* close file if necessary */
					if( lbSaveAllButton == FALSE ) {
						FSClose(output);
						output = 0;
					}
					
					if(lbSaveAllButton == FALSE) {
						DisposDialog(myDialog);
						myDialog = NULL;
					}
				}
			}
		}
	}
	
	if(myDialog) DisposDialog(myDialog);
	if (output) FSClose(output);

	if(gPrefs.format == DNASTAR_FORMAT && (dbcode == DB_SWISS || dbcode == DB_EMBL)) {
		fileType = (dbcode == DB_SWISS) ? 'SEQA' : 'SEQU';
		fileCreator = 'ESEQ';
		SetFileType(&theFSS,fileType,fileCreator,TRUE);
	}
		
	FlushVol(NULL,theFSS.vRefNum);
	HSetState((Handle)hlHdl,oldState);
	DisposPtr((Ptr)inpBuf);
	DisposPtr((Ptr)outBuf);
	EnableMenuBar();
	InitCursor();
	
	/* if we were switched to background we notify user of completion */
	if (gInBackground) {
		bUserNotified = TRUE;
		NotifyUser(&myNotification);
	}
	else
		bUserNotified = FALSE;
		
	/* wait till we're activated */
	WaitUntilForeground();
	
	/* remove notification */
	if(bUserNotified == TRUE)
		RemoveNotification(&myNotification);
}

/**************************************
*	Look up entries in entryname index for export, and call WriteDatabaseEntry to
*	do the job
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean Export1Entry(HitlistHdl hlHdl, short pos,short output,short dbcode,
						StringPtr outFName, char *inpBuf,char *outBuf,DialogPtr myDialog)
{
	OSErr		err;
	short		input;
	long		size,count;
	u_long	rec;
	EnameRec	*enameRec;
	Str255	indexFName,divFName;
	Boolean	ret;
	FSSpec	theFSS;
	
	if ( (enameRec = (EnameRec *)NewPtrClear((Size)gDBInfo[dbcode].ename_recsize)) == NULL)
		return(ErrorMsg(ERR_MEMORY));
		
	/* Open entryname index  */
	pstrcpy(indexFName,gFileList.enameIdxFName);
	MakeFSS(gDBInfo[dbcode].InxVRefNum,indexFName,gDBInfo[dbcode].InxDirID,&theFSS);
	if( OpenMacFileReadOnly(&theFSS,&input, TRUE) != noErr ) {
		DisposePtr((Ptr)enameRec);
		return(FALSE);
	}
	
	/* goto appropriate position */
	size = gDBInfo[dbcode].ename_recsize;
	rec = (*hlHdl)[pos].ename_rec;
	
	if( (err=SetFPos(input,fsFromStart,rec*size + sizeof(Header))) != noErr) {
		sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
					p2cstr(indexFName),err );
		FSClose(input);
		DisposePtr((Ptr)enameRec);
		return(ErrorMsg(0));
	}

	/* read record */
	count=size;
	if( ReadMacFile(input,&count,(Ptr)enameRec,indexFName,TRUE) ) {
		FSClose(input);
		DisposePtr((Ptr)enameRec);
		return(FALSE);
	}
	else
		FSClose(input);
	
	/* convert values */
	TransformEnameRec(enameRec, gDBInfo[dbcode].ename_len);
	ConvertLong(&enameRec->annotation_offset);
	ConvertLong(&enameRec->sequence_offset);
	ConvertShort(&enameRec->div_code);
	
	/* Store data file name */	
	pstrcpy(divFName,
		(StringPtr)(*gDBInfo[dbcode].gDivNames + (enameRec->div_code)));
	
	/* Open data file */
	MakeFSS(gDBInfo[dbcode].DataVRefNum,divFName,gDBInfo[dbcode].DataDirID,&theFSS);
	if(OpenMacFileReadOnly(&theFSS,&input,TRUE) != noErr ) {
		DisposePtr((Ptr)enameRec);
		return(FALSE);
	}
	
	ret=WriteDatabaseEntry(input,enameRec->annotation_offset,enameRec->sequence_offset,
								dbcode,divFName,
								inpBuf,outBuf,output,outFName,myDialog);
	FSClose(input);
	DisposePtr((Ptr)enameRec);
	
	if(!ret)
		return(FALSE);
	else {
		if(gPrefs.format != DNASTAR_FORMAT) {
			count=1;
			WriteMacFile(output,&count,"\r",outFName, TRUE);
		}
		return(TRUE);
	}
}


/**************************************
*	Read an entry from database and write it to a file
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

Boolean WriteDatabaseEntry(short input,long ann_offset,long seq_offset,short dbcode,StringPtr fName,
				char *inpBuf,char *outBuf,short output,StringPtr outFName,
				DialogPtr myDialog)
{
	OSErr		err;
	Boolean	bDone;
	long		count;
	short		i;
	char		ename[255];
	char		*pos;
	long		offset;
	char		c;
	short		format;
	short		enamelen;
	char		*inpPtr;
	OSType	fileType,fileCreator;
	
#define _MAXDESCLEN	60
	char description[_MAXDESCLEN+1];
	
	switch(dbcode) {
		case DB_EMBL:
		case DB_SWISS:
			enamelen = SEQENTRYNAMELEN;
			format = gPrefs.format;
			offset = (format == STADEN_FORMAT) ? seq_offset : ann_offset;
			break;
		case DB_PROSITEDAT:
		case DB_ENZYME:
		case DB_EPD:
			enamelen = PROSITEENTRYNAMELEN;
			format = EMBL_FORMAT;
			offset = ann_offset;
			break;
		case DB_PROSITEDOC:
			enamelen = PROSITEENTRYNAMELEN;
			format = PROSITEDOC_FORMAT;
			offset = ann_offset;
			break;
	}
	
	/* Go to entry in database */
	if( (err=SetFPos(input,fsFromStart,offset)) != noErr) {
		sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),p2cstr(fName),err );
		c2pstr((char *)fName);
		return(ErrorMsg(0));
	}

	if(format == PIR_FORMAT || format == FASTA_FORMAT || format == DNASTAR_FORMAT) {
		RotateWaitCursor();	/* we're busy */
		
		/* read first block of data */
		count=FILEBUF;
		if( (err=ReadMacFile(input,&count,inpBuf,fName,TRUE)) != noErr && err != eofErr )
			return(FALSE);
		if (!count)
			return(TRUE);	/* or false ?! */
	
		/* extract entryname */
		strncpy(ename,inpBuf+5,enamelen);
		ename[enamelen]=EOS;
		rtrim(ename);
	
		/* extract description */
		if ( (pos = strstr(inpBuf,"\nDE   ")) != NULL) {
			for(i=0,pos += 6; *pos != '\r' && i < _MAXDESCLEN; ++i,++pos) 
				description[i] = *pos;
			description[i] = EOS;
		}
		else *description = EOS;
		
		/* skip to sequence */
		RotateWaitCursor();	/* we're busy */
		if( (err=SetFPos(input,fsFromStart,seq_offset)) != noErr) {
			sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),p2cstr(fName),err );
			c2pstr((char *)fName);
			return(ErrorMsg(0));
		}
		
		if(gPrefs.format == PIR_FORMAT) {
			c = (dbcode == DB_SWISS) ? 'P' : 'D';
			sprintf(outBuf,">%c1;%s\r%s\r",c,ename,description);
		}
		else if (gPrefs.format == FASTA_FORMAT) {
			sprintf(outBuf,">%s - %s\r",ename,description);
		}
		else if(gPrefs.format == DNASTAR_FORMAT) {
			sprintf(outBuf,"%s\r\n%s\r\n^^\r\n",ename,description);
		}
		
		count=strlen(outBuf);
		if( WriteMacFile(output,&count,outBuf,outFName, TRUE) )
			return(FALSE);
	}
	
	bDone=FALSE;
	err = noErr;
	while( !bDone && err == noErr ) {
		RotateWaitCursor();	/* we're busy */
		if(UserBreak(myDialog))
			return(FALSE);
			
		/* read from database */
		count=FILEBUF;
		inpPtr = inpBuf;
		while(inpPtr - inpBuf < FILEBUF - 80) {
			count = 80;
			if ( (err = ReadLine(input, &count, inpPtr, fName)) != noErr && err != eofErr )
				return(FALSE);
			if(err == eofErr)
				break;
			inpPtr += count;
		}
		
		/* write to file. check success ?! */
		switch(format) {
			case EMBL_FORMAT:
				bDone=WriteWholeEntry(inpBuf,outBuf,output,outFName,inpPtr-inpBuf,"//");
				break;
			case STADEN_FORMAT:
			case PIR_FORMAT:
			case FASTA_FORMAT:
			case DNASTAR_FORMAT:
				bDone=WriteSeqOnly(inpBuf,outBuf,output,outFName,inpPtr-inpBuf);
				break;
			case PROSITEDOC_FORMAT:
				bDone=WriteWholeEntry(inpBuf,outBuf,output,outFName,inpPtr-inpBuf,"{END}");
				break;
		}
	}
	
	if(gPrefs.format == PIR_FORMAT) {
		*outBuf = '*';	
		count = 1;
		if( WriteMacFile(output,&count,outBuf,outFName, TRUE) )
				return(FALSE);
	}

	return(TRUE);
}

/**************************************
*	Write EMBL format
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean WriteWholeEntry(char *inpBuf,char *outBuf,short output,
								StringPtr outFName,long count, char *endOfEntry)
{
	Boolean		bFoundEnd,bLineStart;
	register		long i,j;
	
	for(i=0,j=0,bFoundEnd=FALSE,bLineStart = TRUE;i<count;++i) {
		if(inpBuf[i] == '\n') {				/* skip over line feeds */
			continue;
		}
		else if (inpBuf[i] == '\r') {
			if (bFoundEnd)
				break;
			bLineStart = TRUE;
		}
		else {
			if (bLineStart) {
				if(!strncmp(inpBuf+i,endOfEntry,strlen(endOfEntry)))
					bFoundEnd = TRUE;
				bLineStart = FALSE;
			}
		}
			
		outBuf[j++]=inpBuf[i];							/* move character to output buffer */
	}
	
	count=j;
	if( WriteMacFile(output,&count,outBuf,outFName,TRUE) )
		return(TRUE);
		
	return(bFoundEnd);
}

/**************************************
*	Write sequence only to a file
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean WriteSeqOnly(char *inpBuf,char *outBuf,short output,
								StringPtr outFName,long count)
{
	Boolean			bFoundEnd;
	register long	i,j;
	
	/* We go through the input buffer and copy every character to output buffer,
		skipping over line feeds. If we find a //, we stop. We don't copy the // */

	for(i=0,j=0,bFoundEnd=FALSE;i<count;++i) {
		if(inpBuf[i] == '/') {			/* check for end of entry */
			bFoundEnd=TRUE;			/* found // */
			break;
		}
		else if (isalpha(inpBuf[i]))	/* dump all alphabetic characters */
			outBuf[j++]=inpBuf[i];
		else if (inpBuf[i] == '\r') {	/* dump linefeed */
			if(gPrefs.format != DNASTAR_FORMAT)	/* but not in DNASTAR format */
				outBuf[j++]=inpBuf[i];	
		}
	}
		
	count=j;
	if( WriteMacFile(output,&count,outBuf,outFName,TRUE) )
		return(TRUE);
	return(bFoundEnd);				
}

/**************************************
*	Extracts entryname from short description
*	Return value:	none
*	By side-effect: entry name in "ename"
*/

void GetEnameFromDesc(CString80Hdl bufHdl, short pos,short dbcode, StringPtr ename)
{
	short length = gDBInfo[dbcode].ename_len;
	
	BlockMove((*bufHdl)[pos]+1,ename,length);
	ename[length]=EOS;
	rtrim((char *)ename);
	c2pstr((char *)ename);
}
	

/**************************************
*	Construct default file name from entryname
*	Return value:	none
*	Side-effect:	Pascal file name in "filename"
*/

void BuildFName(short dbcode, StringPtr ename, StringPtr filename)
{
	Str255 extension;
	
	pstrcpy(filename,ename);
	if(dbcode == DB_EMBL || dbcode == DB_SWISS)
		GetIndString(extension,EXTENSIONS_STR,gPrefs.format);
	else
		GetIndString(extension,EXTENSIONS_STR,1);
	pstrcat(filename,extension);
}

/**************************************
*	Ask user for filename
*	Return value:	Code of selected button
*	Side-effect:	selected file name in "newFName"
*/

static short GetExportFilename(StringPtr oldFName,FSSpecPtr theFSS)
{
	SFReply	reply;
	Point		where;
	Str255	prompt;
	short 	vRefNum;
	long		parID;
	long		procID;
	
	/* Deactivate current front window. SFPut/GetFile sends an activate event
		when it quits but not when it opens ! */
	HandleActivates(FrontWindow(),0);
	GetIndString(prompt,OTHERS,PROMPTSTR);
	CenterSFDlg(SAVEENTRY_DLG,&where);
	lbSkipButton = FALSE;
	SFPPutFile(where, prompt, oldFName, (ProcPtr)myExportDlgHook,
		&reply,SAVEENTRY_DLG,NULL);
	
	if (reply.good) {
		GetWDInfo(reply.vRefNum,&vRefNum, &parID, &procID);
		CloseWD(reply.vRefNum);
		MakeFSS(vRefNum,reply.fName,parID,theFSS);
		return(OK);
	}
	else return(Cancel);
}

/**************************************
*	Dialog hook routine to initialise file dialog and to handle skip button
*	Return value:	theItem
*/

static pascal short myExportDlgHook(short theItem, DialogPtr myDialog)
{
	Str255	str;
	
	switch(theItem) {
		/* initialisation of dialog (called first time before dialog is displayed) */
		case -1:
			if(gPrefs.format == STADEN_FORMAT || gPrefs.format == DNASTAR_FORMAT)
				ActivateButton(myDialog,SAVEALL_BUTTON, INACTIVE);
			else
				ActivateButton(myDialog,SAVEALL_BUTTON, ACTIVE);
				
			GetIndString(str,FORMAT_STRINGS,gPrefs.format);
			SetDlgText(myDialog, FORMAT_STR, str);
			break;
			
		case SKIP_BUTTON:
			lbSkipButton = TRUE;
			theItem=putCancel;
			break;
			
		case SAVEALL_BUTTON:
			lbSaveAllButton = TRUE;
			theItem = OK;
			break;
	}
	
	return(theItem);
}

