/*
*********************************************************************
*	
*	HitStorage.c
*	Handling of results storage and hitlist management
*
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/ 

#include <stdio.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "hitstorage.h"

#include "main.h"
#include "pstr.h"
#include "query.h"
#include "results.h"
#include "util.h"

/*
******************************* Prototypes ***************************
*/


/*
********************************* Globals *****************************
*/



/**************************************
*	Allocates a hitlist
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*	Side-effect:	Handle to the new block,
*/

Boolean NewHitlist(HitmapHdl hitmapHdl,short dbcode, HitlistHdl *new, short *nhits)
{
	register long	i,j,size;
	HitlistHdl		hlHdl;
	SignedByte		oldState;
	register long	*ptr;
	
	/* allocate empty hitlist */
	*new = (HitlistHdl)NewHandle(0L);
	if(*new != NULL) {
		hlHdl = *new;
		*nhits = 0;

/*		for (i=0; i<gDBInfo[dbcode].ename_nrec;++i) {*/
/*			if(BitTst(*hitmapHdl,i)) {*/
/*				++(*nhits);*/
/*				*/
/*				SetHandleSize( *new, *nhits * (long)sizeof(HitlistRec) );*/
/*				if( MemError() != noErr ) {*/
/*					DisposHandle((Handle)*new);*/
/*					return( ErrorMsg(ERR_HITLISTEXT,NULL) );*/
/*				}*/
/*				*/
/*				(**new)[*nhits-1].ename_rec = i;*/
/*				SetSelectState(*new,*nhits-1,FALSE);*/
/*			}*/
/*		}*/

		/* look through hitmap */
		oldState = MyHLock((Handle)hitmapHdl);
		ptr = *hitmapHdl;
		
		/* Assuming that hits are rare we do not test bit by bit, but check each long
			word in the hitmap, eg 32 bits in one go. Only if a long word is not zero,
			we check individual bits */
		size = GetHandleSize((Handle)hitmapHdl)/4;
		for (i=0; i<size && *nhits <= MAXHITS;++i,++ptr) {
			if(*ptr) {
				for(j=0;j<32;++j) {
					if(BitTst(ptr,j)) {
						/* for each hit we add a new record to the hitlist */
						++(*nhits);
			 			if( *nhits > MAXHITS )
			 				break;
						
						SetHandleSize( (Handle)*new, *nhits * sizeof(HitlistRec) );
						if( MemError() != noErr ) {
							DisposHandle((Handle)*new);
							HSetState((Handle)hitmapHdl,oldState);
							return( ErrorMsg(ERR_HITLISTEXT) );
						}
						
						(**new)[*nhits-1].ename_rec = 32L * i + j;
						SetSelectState(*new,*nhits-1,FALSE);
					}
				} /* end for */
				
			}
		}
		HSetState((Handle)hitmapHdl,oldState);
	}
	
	return( *new != NULL );
}

/**************************************
*	Allocate new hitmap. We always create multiples of "long", ie 32 bits.
*	Return value:	TRUE, if selected
*						FALSE, if not
*	Side-effect:	Handle to new block
*/

Boolean NewHitmap(HitmapHdl *new, short dbcode)
{
	register long	i;
	long				nlong;
	
	/* calculate number of long words necessary to hold hitmap. */
	nlong = (gDBInfo[dbcode].ename_nrec-1)/32 + 1;
	
	*new = (HitmapHdl) NewHandleClear((Size)nlong * 4L);
/*	if(*new) {
		for(i=0;i<nlong; ++i)
			(**new)[i] = 0L;
	}
*/
	
	return( *new != NULL );
}


/**************************************
*	Logical Or hitmap 1 with hitmap 2
*	Return value:	none
*	Side-effect:	Result of logical Or in hmHdl1
*/

void OrHitmaps(HitmapHdl hmHdl1, HitmapHdl hmHdl2)
{
	register long	i;
	long				size = GetHandleSize((Handle)hmHdl1)/4;
	
	if(hmHdl1 == hmHdl2)
		return;
		
	for(i=0;i<size;++i)
		(*hmHdl1)[i] |= (*hmHdl2)[i];
}


/**************************************
*	Logical And hitmap 1 with hitmap 2
*	Return value:	none
*	Side-effect:	Result of logical And in hmHdl1
*/

void AndHitmaps(HitmapHdl hmHdl1, HitmapHdl hmHdl2)
{
	register long	i;
	long				size = GetHandleSize((Handle)hmHdl1)/4;
	
	if(hmHdl1 == hmHdl2)
		return;

	for(i=0;i<size;++i)
		(*hmHdl1)[i] &= (*hmHdl2)[i];
}

/**************************************
*	Logical Not hitmap
*	Return value:	none
*	Side-effect:	Result of logical Not in hmHdl
*/

void NotHitmap(HitmapHdl hmHdl)
{
	register long	i;
	long				size = GetHandleSize((Handle)hmHdl)/4;
	
	if(hmHdl == NULL)
		return;

	for(i=0;i<size;++i)
		(*hmHdl)[i] = ~(*hmHdl)[i];
}