/*
*********************************************************************
*	
*	info.c
*	Information alert boxes
*		
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/ 

#include <stdio.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "info.h"

#include "util.h"
#include "environment.h"
#include "main.h"
#include "menus.h"
#include "pstr.h"

/*
******************************* Prototypes ***************************
*/

static StringPtr GetVersion(void);
static pascal void DrawInfoPopUp(WindowPtr wPtr, short itemNo);
static void UpdateInfoDlg(DialogPtr infoDlg, short dbcode);
static void HandleInfoPopUpMenu(DialogPtr myDialog);


/*
******************************* Global variables *********************
*/

static short gCurrentDB;

/**************************************
*	Draws About box
*/

void ShowAbout()
{
	EventRecord	myEvent;
	DialogPtr	myDialog;
	
	myDialog=GetNewCenteredDialog(ABOUT_DLG,0);
	SetDlgText(myDialog,ABOUT_VERSION_STR,GetVersion());
	ShowWindow(myDialog);
	DrawDialog(myDialog);
	while(!WaitNextEvent(keyDownMask|autoKeyMask|mDownMask,&myEvent,0L,NULL))
		;
	DisposDialog(myDialog);
}

/**************************************
*	Reads version number from resource file
*	Return value:	Pointer to Pascal string containing the version number
*/

static StringPtr GetVersion()
{
	short				curResFile;
	VersRecHndl		version;
	static Str255	versNum;

	/* store refnum of current resource file */
	curResFile = CurResFile();
	
	UseResFile(gSystem.appResRef);	
	version=(VersRecHndl)Get1Resource('vers',1);
	if(version != NULL) {
		pstrcpy(versNum,(**version).shortVersion);
		ReleaseResource((Handle)version);
	}
	else pstrcpy(versNum,"\p?");

	UseResFile(curResFile);	
	return(versNum);
}

/**************************************
*	Draws database information dialog
*/

void ShowDBInfo()
{
	DialogPtr	infoDlg;
	short			itemHit;
	
	CenterDA('DLOG',INFO_DLG,33);
	infoDlg=GetNewDialog(INFO_DLG,NULL,(WindowPtr)-1L);
	
	gCurrentDB = FirstValidDB();
	
	/* Set user items to draw frame and surround OK button */
	InstallUserItem(infoDlg,INFO_USRITEM1,-1,DrawFrame);
	InstallUserItem(infoDlg,INFO_USRITEM2,OK,DrawOKBoxRect);
	InstallUserItem(infoDlg,INFO_POPUP,-1,DrawInfoPopUp);

	UpdateInfoDlg(infoDlg,gCurrentDB);
	
	ShowWindow(infoDlg);
	DrawDialog(infoDlg);
	do {
		ModalDialog(NULL,&itemHit);
		if(itemHit == INFO_POPUP)
			HandleInfoPopUpMenu(infoDlg);
	} while (itemHit != OK);
	DisposDialog(infoDlg);
}

/**************************************
*	Draws a pop-up menu for database names
*/

static pascal void DrawInfoPopUp(WindowPtr wPtr, short itemNo)
{
   short			itemType;
   Handle		item;
   Rect			box;
   Str255 		text;
   GrafPtr		savePort;
   
	savePort = ChangePort(wPtr);
	
	/* draw popup-box frame*/
	GetDItem(wPtr,itemNo,&itemType,&item,&box);
	EraseRect(&box);
	InsetRect(&box,-1,-1);
	FrameRect(&box);
	MoveTo(box.left+15,box.bottom-5);
	GetItem (gMenu[DBNAMES], gCurrentDB + 1,text);
   DrawString(text);
	
	/*draw drop shadow */
	MoveTo(box.right,box.top+2);
	LineTo(box.right,box.bottom);
	LineTo(box.left+2,box.bottom);
	
	DrawTriangle(&box);
	
	SetPort(savePort);
}

/**************************************
*	Updates database information in info dialog
*/

static void UpdateInfoDlg(DialogPtr infoDlg, short dbcode)
{
	Str255 str1,str2;
	
	/* Set database release number and date */	
	sprintf((char *)str1,"%s (%s)",gDBInfo[dbcode].DBRelNum,gDBInfo[dbcode].DBRelDate);
	SetDlgText(infoDlg,INFO_RELEASE,c2pstr((char *)str1));

	/* Set # of entries in database to # of records in entry name index */
	NumToString(gDBInfo[dbcode].ename_nrec,str2);
	SetDlgText(infoDlg,INFO_ENTRIES,str2);
}

/**************************************
*	Handle mouse clicks in info pop-up menu
*/

static void HandleInfoPopUpMenu(DialogPtr myDialog)
{
   short		itemType;
   Handle	item;
   Rect		box, popRect;
   short		whichItem;
   short		i;
   GrafPtr	savePort;
	
	savePort = ChangePort(myDialog);
   GetDItem(myDialog,INFO_POPUP,&itemType,&item,&box);
   popRect = box;
	LocalToGlobal(&TopLeft(popRect));
	LocalToGlobal(&BottomRight(popRect));
		
	/* Disable databases not available */
	for(i=0;i<DB_NUM;++i) {
		if(gDBInfo[i].ndiv > 0)
			EnableItem(gMenu[DBNAMES],i + 1);
		else
			DisableItem(gMenu[DBNAMES],i + 1);
			
		SetItemMark(gMenu[DBNAMES],i + 1,' ');
	}
	SetItemMark(gMenu[DBNAMES],gCurrentDB + 1,checkMark);
		
	whichItem=PopUpMenuSelect(gMenu[DBNAMES],popRect.top,popRect.left,gCurrentDB + 1);
	if(whichItem=LoWord(whichItem)) {
		gCurrentDB = whichItem - 1;
	}

	UpdateInfoDlg(myDialog,gCurrentDB);
	
	InvalRect(&box);
	SetPort(savePort);
}
