/*
*********************************************************************
*	
*	locate.c
*	Locating index and data directories
*
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/

#include <stdio.h>
#include <string.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "locate.h"

#include "cd.h"
#include "events.h"
#include "getpath.h"
#include "main.h"
#include "menus.h"
#include "pref.h"
#include "pstr.h"
#include "script.h"
#include "util.h"

/*
******************************* Prototypes ****************************
*/

static void ShortenPath(StringPtr path, DialogPtr myDialog, short item);
static pascal Boolean FoldersOnly(ParmBlkPtr pb);
static pascal short myGetDirHook(short theItem, DialogPtr myDialog);
static pascal void DrawLocatePopUp(WindowPtr wPtr, short itemNo);
static void UpdateLocateDlg(DialogPtr infoDlg, DirSpec *tempInxSpec, DirSpec *tempDataSpec,
							short dbcode);
static void HandleLocatePopUpMenu(DialogPtr myDialog, DirSpec *tempInxSpec, DirSpec *tempDataSpec);


/*
********************************* Globals *****************************
*/

static Boolean	lbCurDirValid, lbCDROM;
static long		lgMyCurDir;
static SFReply	reply;
static short	gCurrentDB;


/**************************************
*	Preferences: change directory locations
*/

void Locate()
{
	EventRecord	myEvent;
	DialogPtr	myDialog;
	Boolean		bQuit = FALSE;
	short			itemHit;
	DirSpec		tempInxSpec[DB_NUM],tempDataSpec[DB_NUM];
	Boolean		bChanged[DB_NUM], bSomethingChanged = FALSE;
	OSErr			ret;
	short			i;
	
	myDialog=GetNewCenteredDialog(LOCATE_DLG,66);

	for(i=0;i<DB_NUM;++i) {
		pstrcpy(tempInxSpec[i].volName, gPrefs.inxDirSpec[i].volName);
		tempInxSpec[i].dirID = gPrefs.inxDirSpec[i].dirID;
		pstrcpy(tempDataSpec[i].volName, gPrefs.dataDirSpec[i].volName);
		tempDataSpec[i].dirID = gPrefs.dataDirSpec[i].dirID;
		bChanged[i] = FALSE;
	}
	
	gCurrentDB = FirstValidDB();
		
	/* install user items to draw frame and default button outline */
	InstallUserItem(myDialog,LOCATE_USRITEM1,-1,DrawFrame);
	InstallUserItem(myDialog,LOCATE_USRITEM2,OK,DrawOKBoxRect);
	InstallUserItem(myDialog,LOCATE_POPUP,-1,DrawLocatePopUp);

	UpdateLocateDlg(myDialog,tempInxSpec,tempDataSpec,gCurrentDB);

	ShowWindow(myDialog);
	while(!bQuit) {	/* display dialog using our standard filter proc */			
		ModalDialog((ProcPtr)myDialogFilter,&itemHit);
		switch(itemHit) {
			case OK:
			case Cancel:
				bQuit=TRUE;
				break;
			case LOCATE_POPUP:
				HandleLocatePopUpMenu(myDialog, tempInxSpec, tempDataSpec);
				break;
			case LOCATE_CHANGEINX_BTN:
				/* ask user for new directory location */
				if(DoGetDirectory(&tempInxSpec[gCurrentDB])) {
					UpdateLocateDlg(myDialog,tempInxSpec, tempDataSpec,gCurrentDB);
					bChanged[gCurrentDB] = TRUE;
				}
				break;
			case LOCATE_CHANGEDAT_BTN:
				/* ask user for new directory location */
				if(DoGetDirectory(&tempDataSpec[gCurrentDB])) {
					UpdateLocateDlg(myDialog,tempInxSpec, tempDataSpec,gCurrentDB);
					bChanged[gCurrentDB] = TRUE;
				}
				break;
		}
	}
	DisposDialog(myDialog);
		
	/* if user clicked OK and something has changed we copy new directory
		information to gPrefs */
	if(itemHit == OK) {
		bSomethingChanged = FALSE;
		for(i=0;i<DB_NUM;++i) {
			if(bChanged[i]) {
				bSomethingChanged = TRUE;
				pstrcpy(gPrefs.inxDirSpec[i].volName, tempInxSpec[i].volName);
				gPrefs.inxDirSpec[i].dirID = tempInxSpec[i].dirID;
				pstrcpy(gPrefs.dataDirSpec[i].volName, tempDataSpec[i].volName);
				gPrefs.dataDirSpec[i].dirID = tempDataSpec[i].dirID;
				gDBInfo[i].ndiv = 0;
			}
		}
		
		if(bSomethingChanged) {
			InitCD();
			ErrorMsg(ERR_PREFCHGWARN);
		}		
	}
}

/**************************************
*	Build a complete path from a DirSpec. If volume name is EOS this indicates
*	that CD-ROM should be used (default).
*/

Boolean GetDirPath(DirSpec *dirSpec,StringPtr path)
{
	Str31 cdStr;
	short vRefNum;
	char	fullPath[256];
	
	GetIndString(cdStr,OTHERS,CDROMPATH);
	pstrcpy(path,cdStr);

	if(*dirSpec->volName == EOS) {
		return(TRUE);
	}
	else {
		/* find volume number for a given volume name */
		if(!GetVRefNumFromName(dirSpec->volName,&vRefNum)) {
			return(FALSE);
		}
		else {
			/* get complete path from dir ID and volume name */
			if(!GetPathFromDirID(vRefNum,dirSpec->dirID,
									"\p",fullPath)) {
				return(FALSE);
			}
			else {
  				strcpy((char *)path,fullPath);
				c2pstr((char *)path);
				return(TRUE);
			}
		}
	}
}

/**************************************
*	Shorten a complete path to fit into a dialog static text item.
*	We first try the complete path. If it doesn't fit, we take the 
*	volume name, add a '' character and then as many subfolder names
*	as possible to fit into the given box. We will always show at least
*	two levels of the file system hierarchy, the volume name and the file name
*	or the directory name in case the path points to a directory.
*	This routine works for both, file and directory path specifications.
*	Return value: by side-effect: modified path in "path"
*/

static void ShortenPath(StringPtr path, DialogPtr myDialog, short item)
{
	short		kind;
	Handle	h;
	Rect		r;
	short		boxWidth;
	GrafPtr	savePort;
	Str255	newPath;
	char		topFolder[256], *p;
	short		depth,maxDepth;
	short		i;
	
	savePort = ChangePort(myDialog);
	
	GetDItem(myDialog,item,&kind,&h,&r);
	boxWidth = r.right - r.left;
	
	TruncString(boxWidth,path,smTruncMiddle);
	
/*
	p2cstr(path);
	strcpy((char *)newPath,(char *)path);
	|* for each directory/volume in path increase depth counter *|
	for(p=(char *)newPath,maxDepth=0;*p;++p) {	
		if(*p == ':')
			++maxDepth;
	}
	|* if last element in path is filename, add one level of depth *|
	if(*(p-1) != ':')			 
		++maxDepth;
		
	|* we will keep at least two levels in the path, therefore we need to do
		any modifications only if maxDepth is greater than 2 *|
	if(maxDepth > 2) {
		strcpy(topFolder,(char *)newPath);
		for(p=topFolder;*p != ':';++p)
			;
		*p = EOS;
		strcat(topFolder,"::");
	}
		
	c2pstr((char *)newPath);
	for(depth=2;depth < maxDepth && StringWidth(newPath) > boxWidth;++depth) {
		strcpy((char *)newPath,topFolder);
		for(p=(char *)path,i=0;i<depth;++p) {
			if(*p == ':')
				++i;
		}
		strcat((char *)newPath,p);
		c2pstr((char *)newPath);
	}
	
	pstrcpy(path,newPath);
*/

	SetPort(savePort);
}

/**************************************
*	Get new directory from user. We use a modified SFGetFile dialog,
*	modelled on Apple DTS SC.18. User can also choose "CD-ROM".
*	Return values:	TRUE, if user selected a directory
*						FALSE; if user cancelled
*/

Boolean DoGetDirectory(DirSpec *dirSpec)
{
	Str31 str;
	Point where;
	
	/* Deactivate current front window. SFPut/GetFile sends an activate event
		when it quits but not when it opens ! */
	HandleActivates(FrontWindow(),0);
	CenterSFDlg(DIRSELECT_DLG,&where);
	SFPGetFile(where,"\p",(ProcPtr)FoldersOnly,-1,NULL,
				(ProcPtr)myGetDirHook,&reply,DIRSELECT_DLG,NULL);
					
	if(lbCurDirValid) {	/* lbCurDirValid is TRUE, if user selected a directory */
		if(lbCDROM) {		/* lbCDROM is TRUE, if user selected CDROM */
			*dirSpec->volName = EOS;	
			dirSpec->dirID = 0;
		}
		else {
			/* Pick up volume vRefNum from LoMem variable and find volume name */
			if(!GetNameFromVRefNum(str,-SFSaveDisk))
				return(FALSE);
			else {
				pstrcpy(dirSpec->volName,str);
				dirSpec->dirID = lgMyCurDir;
			}
		}
		
		return(TRUE);
	}
	else return(FALSE);
}

/**************************************
*	Make sure we show only folders in the SFDialog list
*/

static pascal Boolean FoldersOnly(ParmBlkPtr pb)
{
	return((pb->fileParam.ioFlAttrib & 0x10) == 0);
}

/**************************************
*	Our hook routine to handle the non-standard buttons
*/

static pascal short myGetDirHook(short theItem, DialogPtr myDialog)
{
#define getDirButton 11
#define getDirNowButton 12
#define getCDButton 13

	switch(theItem) {
		/* initialisation of dialog
			(called first time before dialog is displayed) */
		case -1:
			lbCurDirValid = FALSE;
			lbCDROM = FALSE;
			break;
		case getDirButton:
			if(reply.fType != 0) {
				lgMyCurDir = reply.fType;
				lbCurDirValid = TRUE;
				theItem = getCancel;
			}
			break;
		case getDirNowButton:
			lgMyCurDir = CurDirStore; /* get dirID from LoMem variable */
			lbCurDirValid = TRUE;
			theItem = getCancel;
			break;
		case getCDButton:
			lbCDROM = TRUE;
			lbCurDirValid = TRUE;
			theItem = getCancel;
			break;
	}
	
	return(theItem);
}

/**************************************
*	Draws a pop-up menu for database names
*/

static pascal void DrawLocatePopUp(WindowPtr wPtr, short itemNo)
{
   short			itemType;
   Handle		item;
   Rect			box;
   Str255 		text;
   GrafPtr		savePort;
   
   savePort = ChangePort(wPtr);
   
	/* draw popup-box frame*/
	GetDItem(wPtr,itemNo,&itemType,&item,&box);
	EraseRect(&box);
	InsetRect(&box,-1,-1);
	FrameRect(&box);
	MoveTo(box.left+15,box.bottom-5);
	GetItem (gMenu[DBNAMES], gCurrentDB + 1,text);
   DrawString(text);
	
	/*draw drop shadow */
	MoveTo(box.right,box.top+2);
	LineTo(box.right,box.bottom);
	LineTo(box.left+2,box.bottom);
	
	DrawTriangle(&box);
	
	SetPort(savePort);
}

/**************************************
*	Updates path information in locate dialog
*/

static void UpdateLocateDlg(DialogPtr myDialog, DirSpec *tempInxSpec, DirSpec *tempDataSpec,
									short dbcode)
{
	Str255 dirPath;
	
	/* show current path to directory */
	GetDirPath(tempInxSpec+dbcode,dirPath);
	ShortenPath(dirPath,myDialog,LOCATE_INX_TXT);
	SetDlgText(myDialog,LOCATE_INX_TXT,dirPath);
	GetDirPath(tempDataSpec+dbcode,dirPath);
	ShortenPath(dirPath,myDialog,LOCATE_DATA_TXT);
	SetDlgText(myDialog,LOCATE_DATA_TXT,dirPath);
}

/**************************************
*	Handle mouse clicks in locate pop-up menu
*/

static void HandleLocatePopUpMenu(DialogPtr myDialog, DirSpec *tempInxSpec, DirSpec *tempDataSpec)
{
   short		itemType;
   Handle	item;
   Rect		box, popRect;
   short		whichItem;
   short		i;
   GrafPtr	savePort;
	
	savePort = ChangePort(myDialog);
   GetDItem(myDialog,LOCATE_POPUP,&itemType,&item,&box);
   popRect = box;
	LocalToGlobal(&TopLeft(popRect));
	LocalToGlobal(&BottomRight(popRect));
		
	/* Enable all databases */
	for(i=0;i<DB_NUM;++i) {
		EnableItem(gMenu[DBNAMES],i + 1);
		SetItemMark(gMenu[DBNAMES],i + 1,' ');
	}
	SetItemMark(gMenu[DBNAMES],gCurrentDB + 1,checkMark);
		
	whichItem=PopUpMenuSelect(gMenu[DBNAMES],popRect.top,popRect.left,gCurrentDB + 1);
	if(whichItem=LoWord(whichItem)) {
		gCurrentDB = whichItem - 1;
	}

	UpdateLocateDlg(myDialog,tempInxSpec,tempDataSpec,gCurrentDB);
	
	InvalRect(&box);
	SetPort(savePort);
}
