/*
*********************************************************************
*	
*	main.c
*	Initialization, cleaning up, main loop and some general routines
*		
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/ 

#include <AppleEvents.h>
#include <Balloons.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "main.h"

#include "appleevents.h"
#include "checkapp.h"
#include "cd.h"
#include "environment.h"
#include "events.h"
#include "menus.h"
#include "pref.h"
#include "print.h"
#include "query.h"
#include "results.h"
#include "search.h"
#include "util.h"
#include "window.h"

/*
******************************* Prototypes ***************************
*/

static void GetGlobalMouse(Point *mouse);
static void MaintainCursor(Point mouse, RgnHandle cursRgn);
static long GetSleep(void);
static void MainLoop(void);
static void InitDefaults(void);
static void PrepMenus(void);
static void PrepFonts(void);
static void PrepCursors(void);
static void InitThings(void);
static void CleaningWoman(void);


/*
********************************* Global variables *******************
*/

CursHandle	gCursor[8];							/* cursors used							*/
Boolean		gQuitApplication = FALSE;		/* flag for quitting the program		*/
DBInfo		gDBInfo[DB_NUM];					/* holds all information about databases */
IndexFiles	gFileList;							/* array of index file names */
Str255		gVolLabel;							/* default volume name */
FONTINFO		fMonaco,fSystem;					/* font information						*/
Boolean		gInBackground;						/* is set if were currently in
															background */

static CursHandle	editCursor;					/* actually not used */
															

/**************************************
*	Find current mouse position in global coordinates
*	Return value:	none
*	Side-effect:	mouse position in *mouse
*/

static void GetGlobalMouse(Point *mouse)
{
	EventRecord theEvent;
	
	OSEventAvail(0, &theEvent);		/* no need to actually *get* events */
	*mouse=theEvent.where;
	
	/* we could also call GetMouse plus LocalToGlobal, but that requires
		that a valid port is set */
}


/**************************************
*	Adjust cursor and recalculate cursor region for WNE.
*	(Not really needed, we could easily set cursor region to NULL, as long as
*  we don't have edit windows. But we include it to keep the structure of
*  this sample code more general to facilitate further extension.)
*	Return value:	none
*	Side-effect:	new region in cursRgn
*/

static void MaintainCursor(Point mouse, RgnHandle cursRgn)
{
	RgnHandle	arrowRgn;
	RgnHandle	editRgn;

	if (!gInBackground && !IsDAWindow(FrontWindow())) {
		editRgn=NewRgn();
		arrowRgn=NewRgn();	/* Now make it BIG */
		SetRectRgn(arrowRgn,-32768,-32768,32766,32766); /* see TN.SC002 */
		
		/* calculate editRgn */
		
		/* in this example we dont have any */
				
		/* now subtract editRgn from arrowRgn */
		
		DiffRgn(arrowRgn,editRgn,arrowRgn);
		
		/* set cursor accordingly and set cursor region for next WNE call */

		if(PtInRgn(mouse,editRgn)) {
			SetCursor(*editCursor);
			CopyRgn(editRgn,cursRgn);
		}
		else {
			InitCursor();
			CopyRgn(arrowRgn,cursRgn);
		}
		
		DisposeRgn(arrowRgn);
		DisposeRgn(editRgn);
	}
}


/**************************************
*	Calculate value of 'sleep' for WaitNextEvent
*	Return value:	sleep value
*/

static long GetSleep()
{
	return( (gInBackground) ? 0x7FFFFFFF : GetCaretTime() );
}


/**************************************
*	This is the main loop, calling WaitNextEvent 
*/

static void MainLoop()
{
	EventRecord	theEvent;
	RgnHandle	cursRgn=NewRgn();
	Point			mouse;
	Boolean		isEvent,isHandled;
	
	gInBackground=FALSE;
	gLastClick.when=0L;					/* initialize gLastClick */

	InitCursor();
	GetGlobalMouse(&mouse);
	MaintainCursor(mouse,cursRgn);
	MaintainMenus();
	
	if(gPrefs.startQOpen && !IsAppWindow(FrontWindow()))
		NewQuery();

	while(!gQuitApplication){	
		/* if an error occurred while we were in the background show it now */
		if( bErrorOccurred )
			ErrorMsg(0);
		
		isEvent = WaitNextEvent(everyEvent,&theEvent,GetSleep(),cursRgn);
		MaintainCursor(theEvent.where,cursRgn);
		
		if(theEvent.what == osEvt) /* work around bug in DialogSelect */
			DoEvent(theEvent);
		else {
			/* special action needed if event is for a non-modal dialog window */
			isHandled = FALSE;
			if(IsDialogEvent(&theEvent))
				isHandled = DoQueryEvt(theEvent);
				
			if(isEvent && !isHandled)
				DoEvent(theEvent);
		}
	}	/* end while */
	DisposeRgn(cursRgn);
}


/**************************************
*	Sets global variables to default values
*/

static void InitDefaults()
{
	register short i,j;
		
	for (i = 0; i < DB_NUM; ++i) {
		GetIndString(gDBInfo[i].DataDName,EMBLDATA + i,DATA_DIR);
		GetIndString(gDBInfo[i].InxDName,EMBLDATA + i,INDEX_DIR);
		gDBInfo[i].gDivNames = NULL;
		gDBInfo[i].ename_pnxHdl = NULL;
		for(j=0;j<FLD_NUM;++j)
			gDBInfo[i].inxInfo[j].pnxHdl = NULL;
		gDBInfo[i].ndiv = 0;
	}
	
	GetIndString(gFileList.divFName,INDEX_NAMES,DIV_TABLE);
	GetIndString(gFileList.enameIdxFName,INDEX_NAMES,ENAME_IDX);
	GetIndString(gFileList.enamePnxFName,INDEX_NAMES,ENAME_PNX);
	GetIndString(gFileList.briefIdxFName,INDEX_NAMES,BRIEF_IDX);
	GetIndString(gFileList.inxFNames[FLD_ACCNUM].trgFName,INDEX_NAMES,ACNUM_TRG);
	GetIndString(gFileList.inxFNames[FLD_ACCNUM].hitFName,INDEX_NAMES,ACNUM_HIT);
	GetIndString(gFileList.inxFNames[FLD_ACCNUM].pnxFName,INDEX_NAMES,ACNUM_PNX);
	GetIndString(gFileList.inxFNames[FLD_KEYWORD].trgFName,INDEX_NAMES,KEYWORD_TRG);
	GetIndString(gFileList.inxFNames[FLD_KEYWORD].hitFName,INDEX_NAMES,KEYWORD_HIT);
	GetIndString(gFileList.inxFNames[FLD_KEYWORD].pnxFName,INDEX_NAMES,KEYWORD_PNX);
	GetIndString(gFileList.inxFNames[FLD_FREETEXT].trgFName,INDEX_NAMES,FREETEXT_TRG);
	GetIndString(gFileList.inxFNames[FLD_FREETEXT].hitFName,INDEX_NAMES,FREETEXT_HIT);
	GetIndString(gFileList.inxFNames[FLD_FREETEXT].pnxFName,INDEX_NAMES,FREETEXT_PNX);
	GetIndString(gFileList.inxFNames[FLD_AUTHOR].trgFName,INDEX_NAMES,AUTHOR_TRG);
	GetIndString(gFileList.inxFNames[FLD_AUTHOR].hitFName,INDEX_NAMES,AUTHOR_HIT);
	GetIndString(gFileList.inxFNames[FLD_AUTHOR].pnxFName,INDEX_NAMES,AUTHOR_PNX);
	GetIndString(gFileList.inxFNames[FLD_TAXON].trgFName,INDEX_NAMES,TAXON_TRG);
	GetIndString(gFileList.inxFNames[FLD_TAXON].hitFName,INDEX_NAMES,TAXON_HIT);
	GetIndString(gFileList.inxFNames[FLD_TAXON].pnxFName,INDEX_NAMES,TAXON_PNX);
	GetIndString(gFileList.inxFNames[FLD_ENZNAME].trgFName,INDEX_NAMES,ENZNAME_TRG);
	GetIndString(gFileList.inxFNames[FLD_ENZNAME].hitFName,INDEX_NAMES,ENZNAME_HIT);
	GetIndString(gFileList.inxFNames[FLD_ENZNAME].pnxFName,INDEX_NAMES,ENZNAME_PNX);
	
	GetIndString(gVolLabel,OTHERS,VOL_LABEL);
	
	gResNo = gQueryNo = 0;
}


/**************************************
*	Prepare our menus
*/

static void PrepMenus()
{
	register short i;
	Str255 helpMenu;
	MenuHandle mh;
	
	gMenu[APPLE]=GetMenu(APPLE_M);		
	AddResMenu(gMenu[APPLE],'DRVR');		/* insert DAs in Apple-menu 			*/
	
	gMenu[FILE]=GetMenu(FILE_M);
	gMenu[EDIT]=GetMenu(EDIT_M);
	gMenu[OSTUFF]=GetMenu(OSTUFF_M);
	gMenu[PREFS]=GetMenu(PREFS_M);
	gMenu[WINDOWS]=GetMenu(WINDOWS_M);
	gMenu[CREATOR]= GetMenu(CREATOR_M);
	gMenu[FORMAT]=GetMenu(FORMAT_M);
	gMenu[XREF] = GetMenu(XREF_M);
	gMenu[DBFIELDS] = GetMenu(DBFIELDS_M);
	gMenu[DBNAMES] = GetMenu(DBNAMES_M);
	
	for(i=0;i <= WINDOWS;InsertMenu(gMenu[i++],0))
		;
	InsertMenu(gMenu[CREATOR],-1);
	InsertMenu(gMenu[FORMAT],-1);
	InsertMenu(gMenu[XREF],-1);
	InsertMenu(gMenu[DBFIELDS],-1);
	InsertMenu(gMenu[DBNAMES],-1);

	if(gPrefs.creator == OTHERS_I)
		UpdateOtherCreator();
	CheckItem(gMenu[CREATOR],gPrefs.creator,TRUE);
	
	CheckItem(gMenu[FORMAT],gPrefs.format,TRUE);

	/* if we run under System 7 we also insert help into Help menu */
	if(gSystem.bHasBalloonHelp) {
		if(HMGetHelpMenuHandle(&mh) == noErr) {
			GetItem(gMenu[APPLE],HELP_I,helpMenu);
			AppendMenu(mh,helpMenu);
		}
	}

	DrawMenuBar();
}

/**************************************
*	Get font information
*/

static void PrepFonts()
{
	TextSize(9);
	GetFNum("\pMonaco",&fMonaco.num);
	TextFont(fMonaco.num);
	GetFontInfo(&fMonaco.finfo);
	fMonaco.height=fMonaco.finfo.ascent+
						fMonaco.finfo.descent+
	               fMonaco.finfo.leading;
	TextSize(12);
	TextFont(fSystem.num=0);
	GetFontInfo(&fSystem.finfo);
	fSystem.height=fSystem.finfo.ascent+
						fSystem.finfo.descent+
	               fSystem.finfo.leading;
}


/**************************************
*	Read in rotating cursor resources
*/

static void PrepCursors()
{
	register short i;
	
	editCursor = GetCursor(iBeamCursor);
   gCursor[0] = GetCursor(watchCursor);
   for(i=1;i<8;++i)
   	gCursor[i] = GetCursor(256+i);

   for(i=0;i<8;++i)
   	LockHandleHigh((Handle)gCursor[i]);
}


/**************************************
*	Initialisation
*/

static void InitThings()
{
	register short	i;
	EventRecord		event;
	
	MaxApplZone();
	InitGraf(&thePort);
	
	for(i=0;i<10;++i)
		MoreMasters();						/* extra pointer blocks at  bottom of  heap */
	
	InitFonts();							/* startup the font manager 		*/
   InitWindows();                   /* startup the window manager 	*/
   InitMenus();                     /* startup the menu manager 		*/
   TEInit();                        /* startup the text edit manager	*/
   InitDialogs(NULL);               /* startup the dialog manager 	*/

	/*
		This next bit of code is necessary to allow the default button of our
		alert be outlined.
	*/
	 
	for (i = 0; i < 3; ++i)
		EventAvail(everyEvent, &event);

   CheckEnvironment();					/* look for system version 6 	*/
	PrepCursors();							/* prepare cursors 				*/
	GetPrefs();
	InitDefaults();						/* set default values			*/
	PrepFonts();
	PrepWindows();
 	PrepMenus();							/* prepare menus					*/
 	PrepPrint();							/* prepare printing */
 	PrepSearch();
	InitCD();								/* look for CD-ROM drive		*/
	if(gSystem.bHasAppleEvents)
 		InitAppleEvents();

}


/**************************************
*	Tidy up
*/

static void CleaningWoman()
{
	register short i,j;
	
	/* release any previously occupied memory */
	for (i = 0; i < DB_NUM; ++i) {
		if(gDBInfo[i].gDivNames != NULL) {
			DisposHandle((Handle)gDBInfo[i].gDivNames);
			gDBInfo[i].gDivNames = NULL;
		}
		if(gDBInfo[i].ename_pnxHdl != NULL) {
			DisposHandle(gDBInfo[i].ename_pnxHdl);
			gDBInfo[i].ename_pnxHdl = NULL;
		}
		for(j = 0; j<FLD_NUM; ++j) {
			if(gDBInfo[i].inxInfo[j].pnxHdl != NULL) {
				DisposHandle(gDBInfo[i].inxInfo[j].pnxHdl);
				gDBInfo[i].inxInfo[j].pnxHdl = NULL;
			}
		}
	}
	
   for(i=0;i < 8;++i)
   	HUnlock((Handle)gCursor[i]);
}


/**************************************
*	Main
*/

void main()
{
	InitThings();								/* prepare everything necessary			*/
	CheckAppFiles();
	MainLoop();									/* here we do the work						*/
	CleaningWoman();							/* tidy up										*/
}