/*
*********************************************************************
*	
*	Pnx.c
*	Database preindex handling
*
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "pnx.h"

#include "main.h"
#include "util.h"

/*****
#include "bsearch.h"
#include "entry.h"
#include "events.h"
#include "pstr.h"
#include "search.h"
*****/

/*
******************************* Prototypes ****************************
*/

static long PreIndex_BSearch(Handle hdl, void *key, Size size, u_long nrec,
							Boolean *take2);
static void FindBucketLimits(long trgNRec, long nbuckets, Boolean take2, long whichbucket,
					long *min, long *max);
static short pnx_compare(char *key,char *rec);


/**************************************
*	Load content of preindex into memory.
*/

Handle LoadPreIndex(short fd)
{
	long filesize;
	Handle pnxHdl;
	OSErr ret;
	
	pnxHdl = NULL;
	
	ret = GetEOF(fd,&filesize);
	if(ret == noErr) {
		filesize -= sizeof(Header);
		pnxHdl = NewHandle(filesize);
		if(pnxHdl != NULL) {
			HLock(pnxHdl);
			ret = FSRead(fd,&filesize,*pnxHdl);
			HUnlock(pnxHdl);
			if(ret) {
				DisposHandle(pnxHdl);
				pnxHdl = NULL;
			}
		}
	}
	
	return(pnxHdl);
}

Boolean CheckPreIndex(Handle pnxHdl, Size pnxSize, u_long pnxNRec, long trgNRec,
					char *entryName, long *min, long *max)
{
	long whichbucket;
	Boolean take2;

	if(pnxHdl == NULL)
		return(FALSE);			/* preindex not available */
	
	whichbucket = PreIndex_BSearch(pnxHdl, (void *)entryName, pnxSize, pnxNRec,
									&take2);
	
	FindBucketLimits(trgNRec, (long)pnxNRec, take2, whichbucket, min, max);
	return(TRUE);
}

static long PreIndex_BSearch(Handle hdl, void *key, Size size, u_long nrec,
							Boolean *take2)
{
	register long	max;
	register long	min;
	register long	pos,hitPos;
	SignedByte		oldState;
	short			comp;
	Ptr				rec;
		
	oldState = MyHLock(hdl);
	
	max = (long)nrec - 1;
	min = 0L;
	comp = 0;
	
	/* binary search */
	
	do {
		pos=(min+max)/2;

		rec = *hdl + pos*size;
		
		if( (comp = pnx_compare(key,rec)) <= 0) { /* note: comparison uses strncmp,
																	not strcmp */
			max=pos-1;
		}
		else {
			min=pos+1;
		}
	} while(min <= max);
	
	/*	Special check here because of wildcard searches.
		Max points at the bucket with the highest value smaller than the key.
		But assume we are looking for CA* and bucket one starts with AA and bucket
		two starts with CAAA. Then max is bucket one, but we must look at bucket two,
		too, because bucket one may end with, eg BZ, so a search in bucket one alone
		would not find anything. This can only happen if first value in bucket max+1 is
		(partially) identical to key. In this special case we have to look at two
		buckets, otherwise one is enough.
	*/
	
	*take2 = FALSE;
	if(max < (long)nrec-1)
		if(pnx_compare(key,*hdl + (max+1)*size) == 0)
			*take2 = TRUE;
	
	HSetState(hdl,oldState);
	return(max);
}
	
static void FindBucketLimits(long trgNRec, long nbuckets, Boolean take2,
					long whichbucket,
					long *min, long *max)
{
	long bucketsize = (trgNRec-1)/nbuckets + 1;
	long nb;
	
	if (take2)
		nb = 2;
	else
		nb = 1;
		
	*min = bucketsize * whichbucket;
	*max = *min + nb * bucketsize - 1;
	
	if(*max >= trgNRec)
		*max = trgNRec - 1;
	if(*min < 0)
		*min = 0L;
}

/**************************************
*	Compare routine for preindex bsearch
*/

static short pnx_compare(char *key,char *rec)
{
	return( strncmp(key,rec,strlen(key)) );
}
