/*
*********************************************************************
*	
*	save.c
*	Saving window contents to disk
*
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/ 

#include <stdio.h>
#include <string.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "save.h"

#include "environment.h"
#include "export.h"
#include "pref.h"
#include "pstr.h"
#include "query.h"
#include "results.h"
#include "util.h"
#include "window.h"

/*
******************************* Prototypes ***************************
*/

static Boolean SaveResAs(WDPtr wdp);
static Boolean SaveRes(WDPtr wdp, FSSpecPtr theFSS);
static Boolean SaveResData(FSSpecPtr theFSS, ResultHdl resHdl);
static Boolean SaveResRsrc(FSSpecPtr theFSS, ResultHdl resHdl);
static Boolean SaveQueryAs(WDPtr wdp);
static Boolean SaveQuery(WDPtr wdp, FSSpecPtr theFSS);
static Boolean SaveQueryRsrc(FSSpecPtr theFSS, QueryHdl queryHdl);
static Boolean SaveEntry(WDPtr wdp, FSSpecPtr theFSS);
static Boolean SaveEntryData(FSSpecPtr theFSS, EntryRecHdl entryRecHdl);
static Boolean SaveEntryAs(WDPtr wdp);



/*
******************************** Globals *****************************
*/

/**************************************
*	Save routines dispatcher
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

Boolean DoSave(WDPtr wdp,Boolean save)
{
	Boolean ret=TRUE;
	
	if( wdp == NULL)
		return(FALSE);
		
	switch( ((WindowPeek)wdp)->windowKind) {
		case queryW:
			if(save && wdp->inited)
				/* if Save selected and if a file already exists */
				ret=SaveQuery(wdp, &wdp->theFSS);
			else
				ret=SaveQueryAs(wdp);
			break;
		case entryW:
			ret=SaveEntryAs(wdp);
			break;
		case resW:
			if(save && wdp->inited)
				/* if Save selected and if a file already exists */
				ret=SaveRes(wdp, &wdp->theFSS);
			else
				ret=SaveResAs(wdp);
			break;
		default: SysBeep(10);
	}
	return(ret);
}

/******* Save results ********/

/**************************************
*	Save results in a new file 
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean SaveResAs(WDPtr wdp)
{
	Boolean		ret=FALSE;
	FSSpec		theFSS;
		
	if(GetNewFilename(wdp->theFSS.name,&theFSS)) {
		ret=SaveRes(wdp,&theFSS);
		if( ret ) {
			DelWindowFromMenu((WindowPtr)wdp);
			wdp->theFSS = theFSS;
			SetWTitle((WindowPtr)wdp,theFSS.name);
			AddWindowToMenu(theFSS.name, (WindowPtr)wdp);
		}
	}
	return(ret);
}


/**************************************
*	Save result data to a file
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean SaveRes(WDPtr wdp, FSSpecPtr theFSS)
{
	ResultHdl	resHdl;
	Boolean		ret;
	
	resHdl = (ResultHdl)(wdp->userHandle);
	if(resHdl == NULL)
		return(FALSE);

	StartWaitCursor();
	if ( !SaveResData(theFSS,resHdl) )
		return(FALSE);
		
	ret = SaveResRsrc(theFSS,resHdl);
	
	FlushVol(NULL,theFSS->vRefNum);
	
	/* update window descr. record */
	if(ret) {
		wdp->dirty = FALSE;
		wdp->inited = TRUE;
	}
	InitCursor();
	return(ret);
}


/**************************************
* 	Create data fork of result file
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean SaveResData(FSSpecPtr theFSS,ResultHdl resHdl)
{
	short				output;
	long				count;
	CString80Hdl	bufHdl;
	SignedByte		oldState;
	short				hitPos, bufPos;

	/* Create and open a new file */
	if( CreateMacFile(theFSS,kApplSignature,kResFileType,TRUE) != noErr )
	   return(FALSE);

	if( OpenMacFile(theFSS,&output, TRUE) != noErr )
	  	return(FALSE);	
	
	/* Lock down result data */
	bufHdl = (**resHdl).descBufHdl;
	oldState=LockHandleHigh((Handle)bufHdl);
	
	/* now save hit by hit */
	StartWaitCursor();
	for(	hitPos = 0,bufPos = 0;
			hitPos < (**resHdl).nhits;
			++hitPos, ++bufPos) {
		RotateWaitCursor();
		/* refill buffer if necessary */
		if( hitPos < (**resHdl).buftop || hitPos >= (**resHdl).buftop + MAXBUFLINES) {
			if( !FillDEBuffer(resHdl,hitPos,FALSE) )
				break;
			bufPos = 0;
		}
		
		/* write output */
		count = (long) pstrlen((StringPtr)(*bufHdl)[bufPos]);
		if( WriteMacFile(output, &count, (*bufHdl)[bufPos]+1,theFSS->name, TRUE) != noErr ) 
   		break;
   	/* add newline */
   	count = 1L;
   	if( WriteMacFile(output,&count, "\r",theFSS->name,TRUE) != noErr)
   		break;
	}
	FSClose(output);
	HSetState((Handle)bufHdl,oldState);
	InitCursor();
	
	return( hitPos == (**resHdl).nhits );
}


/**************************************
*	Create resource fork of result file
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean SaveResRsrc(FSSpecPtr theFSS, ResultHdl resHdl)
{  
	short			refNum;
	HitmapHdl 	hitmapHdl;
	QueryHdl		queryHdl;
	OSErr			err;
	
	/* open resource fork			*/
   HCreateResFile(theFSS->vRefNum, theFSS->parID, theFSS->name);
   if(gSystem.bHasFSSpec)
   	refNum = FSpOpenResFile(theFSS,fsWrPerm);
   else
   	refNum = HOpenResFile(theFSS->vRefNum, theFSS->parID,
   							 theFSS->name,fsWrPerm);
	if(refNum == -1) {
		sprintf(gError,LoadErrorStr(ERR_OPENRES,FALSE),
					p2cstr(theFSS->name),ResError());
		c2pstr((char *)theFSS->name);
		return(ErrorMsg(0));
	}

	/* Store query */
	
	queryHdl = (**resHdl).queryHdl;
	AddResource((Handle)queryHdl,kQueryRsrcType,kStdRsrc,kQueryRsrcName);
	if( (err=ResError()) != noErr ) {
		DetachResource((Handle)queryHdl);
      CloseResFile(refNum);
		sprintf(gError,LoadErrorStr(ERR_WRITERES,FALSE),
					p2cstr(theFSS->name),err);
		c2pstr((char *)theFSS->name);
		return(ErrorMsg(0));
	}
	WriteResource((Handle)queryHdl);	/* and write it (we have to clear the
												change attr before a DetachResource() */
	if( (err=ResError()) != noErr ) {				/* error 						*/
		DetachResource((Handle)queryHdl);
      CloseResFile(refNum);
		sprintf(gError,LoadErrorStr(ERR_WRITERES,FALSE),
					p2cstr(theFSS->name),err);
		c2pstr((char *)theFSS->name);
		return(ErrorMsg(0));
	}
	/* Get back control over queryHdl from resource mgr */
	DetachResource((Handle)queryHdl);

	/* Now store hitmap */
	
	hitmapHdl = (**resHdl).hitmapHdl;
	AddResource((Handle)hitmapHdl,kHitmapRsrcType,kStdRsrc,kHitmapRsrcName);
	if( (err=ResError()) != noErr ) {
		DetachResource((Handle)hitmapHdl);
      CloseResFile(refNum);
		sprintf(gError,LoadErrorStr(ERR_WRITERES,FALSE),
					p2cstr(theFSS->name),err);
		c2pstr((char *)theFSS->name);
		return(ErrorMsg(0));
	}
	WriteResource((Handle)hitmapHdl);	/* and write it (we have to clear the
												change attr before a DetachResource() */
	if( (err=ResError()) != noErr ) {				/* error 						*/
		DetachResource((Handle)hitmapHdl);
      CloseResFile(refNum);
		sprintf(gError,LoadErrorStr(ERR_WRITERES,FALSE),
					p2cstr(theFSS->name),err);
		c2pstr((char *)theFSS->name);
		return(ErrorMsg(0));
	}
	/* Get back control over hitmapHdl from resource mgr */
	DetachResource((Handle)hitmapHdl);

	CloseResFile(refNum);
	return(TRUE);
}


/********* Save query **********/

/**************************************
*	Save query in a new file 
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean SaveQueryAs(WDPtr wdp)
{
	Boolean	ret=FALSE;
	FSSpec	theFSS;
		
	if(GetNewFilename(wdp->theFSS.name,&theFSS)) {
		ret=SaveQuery(wdp,&theFSS);
		if( ret ) {
			DelWindowFromMenu((WindowPtr)wdp);
			wdp->theFSS = theFSS;
			SetWTitle((WindowPtr)wdp,theFSS.name);
			AddWindowToMenu(theFSS.name, (WindowPtr)wdp);
		}
	}
	return(ret);
}


/**************************************
*	Save query data to a  file
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean SaveQuery(WDPtr wdp, FSSpecPtr theFSS)
{
	QueryHdl		queryHdl;
	Boolean		ret;
	FInfo			fndrInfo;
	
	queryHdl = (QueryHdl)(wdp->userHandle);
	if(queryHdl == NULL)
		return(FALSE);

	UpdateQueryRec(wdp);
	
	StartWaitCursor();

	/* Delete any old version */
	DeleteMacFile(theFSS);
		
	ret = SaveQueryRsrc(theFSS,queryHdl);
	                    
	/* update Finder information */
	SetFileType(theFSS,kQryFileType,kApplSignature,TRUE);
	FlushVol(NULL,theFSS->vRefNum);
	
	/* update window descr. record */
	if(ret) {
		wdp->dirty = FALSE;
		wdp->inited = TRUE;
	}
	InitCursor();
	return(ret);
}


/**************************************
*	Create resource fork of query file
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean SaveQueryRsrc(FSSpecPtr theFSS, QueryHdl queryHdl)
{  
	short			refNum;
	OSErr			err;
	
	/* open resource fork			*/
   HCreateResFile(theFSS->vRefNum, theFSS->parID, theFSS->name);
   if(gSystem.bHasFSSpec)
   	refNum = FSpOpenResFile(theFSS,fsWrPerm);
   else
   	refNum = HOpenResFile(theFSS->vRefNum, theFSS->parID,
   								theFSS->name,fsWrPerm);
	if(refNum == -1) {
		sprintf(gError,LoadErrorStr(ERR_OPENRES,FALSE),
					p2cstr(theFSS->name),ResError());
		c2pstr((char *)theFSS->name);
		return(ErrorMsg(0));
	}

	/* Store query */
	
	AddResource((Handle)queryHdl,kQueryRsrcType,kStdRsrc,kQueryRsrcName);
	if( (err=ResError()) != noErr ) {
		DetachResource((Handle)queryHdl);
      CloseResFile(refNum);
		sprintf(gError,LoadErrorStr(ERR_WRITERES,FALSE),
					p2cstr(theFSS->name),err);
		c2pstr((char *)theFSS->name);
		return(ErrorMsg(0));
	}
	WriteResource((Handle)queryHdl);	/* and write it (we have to clear the
												change attr before a DetachResource() */
	if( (err=ResError()) != noErr ) {				/* error 						*/
		DetachResource((Handle)queryHdl);
      CloseResFile(refNum);
		sprintf(gError,LoadErrorStr(ERR_WRITERES,FALSE),
					p2cstr(theFSS->name),err);
		c2pstr((char *)theFSS->name);
		return(ErrorMsg(0));
	}
	/* Get back control over queryHdl from resource mgr */
	DetachResource((Handle)queryHdl);

	WriteNameStringRsrc(refNum);	/* write name string resource (for system 7.0) */
	
	CloseResFile(refNum);
	return(TRUE);
}


/********** Save entries ************/


/**************************************
*	Save entry in a new file
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean SaveEntryAs(WDPtr wdp)
{
	Str255	ename,oldFName;
	Boolean	ret=FALSE;
	EntryRecHdl entryRecHdl;
	FSSpec	theFSS;
	
	entryRecHdl = (EntryRecHdl)(wdp->userHandle);
	if( !entryRecHdl )
		return;

	GetWTitle((WindowPtr)wdp,ename);
	BuildFName((**entryRecHdl).dbcode,ename,oldFName);

	if(GetNewFilename(oldFName,&theFSS)) {
		ret=SaveEntry(wdp,&theFSS);
		if( ret ) {
			wdp->theFSS = theFSS;
		}
	}
	return(ret);
}

/**************************************
*	Save an entry
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*/

static Boolean SaveEntry(WDPtr wdp, FSSpecPtr theFSS)
{
	EntryRecHdl	entryRecHdl;
	Boolean		ret;
	
	entryRecHdl = (EntryRecHdl)(wdp->userHandle);
	if(entryRecHdl == NULL)
		return(FALSE);

	StartWaitCursor();
	if ( !SaveEntryData(theFSS,entryRecHdl) )
		return(FALSE);
	
	/* update window descr. record */
	wdp->dirty = FALSE;
	wdp->inited = TRUE;

	InitCursor();
	return(ret);
}


static Boolean SaveEntryData(FSSpecPtr theFSS,EntryRecHdl entryRecHdl)
{
	char			*inpBuf,*outBuf;
	short			input,output;
	Str255		divFName;
	Boolean		ret=TRUE;
	DialogTHndl	myHandle;
	DialogPtr	myDialog;
	OSType		fileType,fileCreator;
	
	if( entryRecHdl == NULL )
		return(FALSE);
		
	/* Allocate file buffers. We don't use temporary memory calls because we call
		WaitNextEvent during export */
	inpBuf=NewPtr((Size)FILEBUF);
	if(inpBuf)
		if( !(outBuf=NewPtr((Size)FILEBUF)) )
			DisposPtr((Ptr)inpBuf);
	if ( !inpBuf || !outBuf )
		return(ErrorMsg(ERR_MEMORY));

	if(gSystem.sysVersion >= 0x0700) {
		myHandle = (DialogTHndl)GetResource('DLOG',EXPORT_DLG);
		(**myHandle).procID = movableDBoxProc;
	}
	myDialog = GetNewCenteredDialog(EXPORT_DLG,50);
	DisableMenuBar();
	
	ParamText(theFSS->name,"\p","\p","\p");
	ShowWindow(myDialog);
	DrawDialog(myDialog);

	/* Open data file */
	pstrcpy(divFName,(**entryRecHdl).theFSS.name);
	ret = (OpenMacFileReadOnly(&(**entryRecHdl).theFSS,&input, TRUE) == noErr);
	if(!ret) goto end_proc;
	
	if( !(ret=
		( CreateMacFile(theFSS,gPrefs.creatorSig,kEntryFileType,TRUE) == noErr )) ) {
		FSClose(input);
		goto end_proc;
	}
		
	if( !(ret= (OpenMacFile(theFSS, &output, TRUE) == noErr))) {
		FSClose(input);
		goto end_proc;
	}
	
	ret=WriteDatabaseEntry(input,(**entryRecHdl).bufpart[0],(**entryRecHdl).seqline_offset,
									(**entryRecHdl).dbcode,divFName,
									inpBuf,outBuf,output,theFSS->name,myDialog);	
	FSClose(input);	
	FSClose(output);

	if(gPrefs.format == DNASTAR_FORMAT &&
		((**entryRecHdl).dbcode == DB_SWISS || (**entryRecHdl).dbcode == DB_EMBL)) {
		fileType = ((**entryRecHdl).dbcode == DB_SWISS) ? 'SEQA' : 'SEQU';
		fileCreator = 'ESEQ';
		SetFileType(theFSS,fileType,fileCreator,TRUE);
	}
		
	FlushVol(NULL,theFSS->vRefNum);

end_proc:
	DisposPtr((Ptr)inpBuf);
	DisposPtr((Ptr)outBuf);
	DisposDialog(myDialog);
	EnableMenuBar();
	return(ret);	
}
