/*
*********************************************************************
*	
*	Search.c
*	Lookup of records in target and hit file
*		
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/ 

#include <stdio.h>
#include <string.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "search.h"

#include "bsearch.h"
#include "hitstorage.h"
#include "main.h"
#include "pnx.h"
#include "pstr.h"
#include "util.h"

/*
******************************* Prototypes ***************************
*/

static Boolean MarkHits(short fd, StringPtr fName, TargetRec *myTrgRec, u_long nhits, HitmapHdl hitmapHdl);
static Boolean FindHits(short what, char *key, short dbcode, HitmapHdl hitmapHdl, u_short target_recsize, StringPtr targetFName, u_long target_nrec, short (*compare )(void *, void *), StringPtr hitFName, short maxkeylen);
static Boolean FindEntryname(char *key, short dbcode, HitmapHdl hitmapHdl);


/*
********************************* Global variables *********************
*/

char AndOperator[MAXOPLEN+1];
char OrOperator[MAXOPLEN+1];
char NotOperator[MAXOPLEN+1];


Boolean PrepSearch()
{
	Str255 temp;
	
	GetIndString(temp,BOOL_OP_STR,BOOLEAN_AND);
	p2cstr(temp);
	strncpy(AndOperator,(char *)temp,MAXOPLEN);
	AndOperator[MAXOPLEN] = EOS;

	GetIndString(temp,BOOL_OP_STR,BOOLEAN_OR);
	p2cstr(temp);
	strncpy(OrOperator,(char *)temp,MAXOPLEN);
	OrOperator[MAXOPLEN] = EOS;

	GetIndString(temp,BOOL_OP_STR,BOOLEAN_NOT);
	p2cstr(temp);
	strncpy(NotOperator,(char *)temp,MAXOPLEN);
	NotOperator[MAXOPLEN] = EOS;
	
	return(FALSE); /* we'd better check for success */	
}


HitmapHdl DoSearch(short what, StringPtr key, short dbcode)
{
	HitmapHdl	masterHitmapHdl,hitmapHdl;
	Boolean		ret;
	StringPtr	targetFName,hitFName;
	u_long		target_nrec;
	u_short		target_recsize;
	short			maxkeylen;
	char			str[256],*posOr,*posAnd,*posStop,*token;
	Boolean		done;
	Boolean		bNot;
	short			nextOp,prevOp;
	long			offset;

	pstrcpy((StringPtr)str,(StringPtr)key);
	UprString((StringPtr)str,FALSE);
	rtrim(p2cstr((StringPtr)str));
	
	switch(what) {
		case FLD_ENAME:
			break;
		case FLD_ACCNUM:
		case FLD_KEYWORD:
		case FLD_FREETEXT:
		case FLD_AUTHOR:
		case FLD_TAXON:
		case FLD_ENZNAME:
			targetFName 	= gFileList.inxFNames[what].trgFName;
			hitFName			= gFileList.inxFNames[what].hitFName;
			target_nrec 	= gDBInfo[dbcode].inxInfo[what].trg_nrec;
			target_recsize = gDBInfo[dbcode].inxInfo[what].trg_recsize;
			maxkeylen		= gDBInfo[dbcode].inxInfo[what].trg_recsize-2*sizeof(long);
			break;
		default:
			return(NULL);
	}
			
	masterHitmapHdl = NULL;
	token = str;
	done = FALSE;
	nextOp = prevOp = BOOLEAN_NOOP;
	
	/* break up subquery */
	while(!done && *token != EOS) {
		bNot = FALSE;
		posOr = posAnd = NULL;
			
		/* look for next AND/OR */
		posOr = strstr(token,OrOperator);
		posAnd = strstr(token,AndOperator);
		if(posOr == posAnd)
			nextOp = BOOLEAN_NOOP;
		else {
			if(posOr == NULL)
				nextOp = BOOLEAN_AND;
			else if(posAnd == NULL)
				nextOp = BOOLEAN_OR;
			else if(posAnd < posOr)
				nextOp = BOOLEAN_AND;
			else
				nextOp = BOOLEAN_OR;
		
			if(nextOp == BOOLEAN_AND)
				posStop = posAnd;
			else
				posStop = posOr;

			*posStop = EOS;
		}
						
		/* allocate new hitmap */
		if( !NewHitmap(&hitmapHdl,dbcode) ) {
			ErrorMsg(ERR_MEMORY);
			return(masterHitmapHdl);
		}
		if (masterHitmapHdl == NULL )				/* make first hitmap master hitmap */
			masterHitmapHdl = hitmapHdl;
			
		ltrim(token);
		rtrim(token);
		
		/* NOT operator ? */
		if(!strncmp(token,NotOperator,strlen(NotOperator))) {
			bNot = TRUE;
			token += strlen(NotOperator);
			ltrim(token);
		}
		
		/* search for key, populate hitmap with hits */
		if (what == FLD_ENAME)
			ret=FindEntryname(token,dbcode,hitmapHdl);
		else
			ret=FindHits(what, token,dbcode,hitmapHdl,
						target_recsize,targetFName,target_nrec,
						general_compare,hitFName,maxkeylen);
						
		if(!ret) { /* not found or error */
			DisposHandle((Handle)hitmapHdl);
			if(masterHitmapHdl == hitmapHdl)	/* first query */
				masterHitmapHdl = NULL;
			else if(prevOp == BOOLEAN_AND) {	/* destroy previous results if
													   previous operator was AND and
													   nothing was found in new query */
				DisposHandle((Handle)masterHitmapHdl);
				masterHitmapHdl = NULL;
			}
			if(prevOp == BOOLEAN_AND || nextOp == BOOLEAN_AND)
				done = TRUE;
		}
		else {
			if(bNot)
				NotHitmap(hitmapHdl);
				
			if(prevOp != BOOLEAN_NOOP) {
				if(prevOp == BOOLEAN_OR)
					OrHitmaps(masterHitmapHdl,hitmapHdl);
				else
					AndHitmaps(masterHitmapHdl,hitmapHdl);
					
				DisposHandle((Handle)hitmapHdl);
			}
		}
		
		if(nextOp == BOOLEAN_NOOP)
			done = TRUE;
		else {
			if(nextOp == BOOLEAN_AND)
				offset = (long)strlen(AndOperator);
			else
				offset = (long)strlen(OrOperator);
				
			token = posStop + offset;
			
			if(masterHitmapHdl != NULL)
				prevOp = nextOp;
			else
				prevOp = BOOLEAN_NOOP;	/* in case an AND operation failed, we
													reset prevOp */
		}
	}

	return(masterHitmapHdl);
}


/**************************************
*	Search index for key.
*	We perform a binary search on CD.
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

static Boolean FindHits(short what, char *key, short dbcode, HitmapHdl hitmapHdl,
					  u_short target_recsize,StringPtr targetFName,u_long target_nrec,
					  short (*compare)(void *,void *),
					  StringPtr hitFName, short maxkeylen )
{
	char				mykey[256];
	TargetRec		*myTrgRec;
	u_long			pos,oldpos,n_hits;
	long				count;
	short				i;
	short				fd1,fd2;
	OSErr				err;
	Str255			fName1,fName2;
	Boolean			bWildCardSearch,bDone;
	FSSpec			theFSS;
	long				min,max;
	
	/* we pad search terms to maxlen with blanks, so we can do a direct comparison to
		records we read from index. If we didn't do so we could eg find a hit of
		"ebv" with something like "ebv123" if we did a strncmp(..,strlen(key)) or we
		would have to rtrim all records read from index in order to do a strcmp()
		(trailing blanks!)
		
		For wildcard searches we can now get rid of the trailing blanks
	*/

	strcpy(mykey,key);
	if(mykey[strlen(mykey)-1] == WILDCARD1) {
		bWildCardSearch = TRUE;
		mykey[strlen(mykey)-1] = EOS;
	}
	else {
		bWildCardSearch = FALSE;
		rpad(mykey,' ',maxkeylen);
	}
	
	if(!CheckPreIndex(gDBInfo[dbcode].inxInfo[what].pnxHdl,
							(Size)gDBInfo[dbcode].inxInfo[what].pnx_recsize,
							gDBInfo[dbcode].inxInfo[what].pnx_nrec,
							(long)target_nrec, mykey,
							&min, &max)) {
		min = 0L;
		max = (long)target_nrec - 1;
	}
		
	/* allocate memory for target records */
	if ( (myTrgRec=(TargetRec *)NewPtrClear((Size)target_recsize)) == NULL)
		return(ErrorMsg(ERR_MEMORY));

	/* Open index  */
	pstrcpy(fName1,targetFName);
	MakeFSS(gDBInfo[dbcode].InxVRefNum,fName1,gDBInfo[dbcode].InxDirID,&theFSS);
	if( OpenMacFileReadOnly(&theFSS,&fd1,TRUE) != noErr ) {
		DisposPtr((Ptr)myTrgRec);
		return(FALSE);
	}

	/* do the binary search */
	if( !CDIndex_BSearch(mykey, bWildCardSearch,
								fName1,
								fd1,
								(long)target_recsize,
								min,max,
								compare,
								myTrgRec, &pos) ) {
		DisposPtr((Ptr)myTrgRec);
		FSClose(fd1);
		return(FALSE);
	}
	n_hits = ConvertLong(&myTrgRec->nhits);
	
	RotateWaitCursor();
	
	/* Open hit file */
	pstrcpy(fName2,hitFName);
	MakeFSS(gDBInfo[dbcode].InxVRefNum,fName2,gDBInfo[dbcode].InxDirID,&theFSS);
	if ( OpenMacFileReadOnly(&theFSS,&fd2,TRUE) != noErr ) {
		FSClose(fd1);
		DisposPtr((Ptr)myTrgRec);
		return(FALSE);
	}
	
	/* Mark hits in hitmap */
	if( !MarkHits(fd2,fName2,myTrgRec,n_hits,hitmapHdl) ) {
		FSClose(fd1);
		FSClose(fd2);
		DisposPtr((Ptr)myTrgRec);
		return(FALSE);
	}
		
	if( bWildCardSearch ) {
/*		
		Old wildcard search method:
		After we found a matching record during the binary search, we step backwards
		as long as we find matching records. Then we do the same in the other direction.

		oldpos = pos;				|* store current position *|
		
		|* Now go backwards in index, looking for matches *|
		bDone = FALSE;
		while( !bDone && pos > 0 ) {
			RotateWaitCursor();
			--pos;
			if( (err=SetFPos(fd1,fsFromStart,
						pos*(long)target_recsize+sizeof(Header))) != noErr) {
				sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
						p2cstr(fName1),err );
				c2pstr((char *)fName1);
				FSClose(fd1);
				FSClose(fd2);
				DisposPtr((Ptr)myTrgRec);
				return(ErrorMsg(0));
			}

			count = (long)target_recsize;
			if( ReadMacFile(fd1,&count,(void *)myTrgRec,fName1,TRUE) ) {
				FSClose(fd1);
				FSClose(fd2);
				DisposPtr((Ptr)myTrgRec);
				return(FALSE);
			}
				
			if(compare(mykey,myTrgRec))
				bDone = TRUE;
			else {
				n_hits = ConvertLong(&myTrgRec->nhits);

				if( !MarkHits(fd2,fName2,myTrgRec,n_hits,hitmapHdl) ) {
					FSClose(fd1);
					FSClose(fd2);
					DisposPtr((Ptr)myTrgRec);
					return(FALSE);
				}
			}
		}
		
		pos = oldpos;				|* jump to starting position *|
*/

		/* and go forwards in index, looking for matches */
		bDone = FALSE;
		while (!bDone && pos < target_nrec - 1) {
			RotateWaitCursor();
			++pos;
			if( (err=SetFPos(fd1,fsFromStart,
						pos*(long)target_recsize+sizeof(Header))) != noErr) {
				sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
						p2cstr(fName1),err );
				c2pstr((char *)fName1);
				FSClose(fd1);
				FSClose(fd2);
				DisposPtr((Ptr)myTrgRec);
				return(ErrorMsg(0));
			}

			count = (long)target_recsize;
			if( ReadMacFile(fd1,&count,myTrgRec,fName1,TRUE) ) {
				FSClose(fd1);
				FSClose(fd2);
				DisposPtr((Ptr)myTrgRec);
				return(FALSE);
			}
				
			if(compare(mykey,myTrgRec))
				bDone = TRUE;
			else {
				n_hits = ConvertLong(&myTrgRec->nhits);

				if( !MarkHits(fd2,fName2,myTrgRec,n_hits,hitmapHdl) ) {
					FSClose(fd1);
					FSClose(fd2);
					DisposPtr((Ptr)myTrgRec);
					return(FALSE);
				}
			}
		}
	}
	
	FSClose(fd1);
	FSClose(fd2);
	DisposPtr((Ptr)myTrgRec);
	return(TRUE);
}

/**************************************
*	Find entryname in index.
*	We perform a binary search on CD.
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

static Boolean FindEntryname(char *key, short dbcode, HitmapHdl hitmapHdl)
{
	char		entryName[255];
	u_long	pos,oldpos;
	Boolean	ret;
	EnameRec	*rec;
	Boolean	bWildCardSearch,bDone;
	short		fd;
	Str255	indexName;
	long		count;
	OSErr		err;
	FSSpec	theFSS;
	long		min,max;
	
	/* we pad entrynames to maxlen with blanks, so we can do a direct comparison to
		records we read from index. If we didn't do so we could eg find a hit of
		"ebv" with something like "ebv123" if we did a strncmp(..,strlen(key)) or we
		would have to rtrim all records read from index to in order to do a strcmp()
		(trailing blanks!)
	
	For wildcard searches we don't need the trailing blanks
	*/
	
	strcpy(entryName,key);
	if(entryName[strlen(entryName)-1] == WILDCARD1) {
		bWildCardSearch = TRUE;
		entryName[strlen(entryName)-1] = EOS;
	}
	else {
		bWildCardSearch = FALSE;
		rpad(entryName,' ',gDBInfo[dbcode].ename_len);
	}
	
	if(!CheckPreIndex(gDBInfo[dbcode].ename_pnxHdl,
							(Size)gDBInfo[dbcode].ename_pnxRecsize,
							gDBInfo[dbcode].ename_pnxNrec,
							(long)gDBInfo[dbcode].ename_nrec,
							entryName, &min, &max)) {
		min = 0L;
		max = (long)gDBInfo[dbcode].ename_nrec - 1;
	}
		
	/* allocate memory for ename records */
	if ( (rec=(EnameRec *)NewPtrClear((Size)gDBInfo[dbcode].ename_recsize)) == NULL)
		return(ErrorMsg(ERR_MEMORY));
		
	/* Open index  */
	pstrcpy(indexName,gFileList.enameIdxFName);
	MakeFSS(gDBInfo[dbcode].InxVRefNum,indexName,gDBInfo[dbcode].InxDirID,&theFSS);
	if( OpenMacFileReadOnly(&theFSS,&fd,TRUE) != noErr ) {
		DisposePtr((Ptr)rec);
		return(FALSE);
	}

	/* do the binary search */
	if( !CDIndex_BSearch(entryName, bWildCardSearch,
								indexName,
								fd,
								(Size)gDBInfo[dbcode].ename_recsize,
								min,max,
								ename_compare,
								rec, &pos) ) {
		FSClose(fd);
		DisposePtr((Ptr)rec);
		return(FALSE);
	}
	
	/* Mark hit in hitmap */
	BitSet((Ptr)*hitmapHdl,pos);
	
	if( bWildCardSearch ) {
/*		
		Old wildcard search method:
		After we found a matching record during the binary search, we step backwards
		as long as we find matching records. Then we do the same in the other direction

		oldpos = pos;				|* store current position *|
		
		|* Now go backwards in index, looking for matches *|
		bDone = FALSE;
		while(!bDone && pos > 0) {
			RotateWaitCursor();
			--pos;
			if( (err=SetFPos(fd,fsFromStart,
						pos*gDBInfo[dbcode].ename_recsize+sizeof(Header))) != noErr) {
				sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
						p2cstr(indexName),err );
				c2pstr((char *)indexName);
				FSClose(fd);
				DisposePtr((Ptr)rec);
				return(ErrorMsg(0));
			}

			count = gDBInfo[dbcode].ename_recsize;
			if( ReadMacFile(fd,&count,rec,indexName,TRUE) ) {
				FSClose(fd);
				DisposePtr((Ptr)rec);
				return(FALSE);
			}
				
			if(ename_compare(entryName,rec))
				bDone = TRUE;
			else
				BitSet((Ptr)*hitmapHdl,pos);
		}
		
		pos = oldpos;				|* jump to starting position *|
*/

		/* and go forwards in index, looking for matches */
		bDone = FALSE;
		while(!bDone && pos < gDBInfo[dbcode].ename_nrec - 1) {
			RotateWaitCursor();
			++pos;
			if( (err=SetFPos(fd,fsFromStart,
						pos*gDBInfo[dbcode].ename_recsize+sizeof(Header))) != noErr) {
				sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
						p2cstr(indexName),err );
				c2pstr((char *)indexName);
				FSClose(fd);
				DisposePtr((Ptr)rec);
				return(ErrorMsg(0));
			}

			count = gDBInfo[dbcode].ename_recsize;
			if( ReadMacFile(fd,&count,rec,indexName,TRUE) ) {
				FSClose(fd);
				DisposePtr((Ptr)rec);
				return(FALSE);
			}
				
			if(ename_compare(entryName,rec))
				bDone = TRUE;
			else
				BitSet((Ptr)*hitmapHdl,pos);
		}
	}
	
	FSClose(fd);
	DisposePtr((Ptr)rec);
	return(TRUE);
}
	

/**************************************
*	Read hit file and mark hits in hitmap
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

static Boolean MarkHits(short fd, StringPtr fName, TargetRec *myTrgRec, u_long nhits,
						HitmapHdl hitmapHdl)
{
	u_long	hitpos;
	OSErr		err;
	long		i;
	u_long	count,pos;
	u_long	*buf;
	
	count = sizeof(u_long) * nhits;
	
	if((buf = (u_long *)NewPtrClear((long)count)) == NULL)
		return(ErrorMsg(ERR_MEMORY));
	
	/* Go to first hit in hit file */
	hitpos = ConvertLong(&myTrgRec->hitPtr) - 1;
	if( (err=SetFPos(fd,fsFromStart,
				hitpos*sizeof(u_long) + sizeof(Header))) != noErr) {
		sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),
						p2cstr(fName),err );
		DisposePtr((Ptr)buf);
		return(ErrorMsg(0));
	}

	/* Now read "nhits" hits from there */
	if(ReadMacFile(fd,(long *)&count,buf,fName,TRUE) ) {
		DisposePtr((Ptr)buf);
		return(FALSE);
	}
	
	for(i=0; i < nhits; ++i) {
		/* Convert hit record to C-type array index */
		ConvertLong(buf + i);
		BitSet((Ptr)*hitmapHdl,--buf[i]);
	}
	
	DisposePtr((Ptr)buf);
	return(TRUE);
}



/**************************************
*	Compare routine for entryname bsearch
*/

short ename_compare(void *key,void *rec)
{
	return( strncmp((char *)key,(char *)rec,strlen((char *)key)) );
}


/**************************************
*	Compare routine for other bsearches
*/

short general_compare(void *key,void *rec)
{
	return( strncmp((char *)key,((TargetRec *)rec)->value,strlen((char *)key)));
}
