/*
*********************************************************************
*	
*	Util.c
*	Mixture of general purpose routines
*
*	Rainer Fuchs
*  EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*
*/

#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <Aliases.h>
#include <Script.h>
#include <Notification.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "util.h"

#include "environment.h"
#include "events.h"
#include "main.h"
#include "menus.h"
#include "pstr.h"
#include "window.h"

/*
******************************* Prototypes ***************************
*/

/* see TN 306 */

static pascal GetIconSuite(Handle *theIconSuite, short theResID, long
	selector) =
	{ 0x303C,0x0501,0xABC9};
static pascal DisposeIconSuite(Handle theIconSuite,Boolean disposeData) =
	{ 0x303C,0x0302,0xABC9};

static OSErr OpenMacFileMode(FSSpecPtr theFSS, 
									short *output, Boolean bShowErrMsg, SignedByte permission);
static void CreateAltFSS(FSSpecPtr theFSS, FSSpecPtr altFSS);

/*
******************************* Global variables *********************
*/

char gError[256];
Boolean bErrorOccurred = FALSE;

static short lgCurCursor=0;
static long	gCursorTime;

static Handle iconSuiteHdl;
static Boolean bUsedIconSuite;


/****************************** Byte manipulation utilities *******************
*	ISO format is least significant byte first, Mac format is reversed
*	order.
*/

/**************************************
*	Converts ISO 4-byte integer "val" to Mac long integer.
*	Return value:	the value of "val" in Mac format
*	Side-effect:	"val" becomes converted to Mac format
*/

u_long ConvertLong(u_long *val)
{
	register u_byte temp;
	
	temp=((u_byte *)val)[0];
	((u_byte *)val)[0]=((u_byte *)val)[3];
	((u_byte *)val)[3]=temp;
	
	temp=((u_byte *)val)[1];
	((u_byte *)val)[1]=((u_byte *)val)[2];
	((u_byte *)val)[2]=temp;
	
	return(*val);
}


/**************************************
*	Converts ISO 2-byte integer to Mac integer. in place and also
*	Return value:	the value of "val" in Mac format
*	Side-effect: 	"val" becomes converted to Mac format
*/

u_short ConvertShort(u_short *val)
{
	register u_byte temp;
	
	temp=((u_byte *)val)[0];
	*val <<= 8;
	((u_byte *)val)[1]=temp;
	
	return(*val);
}


/******************************* String manipulation utilities *******************/


/**************************************
*	Converts a C string "str" to uppercase.
*	Return value:	Pointer to the converted string
*	Side-effect: 	the converted string in "str"
*/

char * str2upper(char *str)
{
	if( !str )
		return(NULL);
		
	c2pstr(str);
	UprString((StringPtr) str,FALSE);
	p2cstr((StringPtr) str);
	
	return(str);
}


/**************************************
*	Removes trailing blanks from a string "str".
*	Return value:	Pointer to the converted string
*	Side-effect: 	the converted string in "str"
*/

char * rtrim(char *str)
{
	register short p;
	
	if (!str || *str == EOS)
		return(str);
		
	p=strlen(str)-1;
	while ( p >= 0 && isspace(str[p]) )
		--p;
		
	str[p + 1] = EOS;
	
	return( str );
}

/**************************************
*	Removes leading blanks from a string "str".
*	Return value:	Pointer to the converted string
*	Side-effect: 	the converted string in "str"
*/

char * ltrim(char *str)
{
	register char *p;
	
	if (!str || *str == EOS)
		return(str);
		
	p = str;
	while(*p && isspace(*p))
		++p;
		
	if(p != str)
		strcpy(str,p);
	
	return( str );
}

/**************************************
*	Pads "str" to "len" characters with "c". "str" must be long enough to hold "len"
*  characters!!!
*	Return value:	Pointer to the converted string
*	Side-effect: 	the converted string in "str"
*/

char * rpad(char *str,char c,short len)
{
	register short i;
	
	for(i=strlen(str); i < len; ++i)
			str[i] = c;
	str[len] = EOS;

	return(str);
}


/**************************************
*	Removes all blanks from "str".
*	Return value:	Pointer to converted string
*	Side-effect:	 the converted string in "str"
*/

char * compress(char *str)
{
	register char *s,*t=str;
	
	for(s=str;*s;++s)
		if( !isspace(*s) )
			*str++ = *s;

	*str=EOS;
	return(t);
}

/**************************************
*	Checks for EMBL-style line type identifiers
*	Return value:	TRUE, if line starts with "id",
						FALSE, if not
*/

Boolean linetype(char *line, char *id)
{
	return( strncmp(line,id,strlen(id)) == 0);
}


/************************* File handling utilities *********************************/


/**************************************
*	This routine is used to center an SFPutFile/SFGetFile dialog
*	Return value: 
*/

void CenterSFDlg(short which, Point *where)
{
	DialogTHndl	myHandle;
	short			width, height;
	short			mBarHeight=GetMBarHeight();			/* subtract menu bar height */
	
	/* get the resource and calculate width and height */	
	myHandle=(DialogTHndl)GetResource('DLOG',which);
	if(myHandle == NULL) {
		/* set default values acc. to IM-I */
		if(which = putDlgID) {
			height = 184;  width = 304;
		}
		else {
			height = 200; width = 384;
		}
	}
	
	else {
		height=(**myHandle).boundsRect.bottom-(**myHandle).boundsRect.top;
		width=(**myHandle).boundsRect.right-(**myHandle).boundsRect.left;
	}
	
	where->v=((screenBits.bounds.bottom-screenBits.bounds.top)-height)/2 + mBarHeight;
	where->h=((screenBits.bounds.right-screenBits.bounds.left)-width)/2;
}

/**************************************
*	Delete a Mac file
*	Return value:	noErr, if successful
*						OS error code, if error occurred
*/

OSErr DeleteMacFile(FSSpecPtr theFSS)
{
	OSErr err;
	
	err=MyResolveAlias(theFSS);
	if(gSystem.bHasFSSpec)
		err = FSpDelete(theFSS);
	else
		err = HDelete(theFSS->vRefNum, theFSS->parID, theFSS->name);
		
	return(err);
}


/**************************************
*	Create a new Mac file
*	Return value:	noErr, if successful
*						OS error code, if error occurred
*/

OSErr CreateMacFile(FSSpecPtr theFSS, OSType creator, OSType type, Boolean bShowErrMsg)
{
	OSErr err;
	
	/* Delete any old version */
	DeleteMacFile(theFSS);
	
	if(gSystem.bHasFSSpec)
		err = FSpCreate(theFSS,creator,type,smSystemScript);
	else
		err = HCreate(theFSS->vRefNum,theFSS->parID,theFSS->name,creator,type);
		
	if( err != noErr && bShowErrMsg) {
		sprintf(gError,LoadErrorStr(ERR_CREATEFILE,FALSE),p2cstr(theFSS->name),err);
		c2pstr((char *)theFSS->name);
	   ErrorMsg(0);
	}
	return(err);
}


/**************************************
*	Open a Mac file
*	Return value:	noErr, if successful
*						OS error code, if error occurred
*	Side-effect:	file id in "output"
*/

OSErr OpenMacFile(FSSpecPtr theFSS, short *output, Boolean bShowErrMsg)
{
	return(OpenMacFileMode(theFSS,output,bShowErrMsg,fsCurPerm));
}

/**************************************
*	Open a Mac file in READONLY mode.
*	This is useful for reading in the index file so that they can be moved to a shared
*	disk.
*	Return value:	noErr, if successful
*						OS error code, if error occurred
*	Side-effect:	file id in "output"
*/

OSErr OpenMacFileReadOnly(FSSpecPtr theFSS, short *output, Boolean bShowErrMsg)
{
	return(OpenMacFileMode(theFSS,output,bShowErrMsg,fsRdPerm));
}

/**************************************
*	Open a Mac file in a certain mode.
*	Return value:	noErr, if successful
*						OS error code, if error occurred
*	Side-effect:	file id in "output"
*/

static OSErr OpenMacFileMode(FSSpecPtr theFSS, short *output, Boolean bShowErrMsg, 
									SignedByte permission)
{
	OSErr err;
	HParamBlockRec myPB;
	FSSpec altFSS;
	
	if( (err=MyResolveAlias(theFSS)) == noErr ) {
		if(gSystem.bHasFSSpec) {
			err = FSpOpenDF(theFSS,permission,output);
		}
		else {
			err = HOpenDF(theFSS->vRefNum,theFSS->parID, theFSS->name, permission,
								output);
			if(err == paramErr)
				err = HOpen(theFSS->vRefNum,theFSS->parID, theFSS->name, permission,
								output);
		}			
	}

		
	if(err == fnfErr) {
		CreateAltFSS(theFSS,&altFSS);
		if( (err=MyResolveAlias(&altFSS)) == noErr ) {
			if(gSystem.bHasFSSpec)
				err = FSpOpenDF(&altFSS,permission,output);
			else {
				err = HOpenDF(altFSS.vRefNum,altFSS.parID, altFSS.name, permission,
									output);
				if(err == paramErr)
					err = HOpen(altFSS.vRefNum,altFSS.parID, altFSS.name, permission,
									output);
			}
		}
	}
	
	if( err != noErr && bShowErrMsg) {
		sprintf(gError,LoadErrorStr(ERR_OPENFILE,FALSE),p2cstr(theFSS->name),err);
   	c2pstr((char *)theFSS->name);
	  	ErrorMsg(0);
	}

	return(err);
}

/**************************************
*	Create an alternative FSSpec:
*	If name ends with ;1, remove ;1. If not, add ;1
*	Return value:	TRUE, if successful
*						FALSE, if not
*/

static void CreateAltFSS(FSSpecPtr theFSS, FSSpecPtr altFSS)
{
	BlockMove(theFSS,altFSS,sizeof(FSSpec));
	
	if(altFSS->name[*altFSS->name-1] == ';' && altFSS->name[*altFSS->name] == '1')
		*altFSS->name -= 2;
	else
		pstrcat(altFSS->name,"\p;1");
}


/**************************************
*	Write to a Mac file
*	Return value:	noErr, if successful
*						OS error code, if error occurred
*	Side-effect: 	number of bytes written successfully in "count"
*/

OSErr WriteMacFile(short output, long *count, void *what, StringPtr fName,
						Boolean bShowErrMsg)
{
	OSErr err;
	
	if( (err=FSWrite(output, count, what)) != noErr  && bShowErrMsg) {
		sprintf(gError,LoadErrorStr(ERR_WRITEFILE,FALSE),p2cstr(fName),err);
      c2pstr((char *)fName);
   	ErrorMsg(0);
 	}
 	
 	return(err);
}
 

/**************************************
*	Read from a Mac file
*	Return value:	noErr, if successful
*						OS error code, if error occurred
*	Side-effect:	number of bytes read successfully in "count"
*/

OSErr ReadMacFile(short input, long *count, void *what, StringPtr fName,
						Boolean bShowErrMsg)
{
	OSErr err;
	
	if( (err=FSRead(input, count, what)) != noErr && bShowErrMsg) {
		sprintf(gError,LoadErrorStr(ERR_READFILE,FALSE),p2cstr(fName),err);
      c2pstr((char *)fName);
   	ErrorMsg(0);
 	}
 	
 	return(err);
}


/**************************************
*	Ask user for new file name
*	Return value:	TRUE, if successful
*						FALSE, if error occurred
*	Side-effect:	new filename in newFName
*/

Boolean GetNewFilename(Str255 oldFName,FSSpecPtr theFSS)
{
	SFReply	reply;
	StandardFileReply sys7reply;
	Point		where;
	Str255	prompt;
	short		vRefNum;
	long		parID,procID;
	
	/* Deactivate current front window. SFPut/GetFile sends an activate event
		when it quits but not when it opens ! */
	HandleActivates(FrontWindow(),0);
	GetIndString(prompt,OTHERS,PROMPTSTR);
	if(gSystem.bHasNewSFCalls) {
		StandardPutFile(prompt,oldFName,&sys7reply);
		if(sys7reply.sfGood) {
			*theFSS = sys7reply.sfFile;
			return(TRUE);
		}
	}
	else {
		where.h=((screenBits.bounds.right-screenBits.bounds.left)-304)/2;
		where.v=((screenBits.bounds.bottom-screenBits.bounds.top)-184)/2+20;
		SFPutFile(where, prompt, oldFName, NULL,&reply);
		if (reply.good) {
			GetWDInfo(reply.vRefNum,&vRefNum, &parID, &procID);
			CloseWD(reply.vRefNum);
			MakeFSS(vRefNum,reply.fName,parID,theFSS);
			return(TRUE);
		}
	}
		
	return(FALSE);
}

/**************************************
*	Get real vRefNum from volume name (without colon!)
*/

Boolean GetVRefNumFromName(StringPtr vName, short *vRefNum)
{
	ParamBlockRec myPB;
	Str255 ioName;
	OSErr ret;
	
	pstrcpy(ioName,vName);
	pstrcat(ioName,"\p:");
	
	myPB.volumeParam.ioNamePtr = ioName;
	myPB.volumeParam.ioVRefNum = 0x8000;
	myPB.volumeParam.ioVolIndex = -1;
	
	if((ret = PBGetVInfo(&myPB,FALSE)) == noErr)
		*vRefNum = myPB.volumeParam.ioVRefNum;
		
	return(ret == noErr);
}

/**************************************
*	Get volume name from real vRefNum
*/

Boolean GetNameFromVRefNum(StringPtr vName, short vRefNum)
{
	ParamBlockRec myPB;
	Str255 ioName;
	OSErr ret;
	
	*vName = 0;
	
	myPB.volumeParam.ioNamePtr = vName;
	myPB.volumeParam.ioVRefNum = vRefNum;
	myPB.volumeParam.ioVolIndex = 0;
	
	return( (ret = PBGetVInfo(&myPB,FALSE)) == noErr);
}


/**************************************
*	Resolve aliases if Alias Manager and new File Manager calls are available.
*	Return value:	noErr, if successful
*						OS error code, if error occurred
*/

OSErr MyResolveAlias(FSSpecPtr theFSS)
{
	OSErr err;
	Boolean targetIsFolder,wasAliased;
	
	err = noErr;
	if(gSystem.bHasAliasMgr && gSystem.bHasFSSpec) {
		/* resolve alias */
		err=ResolveAliasFile(theFSS,TRUE,&targetIsFolder,&wasAliased);
		if(targetIsFolder)
			err=paramErr;	/* cannot open a folder */
	}
	
	return(err);
}

/**************************************
*	Sets file type to fileType and creator to fileCreator
*	Return value:	noErr, if successful
*						OS error code, if error occurred
*/

OSErr SetFileType(FSSpecPtr theFSS, OSType fileType, OSType fileCreator,
							Boolean bShowErrMsg)
{
	OSErr err;
	FInfo fndrInfo;
	
	if( (err=MyResolveAlias(theFSS)) == noErr ) {
		if(gSystem.bHasFSSpec)
			err = FSpGetFInfo(theFSS,&fndrInfo);
		else
			err = HGetFInfo(theFSS->vRefNum,theFSS->parID, theFSS->name, &fndrInfo);
			
		if(err == noErr) {
			fndrInfo.fdType = fileType;
			fndrInfo.fdCreator = fileCreator;
			if(gSystem.bHasFSSpec)
				err = FSpSetFInfo(theFSS,&fndrInfo);
			else
				err = HSetFInfo(theFSS->vRefNum,theFSS->parID, theFSS->name, &fndrInfo);
		}
	}
			
	if( err != noErr && bShowErrMsg) {
		sprintf(gError,LoadErrorStr(ERR_CREATEFILE,FALSE),p2cstr(theFSS->name),err);
   	c2pstr((char *)theFSS->name);
	  	ErrorMsg(0);
	}

	return(err);
}

OSErr MakeFSS (short vRefNum, StringPtr fName, long parID, FSSpecPtr theFSS)
{
	OSErr err;
	
	err = noErr;
	if(gSystem.bHasFSSpec) {
		err=FSMakeFSSpec(vRefNum,parID,fName,theFSS);
		if(err == fnfErr)
			err = noErr;
	}
	else {
		theFSS->vRefNum = vRefNum;
		theFSS->parID = parID;
		pstrcpy(theFSS->name,fName);
	}
	
	return(err);
}

/********************************** General utilities *****************************/

/**************************************
*	Changes GrafPort
*	Return value: previous GrafPort
*/

GrafPtr ChangePort(GrafPtr newPort)
{
	GrafPtr oldPort;
	
	GetPort(&oldPort);
	SetPort(newPort);
	
	return(oldPort);
}

/**************************************
*	Moves a relocatable block high up in memory (to prevent memory
*	fragmentation) and locks it.
*	Return value:	Old status of handle flags
*/

SignedByte LockHandleHigh(Handle theHandle)
{
	SignedByte	hstate;

	hstate = HGetState(theHandle);
	HLockHi(theHandle);
	return(hstate);
}

/**************************************
*	Locks a relocatable block.
*	Return value:	Old status of handle flags
*/

SignedByte MyHLock(Handle theHandle)
{
	SignedByte	hstate;

	hstate = HGetState(theHandle);
	HLock(theHandle);
	return(hstate);
}

#define TICK_PERIOD 120

/*************************************
*	Draw first of our wait cursors
*/

void StartWaitCursor()
{
	SetCursor(*gCursor[lgCurCursor=0]);
	gCursorTime = TickCount()+TICK_PERIOD;

}


/*************************************
*	Rotate wait cursor. Every TICK_PERIOD/60 seconds we change it. This way we
*	are somewhat independent of machine speed.
*/

void RotateWaitCursor()
{
	long now;
	
	if( (now=TickCount()) >= gCursorTime) {
		gCursorTime = now + TICK_PERIOD;			/* approx. every two seconds */
		++lgCurCursor;
		if(lgCurCursor == 8) lgCurCursor = 0;
		SetCursor(*gCursor[lgCurCursor]);
	}
}


/***************************** Dialog handling utilities ****************************/

/**************************************
*	Check for application window
*	Return value:	TRUE, if window belongs to application
*						FALSE, if not (or NULL was passed)
*/

Boolean IsAppWindow(WindowPtr wPtr)
{
	if ( wPtr == NULL )
		return(FALSE);
	else				/* application windows have windowKinds = userKind (8) */
		return (((WindowPeek) wPtr)->windowKind >= userKind ||
				  ((WindowPeek) wPtr)->windowKind == dialogKind);
}


/**************************************
*	Check for desk accessory window
*	Return value:	TRUE, if window belongs to accessory
*						FALSE, if not (or NULL was passed)
*/

Boolean IsDAWindow(WindowPtr wPtr)
{
	if ( wPtr == NULL )
		return(FALSE);
	else							/* DA windows have negative windowKinds */
		return (((WindowPeek) wPtr)->windowKind < 0);
}

/**************************************
*	Centers dialogs and alerts dependent on screen size
*	"what" is the resource type ('DLOG' or 'ALRT')
*	"id" is the resource ID
*	"offset" is the vertical offset from central position in percent (0 =center,
*	100 = top of screen)
*	Return value: none
*/

void CenterDA(ResType what,short id,short offset)
{
/*	Since the first field in both AlertTemplate and DialogTemplate is boundsRect,
	Alerts are simply treated as Dialogs...This is similar to WindowPtr vs.
	DialogPtr.
*/

	DialogTHndl	myHandle;
	short			top,height,screenHeight,width;
	short			mBarHeight=GetMBarHeight();			/* subtract menu bar height */

	/* get the resource and calculate width and height */
	
	myHandle=(DialogTHndl)GetResource(what,id);
	if(myHandle == NULL) return;
	
	height=(**myHandle).boundsRect.bottom-(**myHandle).boundsRect.top;
	width=(**myHandle).boundsRect.right-(**myHandle).boundsRect.left;
	
	/* center the window (taking into account menu bar height) */
	
	screenHeight = screenBits.bounds.bottom - screenBits.bounds.top - mBarHeight;
	top = (screenHeight - height)/2;
	top -= top * (offset/100.0);	/* Adjust for offset */
	top += screenBits.bounds.top + mBarHeight;
	if(top < screenBits.bounds.top + mBarHeight)
		top = screenBits.bounds.top + mBarHeight;
	(**myHandle).boundsRect.top = top;
	
	(**myHandle).boundsRect.bottom=(**myHandle).boundsRect.top+height;
	(**myHandle).boundsRect.left=
		((screenBits.bounds.right - screenBits.bounds.left) - width)/2;
	(**myHandle).boundsRect.right=(**myHandle).boundsRect.left+width;
}

/**************************************
*	Reads a dialog resource and centers the dialog dependent on screen size
*	"id" is the resource ID
*	"offset" is the vertical offset from central position in percent (0 =center,
*	100 = top of screen)
*	Return value: pointer to Dialog
*/

DialogPtr GetNewCenteredDialog(short id,short offset)
{
	CenterDA('DLOG',id,offset);
	return(GetNewDialog(id,NULL,(WindowPtr)-1));
}

/**************************************
*	Standard filter proc for dialogs.
*	Notice that we assume that default button is always item #1!
*	Return value:	TRUE, if we handled the event
*						FALSE, if not
*/

pascal Boolean myDialogFilter(DialogPtr myDialog,EventRecord *theEvent,
										short *itemHit)
{
	char charCode;
      
   /* first check for keydown events */
   if ((theEvent->what == keyDown) || (theEvent->what == autoKey)) {
      charCode=(char)(theEvent->message & charCodeMask);
      
      /* ESCape pressed (extended keyboard) or Cmd-Period ? */
   	if( (theEvent->message & keyCodeMask)>>8 == ESCAPE || CmdPeriod(theEvent) ) {
   		/* Fake click on Cancel button */
			FakeClick(myDialog,Cancel);	
			*itemHit=Cancel;
      	return(TRUE);
		}
   	
   	if(theEvent->modifiers & cmdKey) {
   		switch(charCode) {
          /* or keyboard equivalents to Edit menu items ?
          	(we support clipboard usage in dialogs!)			*/
            case 'v':
            case 'V':
            	TEFromScrap();
            	DlgPaste(myDialog);
      			return(TRUE);
            break;
            	
            case 'x':
            case 'X':
            	DlgCut(myDialog);
            	ZeroScrap();
            	TEToScrap();
      			return(TRUE);
            break;
 
            case 'c':
            case 'C':
            	DlgCopy(myDialog);
            	ZeroScrap();
            	TEToScrap();
      			return(TRUE);
            break;
            
            default:return(FALSE);
   		} /* end switch */
   	} /* end if */
   	
   	/* Return or Enter ? */
      if(charCode == 3 || charCode == 13) {
			FakeClick(myDialog,OK);	
         *itemHit=OK;
         return(TRUE);
		}
			
   	return(FALSE);
   } /* end if */
   return(FALSE);
}


/**************************************
*	Mark default button by drawing a thick border.
*/

pascal void DrawOKBoxRect(WindowPtr myWindow,short itemNo)
{
	short		kind;
	Handle	h;
	Rect		r;
	short		diameter;
	PenState	oldPenState;
   
	GetPenState(&oldPenState);					/* save old pen state */
	PenSize(3,3);
   
	GetDItem(myWindow,itemNo,&kind,&h,&r);		/* get rect of OK button */
	diameter=(r.bottom - r.top)/2 + 6;
	if(diameter < 16)
		diameter = 16;
	InsetRect(&r,-4,-4);							/* and draw border */
	FrameRoundRect(&r,diameter,diameter);
	SetPenState(&oldPenState);					/* restore pen state */
}

/**************************************
*	Draw a thick border around a user item.
*/

pascal void DrawFrame(WindowPtr myWindow,short itemNo)
{
	short		kind;
	Handle	h;
	Rect		r;
	PenState	oldPenState;
   
	GetPenState(&oldPenState);					/* save old pen state */
	PenSize(2,2);
   
	GetDItem(myWindow,itemNo,&kind,&h,&r);	/* get rect of item */
	InsetRect(&r,-2,-2);								/* and draw border */
	FrameRect(&r);
	SetPenState(&oldPenState);					/* restore pen state */
}

/**************************************
*	Set radio button or check box to "status"
*/

void SetRadioButton(DialogPtr myDialog, short item, short status)
{
	short		kind;
	Handle	h;
	Rect		r;
	
	GetDItem(myDialog,item,&kind,&h,&r);
	SetCtlValue((ControlHandle)h,status);
}

/**************************************
*	Get status of radio button or check box
*/

short GetRadioButton(DialogPtr myDialog, short item)
{
	short		kind;
	Handle	h;
	Rect		r;
	
	GetDItem(myDialog,item,&kind,&h,&r);
	return(GetCtlValue((ControlHandle)h));
}

/**************************************
*	Toggle status of radio button or check box
*/

void ToggleRadioButton(DialogPtr myDialog, short item)
{
	SetRadioButton(myDialog,item,!GetRadioButton(myDialog,item));
}

/**************************************
*	Select one from a group of radio buttons. Buttons must be numbered
*	consecutively in the rsrc file.
*/

void SelRadioButton(DialogPtr myDialog, short firstItem, short n,
							short whichItem)
{
	register short i;
	
	for(i=0; i<n; ++i)
		SetRadioButton(myDialog,firstItem+i,BTNOFF);
	
	SetRadioButton(myDialog,whichItem,BTNON);
}

/**************************************
*	Set the text of a dialog item
*/

void SetDlgText(DialogPtr myDialog, short item, StringPtr text)
{
	short		kind;
	Handle	h;
	Rect		r;
	
	GetDItem(myDialog,item,&kind,&h,&r);
	SetIText(h,text);
}

/**************************************
*	Get the text of a dialog item
*/

void GetDlgText(DialogPtr myDialog, short item, StringPtr text)
{
	short		kind;
	Handle	h;
	Rect		r;
	
	GetDItem(myDialog,item,&kind,&h,&r);
	GetIText(h,text);
}

/**************************************
*	Install a user item. If useRect != -1 then this procedure will set the rect of item
*	to the rect of the useRect control.
*/

void InstallUserItem(DialogPtr myDialog, short item, short useRect,
	pascal void (*userRoutine)(WindowPtr myWindow,short itemNo))
{
	short		kind;
	Handle	h;
	Rect		r;
	
	GetDItem(myDialog,item,&kind,&h,&r);
	if(useRect != -1)
		GetDItem(myDialog,useRect,&kind,&h,&r);
		
	SetDItem(myDialog,item,userItem,(Handle)userRoutine,&r);
}

/**************************************
*	(De)Activate a dialog button
*/

void ActivateButton(DialogPtr myDialog, short item, short what)
{
	short		kind;
	Handle	h;
	Rect		r;
	
	GetDItem(myDialog,item,&kind,&h,&r);
	HiliteControl((ControlHandle)h,what);								
}

/**************************************
*	Hilite a clicked button to show its activation
*/

void FakeClick(DialogPtr myDialog, short item)
{
	short				kind;
	ControlHandle	h;
	Rect				r;
	long				ignore;

	GetDItem(myDialog,item,&kind,(Handle *)&h,&r);
	HiliteControl(h,1);
	Delay(8,&ignore);
	HiliteControl(h,0);
}

/**************************************
*	Check for Cmd-Period according to TN 263
*/

Boolean CmdPeriod(EventRecord *theEvent)
{
#define kMaskModifiers 0xFE00			/* we need the modifiers without the cmd key for KeyTrans */
#define kMaskVirtualKey 0x0000FF00	/* get virtual key from event message for KeyTrans */
#define kMaskASCII1 0x00FF0000
#define kMaskASCII2 0x000000FF
#define kPeriod 0x2E
#define kShiftWord 8						/* shift the virt. key  mask it into the keyCode for
													KeyTrans */
													
	Boolean	fTimeToQuit;
	short		keyCode;
	long 		virtualKey,keyInfo,lowChar,highChar,state,keyCId;
	Handle	hKCHR;
	Ptr		KCHRPtr;
	
	fTimeToQuit = FALSE;
	
	if(theEvent->what == keyDown || theEvent->what == autoKey) {
		if (theEvent->modifiers & cmdKey ) {
			virtualKey = (theEvent->message & kMaskVirtualKey) >> kShiftWord;
			/* And out the the cmd key and Or in the virtual key */
			keyCode = (theEvent->modifiers & kMaskModifiers) | virtualKey;
			state = 0;
			
			hKCHR = NULL;
			KCHRPtr = (Ptr)GetEnvirons(smKCHRCache); /* Pre-System 7 will return NULL */
			
			if(KCHRPtr == NULL) {	/* System < 7 */
				keyCId = GetScript(GetEnvirons(smKeyScript),smScriptKeys);
				hKCHR = GetResource('KCHR',keyCId);
				KCHRPtr = *hKCHR;
			}
			
			if(KCHRPtr != NULL) {
				/* Don't bother locking since KeyTrans will never move memory */
				keyInfo = KeyTrans(KCHRPtr, keyCode, &state);
				if(hKCHR)
					ReleaseResource(hKCHR);
			}
			else
				keyInfo = theEvent->message;
				
			lowChar = keyInfo & kMaskASCII2;
			highChar = (keyInfo & kMaskASCII1) >>16;
			if(lowChar == kPeriod || highChar == kPeriod)
				fTimeToQuit = TRUE;
				
		}
	}
	
	return(fTimeToQuit);
}

/**************************************
*	Draws downwards triangle in pull-down menu
*/

void DrawTriangle(Rect *boxPtr)
{
	PolyHandle	downArrow;

	downArrow = OpenPoly();								/* build polygon for down-arrow */
		MoveTo(boxPtr->right-3,boxPtr->top+6);		/* location for 16-pixel-high box */
 		Line(-12,0);
		Line(6,6);
		Line(6,-6);
	ClosePoly();
	FillPoly(downArrow,black);							/* and draw arrow */
	KillPoly(downArrow);
}


/************************** Error messaging ***************************************/


/**************************************
*	Get a string from resource file from STR# Error messages.
*	Depending on value of 'pas',  a pascal (pas==TRUE) or a C string (pass==FALSE)
*	is returned.
*	Return value:	Pascal or C string
*/

char * LoadErrorStr(short index, Boolean pas)
{
	static Str255 theString;
		
	*theString = EOS;
	GetIndString(theString,ERROR_MSGS,index);
	if(pas) return((char *)theString);
	else return(p2cstr(theString));
}


/**************************************
*	Notify user by displaying an error alert
*
*	If 'index' is not 0, then a message is read from resource file.
*	If 'index' is 0, then gError is used instead. gError contains an error msg in C format.
*	We check whether we're in the background. If no, we simply display the error msg,
*	otherwise we only set the bErrorOccurred flag.
*	Return value:	FALSE
*/

Boolean ErrorMsg(short index)
{
	char *str;
	
	InitCursor();										/* back to arrow cursor */
	if(index) {
		/* get error message from resouce */
		strcpy(gError,LoadErrorStr(index,FALSE));
	}
   
   if(gInBackground) {
   	bErrorOccurred = TRUE;
   }
   else {
		ParamText(c2pstr(gError),"\p","\p","\p");
		CenterDA('ALRT',ERROR_ALRT,66);				/* Calculate position on screen */
		StopAlert(ERROR_ALRT,NULL);
   	*gError = EOS;
   	bErrorOccurred = FALSE;
	}
	
	return(FALSE);										/* always returns false */
}


/**************************************
*	Notify user by displaying an error alert, then exit to shell
*/

void FatalErrorMsg(short index)
{
	ErrorMsg(index);
	ExitToShell();
}

/**************************************
*	Follow Apple guidelines for inverting
*/

void DoInvertRect(Rect *r)
{
#define _HiliteMode 0x938

	BitClr((Ptr)_HiliteMode,pHiliteBit);
	InvertRect(r);
}

/**************************************
*	Read name string resource
*/

StringHandle GetNameStringRsrc()
{
	StringHandle	strHdl;
	short				curResFile;

	/* store refnum of current resource file */
	curResFile = CurResFile();

	UseResFile(gSystem.appResRef);	
	strHdl = GetString(kNameStringRsrcTmpl);
	DetachResource((Handle)strHdl);
	UseResFile(curResFile);
	
	return(strHdl);
}

/**************************************
*	Write name string resource to resource file
*/

OSErr WriteNameStringRsrc(short refNum)
{
	Handle			newNameHdl,oldNameHdl;
	OSErr				err;
	short				curResFile;
	
	err = noErr;
	
	if( (newNameHdl = (Handle)GetNameStringRsrc()) != NULL) {
		curResFile = CurResFile();
		UseResFile(refNum);
	
		/* delete any old incarnation of the name string resource */
		oldNameHdl = Get1Resource(kNameStringRsrcType,kNameStringRsrc);
		if(oldNameHdl != NULL) {
			RmveResource(oldNameHdl);
			UpdateResFile(refNum);
			DisposHandle(oldNameHdl);
		}
	
		/* add name string resource */
		AddResource(newNameHdl,
			kNameStringRsrcType,kNameStringRsrc,kNameStringRsrcName);
		
		if( (err=ResError()) == noErr ) {
			WriteResource(newNameHdl);
			err = ResError();
		}
		
		if(err != noErr) {
			DetachResource(newNameHdl);
			DisposHandle(newNameHdl);
		}
			
		UseResFile(curResFile);
	}
	
	return(err);
}



/**************************************
*	EventHandling during movable modal dialog
*	Return value: TRUE if user cancelled
*/

Boolean UserBreak(DialogPtr myDialog)
{
	EventRecord theEvent;
	DialogPtr	whichDialog;
	short			itemHit;
	WindowPtr	whichWindow;
	static long	ticks = 0;
	
	if(gInBackground || TickCount() - ticks > 30) {
		ticks = TickCount();
		if(WaitNextEvent(everyEvent,&theEvent,
								gInBackground ? 25 : 0,NULL)) {
			if(theEvent.what == osEvt) /* see "Zen and..." (bug in IsDialogEvent) */
				DoEvent(theEvent);
			else if(theEvent.what == keyDown || theEvent.what == autoKey) {
				if ( CmdPeriod(&theEvent)  ||
					( (theEvent.message & keyCodeMask) >> 8 == ESCAPE))
					return(TRUE);
			}
			else {
				if(IsDialogEvent(&theEvent)) {
					if(DialogSelect(&theEvent,&whichDialog,&itemHit))
						if(whichDialog == myDialog && itemHit == OK)
							return(TRUE);
				}
				else {
					switch(theEvent.what) {
						case updateEvt:
						case activateEvt:
							DoEvent(theEvent);
							break;
						case mouseDown:
							switch (FindWindow( theEvent.where, &whichWindow )) {
								case inSysWindow:
									SystemClick( &theEvent, whichWindow );
									break;
								case inDrag:
									if(whichWindow == myDialog)
										DragWindow(whichWindow,theEvent.where,&gDragRect);
									break;
									case inMenuBar:
								MenuSelect(theEvent.where);
									HiliteMenu(0);
									break;
								default:
									SysBeep(10);
									break;
							}
							break;
						default:
							break;
					}
				}
			}
		}
	}
	
	return(FALSE);
}

/**************************************
*	Disable menu bar before drawing movable modal dialog
*/

void DisableMenuBar()
{
	register short i;
	
	HiliteMenu(0);	
	for(i=0;i <= WINDOWS;DisableItem(gMenu[i++],0));
	DrawMenuBar();
}

/**************************************
*	reenable menu bar after "movable modal dialog"
*/

void EnableMenuBar()
{
	register short i;
	
	for(i=0;i <= WINDOWS;EnableItem(gMenu[i++],0));
	DrawMenuBar();
}

/**************************************
*	Simply looping until we are switched to foreground
*/

void WaitUntilForeground()
{
	EventRecord theEvent;
	
	while(gInBackground) {
		if(WaitNextEvent(everyEvent,&theEvent,0x7FFFFFFF,NULL)) {
			switch (theEvent.what) {
				case updateEvt:
				case activateEvt:
				case osEvt:
					DoEvent(theEvent);
					break;
				default:
					break;
			}
		}
	}
}

/**************************************
*	Install notification record in system queue
*/

void NotifyUser(NMRec *myNMPtr)
{
	OSErr err;
	
#define svAllAvailableData 0xffffffff

	iconSuiteHdl = NULL;
	bUsedIconSuite = TRUE;
	if(gSystem.sysVersion >= 0x0700)	/* Under System 7 use color icon suites
												*/
		err = GetIconSuite(&iconSuiteHdl,128,svAllAvailableData);
		
	if(gSystem.sysVersion < 0x0700 || err != noErr || iconSuiteHdl == NULL) {
		bUsedIconSuite = FALSE;
		iconSuiteHdl = GetResource('SICN',128);
	}
		
	myNMPtr->qType = nmType;
	myNMPtr->nmMark = 1;
	myNMPtr->nmIcon = iconSuiteHdl;
	myNMPtr->nmSound = (Handle)-1;
	myNMPtr->nmStr = NULL;
	myNMPtr->nmResp = (ProcPtr)NotifyResponse;
	
	NMInstall(myNMPtr);
}

/**************************************
*	Notification response procedure. We don't do anything
*/

pascal void NotifyResponse(NMRec *myNMPtr)
{
}

/**************************************
*	Removal of notification record from system queue
*/

void RemoveNotification(NMRec *myNMPtr)
{
	NMRemove(myNMPtr);
	
	if(bUsedIconSuite) {
		if(iconSuiteHdl)
			DisposeIconSuite(iconSuiteHdl,TRUE);
	}
	else
		if(iconSuiteHdl)
			ReleaseResource(iconSuiteHdl);
}

short FirstValidDB(void)
{
	register short i=0;

	/* find first valid database */
	for(i=0;i<DB_NUM;++i) {
		if(gDBInfo[i].ndiv > 0) {
			break;
		}
	}

	return(i);
}
