/*
*********************************************************************
*	
*	XRef.c
*	Handling of cross-references
*
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*
*	Copyright  1992 EMBL Data Library
*		
**********************************************************************
*	
*/ 

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "EMBL-Search.h"
#include "EMBL-Search.rsrc.h"

#include "xref.h"

#include "entry.h"
#include "hitstorage.h"
#include "main.h"
#include "menus.h"
#include "query.h"
#include "results.h"
#include "search.h"
#include "util.h"
#include "window.h"

/*
******************************* Prototypes ***************************
*/

static Boolean NewXRefBuf(XRefHdl *new);
static void RemoveXRefsFromMenu(void);


/*
******************************** Global variables *****************
*/



void ShowXrefList(WDPtr wdp,short dbcode)
{
	XRefHdl		xrefHdl;
	EntryRecHdl	entryRecHdl;
	short			nxref;
	char			key[PROSITEENTRYNAMELEN + 1];
	HitmapHdl	masterHmHdl,newHmHdl;
	QueryHdl		newQueryHdl;
	ResultHdl	resHdl;
	WDPtr			resWdp;
	short			w;
	register short i;
	SignedByte	oldState;
	Str255		title;
	char			temp[256];
	
	if(wdp == NULL || ((WindowPeek)wdp)->windowKind != entryW)
		return;
		
	if( (entryRecHdl = (EntryRecHdl)(wdp->userHandle)) == NULL )
		return;

	if( (xrefHdl = (**entryRecHdl).xrefHdl) == NULL )
		return;
		
	oldState = MyHLock((Handle)xrefHdl);
	
	nxref = (short)(GetHandleSize((Handle)xrefHdl)/sizeof(XRef));
	
	StartWaitCursor();

	masterHmHdl = NULL;
	for( i = 0; i < nxref; ++i ) {
		if((*xrefHdl)[i].dbcode == dbcode) {
			strncpy(key,(*xrefHdl)[i].primaryID,PROSITEENTRYNAMELEN);
			key[PROSITEENTRYNAMELEN] = EOS;
	
			/* search for key, populate hitlist with hits */
			newHmHdl = DoSearch((*xrefHdl)[i].keytype,c2pstr(key),dbcode);
	
			if(newHmHdl != NULL) {
				if(masterHmHdl == NULL)
					masterHmHdl = newHmHdl;
				else {
					OrHitmaps(masterHmHdl, newHmHdl);
					DisposHandle((Handle)newHmHdl);
				}
			}
		}
	}
	
	HSetState((Handle)xrefHdl,oldState);
	
	if(masterHmHdl == NULL) {
		return;
	}
	else {
		if( (w=GetFreeWindow()) == -1 ) {
			ErrorMsg(ERR_MAXWIN);
			return;
		}
		else
			wdp = &gWindows[w];

		resWdp = &gWindows[w];
		/* Create an empty query block */
		if( !NewQueryBlock(dbcode, &newQueryHdl, BOOLEAN_OR) ) {
			ErrorMsg(ERR_MEMORY);
			DisposeHandle((Handle)masterHmHdl);
			return;
		}
		/* get a new result record and initialise it */
		else {
			if( !InitResultRec(&resHdl,dbcode,masterHmHdl,newQueryHdl) ) {
				DisposHandle((Handle)masterHmHdl);
				DisposHandle((Handle)newQueryHdl);
				ErrorMsg(ERR_MEMORY);
			}
			else {
			 	if( (**resHdl).nhits > MAXHITS ) {
					DisposeResRec(resHdl);
					ErrorMsg(ERR_MAXHITSEXC);
				}
				else if( (**resHdl).nhits == 0 ) {
					DisposeResRec(resHdl);
					ErrorMsg(ERR_NOTFOUND);
				}
				else {
					/* Open window */
					GetIndString(title,OTHERS,RESTITLE);
					sprintf(temp,p2cstr(title),++gResNo);
	
					if( !NewResultWindow(w,resWdp,resHdl, c2pstr((char *)temp)) ) {
						DisposeResults(resWdp);
					} /* end if !NewResultsWindow() */
				}
			}	/* end else */
		}
	}	
}

/**************************************
*	When user selected an item from the XREF menu we search for
*	this entry and open a new window.
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

Boolean OpenXRefFromMenu(WDPtr wdp,short which)
{
	EntryRecHdl	entryRecHdl;
	XRefPtr		xrefPtr;
	SignedByte	oldState;
	short			dbcode,keytype;
	char			key[PROSITEENTRYNAMELEN + 1];

	entryRecHdl = (EntryRecHdl)wdp->userHandle;

	oldState = MyHLock((Handle)(**entryRecHdl).xrefHdl);
	xrefPtr = *(**entryRecHdl).xrefHdl;	
	dbcode = xrefPtr[which].dbcode;
	strncpy(key,xrefPtr[which].primaryID,PROSITEENTRYNAMELEN);
	key[PROSITEENTRYNAMELEN] = EOS;
	keytype=xrefPtr[which].keytype;
	HSetState((Handle)(**entryRecHdl).xrefHdl,oldState);
	
	return(OpenXRef(dbcode,key,keytype));
}
	
/**************************************
*	Acc# search or ename search for a xref and open a new window.
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

Boolean OpenXRef(short dbcode, char * key, short keytype)
{
	HitmapHdl	hitmapHdl;
	ResultHdl	resHdl;
	register short i;
	Boolean		ret;
	
	StartWaitCursor();
	
	/* search for key, populate hitlist with hits */
	hitmapHdl = DoSearch(keytype,c2pstr(key),dbcode);
	
	if(hitmapHdl == NULL) { /* not found or error */
		return( ErrorMsg(ERR_NOTFOUND) );
	}
	else {	/* found! */
		/* get a new result record and initialise it */
		if( !InitResultRec(&resHdl,dbcode,hitmapHdl,NULL) ) {
			DisposHandle((Handle)hitmapHdl);
			return( ErrorMsg(ERR_MEMORY) );
		}
		else {
			/* fill short description buffer */
			StartWaitCursor();
			ret = FillDEBuffer(resHdl,0,FALSE);
			InitCursor();
			if(ret) {
				for(i=0;i < (**resHdl).nhits; ++i) {
					if( (ret=NewEntryWindow(resHdl,i)) == FALSE )
						break;
				}
			}
			DisposeResRec(resHdl);
			return(ret);
		}
	}
}


Boolean IsXref(short dbcode, StringPtr line)
{
	char *l;
	Boolean foundXRef = FALSE;
	
	l = p2cstr(line);
	
	if( linetype(l,"DR") ) {
		switch( dbcode ) {
			/* SWISS-PROT: Pointers to EMBL and Prosite.dat */
			case DB_SWISS:
				if ( gDBInfo[DB_EMBL].ndiv > 0 && strstr(l,"EMBL") != NULL )
					foundXRef = TRUE;
				else if ( gDBInfo[DB_PROSITEDAT].ndiv > 0 && strstr(l,"PROSITE") != NULL )
					foundXRef = TRUE;
			break;
				
			/* EMBL: Pointers to SWISS-PROT */
			case DB_EMBL:
				if ( gDBInfo[DB_SWISS].ndiv > 0 && strstr(l,"SWISS-PROT") != NULL )
					foundXRef = TRUE;
				else if ( gDBInfo[DB_EPD].ndiv > 0 && strstr(l,"EPD") != NULL )
					foundXRef = TRUE;
				break;
				
			/* Prosite.dat: Pointers to SWISS-PROT. All DR lines are SWISS-PROT xrefs */
			case DB_PROSITEDAT:
				if(gDBInfo[DB_SWISS].ndiv > 0)
					foundXRef = TRUE;
				break;
				
			/* Enzyme: Pointers to SWISS-PROT. All DR lines are SWISS-PROT xrefs */
			case DB_ENZYME:
				if(gDBInfo[DB_SWISS].ndiv > 0)
					foundXRef = TRUE;
				break;
		}
	}
	else if ( dbcode == DB_PROSITEDAT && linetype(l,"DO")) {
		if(gDBInfo[DB_PROSITEDOC].ndiv > 0)
			foundXRef = TRUE;
	}
	else if ( dbcode == DB_PROSITEDOC && linetype(l,"{PS")) {
		if(gDBInfo[DB_PROSITEDAT].ndiv > 0)
			foundXRef = TRUE;
	}
	else if( dbcode == DB_EPD && (linetype(l,"FP") || linetype(l,"F-"))) {
		if(gDBInfo[DB_EMBL].ndiv > 0)
			foundXRef = TRUE;
	}

	c2pstr((char *)line);
	return(foundXRef);
}

/**************************************
*	Every DR line passed to this routine is checked for cross-references.
*	Information from each xref is extracted and stored into an XRefRec.
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/


Boolean ParseDRLine(short dbcode, EntryRecHdl entryRecHdl, char *line)
{
	Boolean foundXRef = FALSE;
	char	*pos;
	short	i;
	short	xrefCode, keytype;
	char	xrefPrimaryID[PROSITEENTRYNAMELEN+1], xrefSecondaryID[PROSITEENTRYNAMELEN+1];
		
	switch( dbcode ) {
		/* SWISS-PROT: Pointers to EMBL and Prosite.dat */
		case DB_SWISS:
			if ( gDBInfo[DB_EMBL].ndiv > 0 && (pos = strstr(line,"EMBL")) != NULL ) {
				foundXRef = TRUE;
				xrefCode = DB_EMBL;
				keytype = FLD_ACCNUM;
				
				while(*pos++ != ';') ;	/* skip db name */
				while(isspace(*pos)) ++pos;	/* skip white space */
				for( i=0; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
					xrefPrimaryID[i] = *pos;
				xrefPrimaryID[i] = EOS;
				while(*pos++ != ';') ; /* go to entryname */
				while(isspace(*pos)) ++pos;
				for( i=0; *pos != '.' && i < PROSITEENTRYNAMELEN; ++pos,++i)
					xrefSecondaryID[i] = *pos;
				xrefSecondaryID[i] = EOS;
				
				if (!AddXRef(entryRecHdl,xrefCode,xrefPrimaryID,xrefSecondaryID, keytype))
					return(FALSE);
			}
			else if ( gDBInfo[DB_PROSITEDAT].ndiv > 0 && (pos = strstr(line,"PROSITE")) != NULL ) {
				foundXRef = TRUE;
				xrefCode = DB_PROSITEDAT;
				keytype = FLD_ACCNUM;
				
				while(*pos++ != ';') ;	/* skip db name */
				while(isspace(*pos)) ++pos;	/* skip white space */
				for( i=0; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
					xrefPrimaryID[i] = *pos;
				xrefPrimaryID[i] = EOS;
				while(*pos++ != ';') ; /* go to entryname */
				while(isspace(*pos)) ++pos;
				for( i=0; *pos != '.' && i < PROSITEENTRYNAMELEN; ++pos,++i)
					xrefSecondaryID[i] = *pos;
				xrefSecondaryID[i] = EOS;

				if (!AddXRef(entryRecHdl,xrefCode,xrefPrimaryID,xrefSecondaryID,keytype))
					return(FALSE);
			}
			break;
			
		/* EMBL: Pointers to SWISS-PROT */
		case DB_EMBL:
			if ( gDBInfo[DB_SWISS].ndiv > 0 && (pos = strstr(line,"SWISS-PROT")) != NULL ) {
				foundXRef = TRUE;
				xrefCode = DB_SWISS;
				keytype = FLD_ACCNUM;
			
				while(*pos++ != ';') ;	/* skip db name */
				while(isspace(*pos)) ++pos;	/* skip white space */
				for( i=0; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
					xrefPrimaryID[i] = *pos;
				xrefPrimaryID[i] = EOS;
				while(*pos++ != ';') ; /* go to entryname */
				while(isspace(*pos)) ++pos;
				for( i=0; *pos != '.' && i < PROSITEENTRYNAMELEN; ++pos,++i)
					xrefSecondaryID[i] = *pos;
				xrefSecondaryID[i] = EOS;
				
				if (!AddXRef(entryRecHdl,xrefCode,xrefPrimaryID,xrefSecondaryID,keytype))
					return(FALSE);
			}
			else if ( gDBInfo[DB_EPD].ndiv > 0 && (pos = strstr(line,"EPD")) != NULL ) {
				foundXRef = TRUE;
				xrefCode = DB_EPD;
				keytype = FLD_ACCNUM;
			
				while(*pos++ != ';') ;	/* skip db name */
				while(isspace(*pos)) ++pos;	/* skip white space */
				for( i=0; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
					xrefPrimaryID[i] = *pos;
				xrefPrimaryID[i] = EOS;
				while(*pos++ != ';') ; /* go to entryname */
				while(isspace(*pos)) ++pos;
				for( i=0; *pos != '.' && i < PROSITEENTRYNAMELEN; ++pos,++i)
					xrefSecondaryID[i] = *pos;
				xrefSecondaryID[i] = EOS;
				
				if (!AddXRef(entryRecHdl,xrefCode,xrefPrimaryID,xrefSecondaryID,keytype))
					return(FALSE);
			}
			break;
			
		/* Prosite.dat: Pointers to SWISS-PROT. All DR lines are SWISS-PROT xrefs */
		case DB_PROSITEDAT:
			if(gDBInfo[DB_SWISS].ndiv > 0) {
				foundXRef = TRUE;
				xrefCode = DB_SWISS;
				keytype = FLD_ACCNUM;
				
				pos = line + 5;
				while(*pos && *pos != '\r') {
					while(isspace(*pos)) ++pos;	/* skip white space */
					for( i=0; *pos != ',' && i < PROSITEENTRYNAMELEN; ++pos,++i )
						xrefPrimaryID[i] = *pos;
					xrefPrimaryID[i] = EOS;
					while(*pos++ != ',') ; /* go to entryname */
					while(isspace(*pos)) ++pos;
					for( i=0; *pos != ',' && i < PROSITEENTRYNAMELEN; ++pos,++i)
						xrefSecondaryID[i] = *pos;
					xrefSecondaryID[i] = EOS;
				
				
					if (!AddXRef(entryRecHdl,xrefCode,xrefPrimaryID,xrefSecondaryID,keytype))
						return(FALSE);
						
					while(*pos++ != ';') ; /* go to end of xref, skip T, P */
				}
			}
			break;
			
		/* Enzyme: Pointers to SWISS-PROT. All DR lines are SWISS-PROT xrefs */
		case DB_ENZYME:
			if(gDBInfo[DB_SWISS].ndiv > 0) {
				foundXRef = TRUE;
				xrefCode = DB_SWISS;
				keytype = FLD_ACCNUM;
				
				pos = line + 5;
				while(*pos && *pos != '\r') {
					while(isspace(*pos)) ++pos;	/* skip white space */
					for( i=0; *pos != ',' && i < PROSITEENTRYNAMELEN; ++pos,++i )
						xrefPrimaryID[i] = *pos;
					xrefPrimaryID[i] = EOS;
					while(*pos++ != ',') ; /* go to entryname */
					while(isspace(*pos)) ++pos;
					for( i=0; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i)
						xrefSecondaryID[i] = *pos;
					xrefSecondaryID[i] = EOS;
				
					if (!AddXRef(entryRecHdl,xrefCode,xrefPrimaryID,xrefSecondaryID,keytype))
						return(FALSE);

					while(*pos++ != ';') ; /* go to end of xref */
				}
			}
			break;
	}
	
	return(TRUE);
}


/**************************************
*	Every Prosite DO line is passed to this.
*	Information from each xref is extracted and stored into an XRefRec.
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

Boolean ParseDOLine(EntryRecHdl entryRecHdl, char *line)
{
	char		*pos;
	short		i;
	char		xrefPrimaryID[PROSITEENTRYNAMELEN+1];

	if(gDBInfo[DB_PROSITEDOC].ndiv > 0) {
		for( i=0, pos = line + 5; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
			xrefPrimaryID[i] = *pos;
		xrefPrimaryID[i] = EOS;

		return(AddXRef(entryRecHdl,DB_PROSITEDOC,xrefPrimaryID,NULL,FLD_ENAME));
	}
	
	return(TRUE);
}


/**************************************
*	Every Prosite {PS... line is passed to this.
*	Information from each xref is extracted and stored into an XRefRec.
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

Boolean ParsePSLine(EntryRecHdl entryRecHdl, char *line)
{
	char		*pos;
	short		i;
	char		xrefPrimaryID[PROSITEENTRYNAMELEN+1], xrefSecondaryID[PROSITEENTRYNAMELEN+1];
				
	if(gDBInfo[DB_PROSITEDAT].ndiv > 0) {
		for( i=0, pos = line + 1; *pos != ';' && i < PROSITEENTRYNAMELEN; ++pos,++i )
			xrefPrimaryID[i] = *pos;
		xrefPrimaryID[i] = EOS;
	
		while(*pos++ != ';') ; /* go to entryname */
		while(isspace(*pos)) ++pos;
		for( i=0; *pos != '}' && i < PROSITEENTRYNAMELEN; ++pos,++i)
			xrefSecondaryID[i] = *pos;
		xrefSecondaryID[i] = EOS;

		return(AddXRef(entryRecHdl,DB_PROSITEDAT,xrefPrimaryID,xrefSecondaryID,FLD_ACCNUM));
	}
	
	return(TRUE);
}

/**************************************
*	Every EPD FP and F- line is passed to this.
*	Information from each xref is extracted and stored into an XRefRec.
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

Boolean ParseFPLine(EntryRecHdl entryRecHdl, char *line)
{
	char		*pos;
	short		i;
	char		xrefPrimaryID[PROSITEENTRYNAMELEN+1];

	if(gDBInfo[DB_EMBL].ndiv > 0) {
		for( i=0, pos = line + 34; !isspace(*pos) && i < SEQENTRYNAMELEN; ++pos,++i )
			xrefPrimaryID[i] = *pos;
		xrefPrimaryID[i] = EOS;

		return(AddXRef(entryRecHdl,DB_EMBL,xrefPrimaryID,NULL,FLD_ENAME));
	}
	
	return(TRUE);
}


/**************************************
*	Add a new xref to xref buffer
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

Boolean AddXRef(EntryRecHdl entryRecHdl, short dbcode,
					char *primaryID, char *secondaryID, short keytype)
{
	XRefPtr		xrefPtr;
	SignedByte	oldState;
	Size			newSize;
	short			n;
	
	oldState = MyHLock((Handle)entryRecHdl);
	
	/* create a new xref block, if necessary */
	if( (**entryRecHdl).xrefHdl == NULL ) {
		if ( !NewXRefBuf(&(**entryRecHdl).xrefHdl) ) {
			HSetState((Handle)entryRecHdl,oldState);
			return(ErrorMsg(ERR_MEMORY));
		}
		else
			n = 1;
	}
	else {	/* grow xref block */
		newSize = GetHandleSize((Handle)(**entryRecHdl).xrefHdl) + sizeof(XRef);
		SetHandleSize((Handle)(**entryRecHdl).xrefHdl, newSize);
		if(MemError() != noErr)  {
			HSetState((Handle)entryRecHdl,oldState);
			return(ErrorMsg(ERR_MEMORY));
		}
		
		n = newSize / sizeof(XRef);
	}
	
	--n;	/* change to C array offset */
	
	HLock((Handle)(**entryRecHdl).xrefHdl);
	xrefPtr = *(**entryRecHdl).xrefHdl;
	
	xrefPtr[n].dbcode = dbcode;
	
	if(primaryID) {
		strncpy(xrefPtr[n].primaryID,primaryID,PROSITEENTRYNAMELEN);
		xrefPtr[n].primaryID[PROSITEENTRYNAMELEN] = EOS;
	}
	else xrefPtr[n].primaryID[0] = EOS;
	
	if(secondaryID) {
		strncpy(xrefPtr[n].secondaryID,secondaryID,PROSITEENTRYNAMELEN);
		xrefPtr[n].secondaryID[PROSITEENTRYNAMELEN] = EOS;
	}
	else xrefPtr[n].secondaryID[0] = EOS;
	
	xrefPtr[n].keytype = keytype;
	
	HUnlock((Handle)(**entryRecHdl).xrefHdl);
	HSetState((Handle)entryRecHdl,oldState);
	
	return(TRUE);
}
	
/**************************************
*	Allocate memory for new xref buffer
*	Return value:	TRUE, if successful
*						FALSE, if an error occurred
*/

static Boolean NewXRefBuf(XRefHdl *new)
{
	*new = (XRefHdl)NewHandleClear(sizeof(XRef));
	return( *new != NULL );
}


/**************************************
*	Add one menu item for each cross-reference to XREF hierarchical menu
*/

void AddXRefsToMenu(WDPtr wdp)
{
	XRefHdl		xrefHdl;
	EntryRecHdl	entryRecHdl;
	short			nxref;
	short			i;
	char			temp[256];
	SignedByte	oldState;
	
	if(wdp == NULL || ((WindowPeek)wdp)->windowKind != entryW)
		return;
		
	/* remove old xrefs */
	RemoveXRefsFromMenu();
	
	if( (entryRecHdl = (EntryRecHdl)(wdp->userHandle)) == NULL )
		return;
		
	if( (xrefHdl = (**entryRecHdl).xrefHdl) == NULL )
		return;
		
	oldState = MyHLock((Handle)xrefHdl);
	
	nxref = (short)(GetHandleSize((Handle)xrefHdl)/sizeof(XRef));
	for( i = 0; i < nxref && i < MAXMENUXREFS; ++i ) {
		strcpy(temp,gDBInfo[(*xrefHdl)[i].dbcode].DBName);
		strcat(temp," ");
		strcat(temp,(*xrefHdl)[i].primaryID);
		if( *(*xrefHdl)[i].secondaryID ) {
			strcat(temp," [");
			strcat(temp,(*xrefHdl)[i].secondaryID);
			strcat(temp,"]");
		}
		
		c2pstr(temp);
		AppendMenu(gMenu[XREF],(StringPtr)temp);
		EnableItem(gMenu[XREF],i+2);
	}
	
	HSetState((Handle)xrefHdl,oldState);
}

/**************************************
*	Remove all cross-references from XREF menu
*/

static void RemoveXRefsFromMenu()
{
	short i;
	
	for(i = CountMItems(gMenu[XREF]); i > 2 ; --i)
		DelMenuItem(gMenu[XREF],i);
}

