#pragma once

/*
*********************************************************************
*	
*	EMBL_Search.h
*	Main header file
*		
*	Rainer Fuchs
*	EMBL Data Library
*	Postfach 10.2209
*	D-6900 Heidelberg, FRG
*	E-mail: fuchs@embl-heidelberg.de
*		
**********************************************************************
*	
*/ 

#include "general.h"

/*
*	symbols
*/

#define kApplSignature	'DLCD'
#define kResFileType		'TEXT'
#define kQryFileType		'QuRy'
#define kPrefFileType	'CDpr'
#define kEntryFileType	'TEXT'

#define kQueryRsrcType	'QRY '
#define kQueryRsrcName	"\pQuery"
#define kHitmapRsrcType	'HITS'
#define kHitmapRsrcName	"\pHitmap"
#define kStdRsrc			128

#define kNameStringRsrcTmpl	300
#define kNameStringRsrc			-16396
#define kNameStringRsrcType	'STR '
#define kNameStringRsrcName	"\pName string"

#define SEQENTRYNAMELEN			10		/* maximum length of entry names */
#define PROSITEENTRYNAMELEN	21

#define DIVNAMELEN		12 	/* maximum length of division file names */
#define ACCNOLEN			10		/* maximum length of an accession number */

#define MAXBUFLINES 260		/* No of lines in results short description buffer */
									/* Must always be greater than max. no of lines
										which can be displayed on a screen */
#define ENTRYBUFLINES 1000	/* lines in entry buffer */

#define MAXBUFPARTS	30		/* number of entry buffer chunks.
										( less than  (USHRT_MAX/2)/ENTRYBUFLINES)  */ 

#define MAXWIN			20		/* Maximum of windows we allow. We allocate memory
										for the windows at startup to avoid heap
										fragmentation */


#define DB_NUM				6		/* Database codes */
#define DB_EMBL			0
#define DB_SWISS			1
#define DB_PROSITEDAT	2
#define DB_PROSITEDOC	3
#define DB_ENZYME			4
#define DB_EPD				5

#define FLD_NUM		8			/* # of fields plus 1 (first is unused) */
#define FLD_ENAME		1
#define FLD_ACCNUM	2
#define FLD_KEYWORD 	3
#define FLD_FREETEXT 4
#define FLD_AUTHOR	5
#define FLD_TAXON		6
#define FLD_ENZNAME	7

#define DBNAMELEN	20			/* length of database names */
#define DBRELNUMLEN 10		/* length of release number string */

/*
*	General type definitions.
*	Pascal strings are of type Str255, C strings of type char []
*/

typedef char CString80[81];
typedef CString80 *CString80Ptr, **CString80Hdl;
typedef char CStringDiv[DIVNAMELEN + 3]; /* for ;1 and EOS */
typedef CStringDiv **CStringDivHdl;

typedef long **HitmapHdl;

typedef struct XRef {
	short	dbcode;
	char	primaryID[PROSITEENTRYNAMELEN+1];
	char	secondaryID[PROSITEENTRYNAMELEN+1];
	short keytype;
} XRef, *XRefPtr, **XRefHdl;

typedef struct DirSpec {		/* if index files are moved to a hard disk we store
										the volume name and the dirID of the new folders */
	Str27	volName;
	long	dirID;
}	DirSpec;

typedef struct InxInfo {
	u_long	hit_nrec;				/* no of records in hit file */
	u_long	trg_nrec;				/* no of records in target file */
	u_short	trg_recsize;			/* target file record size */
	u_long	pnx_nrec;				/* no of records in preindex */
	u_short	pnx_recsize;			/* preindex record size */
	Handle	pnxHdl;					/* Handle to preindex */
} InxInfo;
	
/*
*	Index files
*/

typedef struct {						/* database information */
	short		DataVRefNum;			/* volume reference number of data disk */
	Str255	DataDName;				/* name of directory that contains data files */
	long		DataDirID;				/* DirID of data directory */
	short		InxVRefNum;				/* volume reference number of index disk */
	Str255	InxDName;				/* name of directory that contains index files */
	long		InxDirID;				/* DirID of index directory */
	char		DBName[DBNAMELEN+1];	/* Database name */
	char		DBRelNum[DBRELNUMLEN+1]; /* Release number */
	char		DBRelDate[50];			/* Release date */
	CStringDivHdl gDivNames;			/* Handle to array of division names */
	short		ndiv;						/* no of database division files */
	u_long	ename_nrec;				/* no of records in entryname index */
	u_short	ename_recsize;
	u_short	ename_len;
	Handle	ename_pnxHdl;
	u_long	ename_pnxNrec;
	u_short	ename_pnxRecsize;
	u_short	brief_recsize;
	InxInfo	inxInfo[FLD_NUM];
} DBInfo;

typedef struct {				/* division index record */
	u_short	div_code;
	char 		filename[12];
} DivisionRec;

typedef struct {				/* date information */
	u_byte dummy;
	u_byte year;
	u_byte month;
	u_byte day;
} Date;

typedef struct {				/* index file header */
	u_long	file_size;
	u_long	nrecords;
	u_short	record_size;
	char		db_name[DBNAMELEN];
	char		db_relnum[DBRELNUMLEN];
	Date		db_reldate;
	u_byte	free_space[256];
} Header;

typedef struct InxFNames {
	Str255 hitFName;
	Str255 trgFName;
	Str255 pnxFName;
} InxFNames;

typedef struct {						/* list of file names */
	Str255 divFName;					/* Division lookup table file name */
	Str255 enameIdxFName;			/* Entryname index file name */
	Str255 enamePnxFName;
	Str255 briefIdxFName;			/* Short description file name */
	InxFNames inxFNames[FLD_NUM];
} IndexFiles;

typedef struct {				/* entry name index record. value moved to the end. */
	u_long	annotation_offset;
	u_long	sequence_offset;
	u_short	div_code;
	char		value[];
} EnameRec;

typedef struct {				/* short description index record. Entryname moved to end. */
	char		accno[10];
	u_long	seqlen;
	char		description[80];
	char		entryname[];
} BriefDescRec;

typedef u_long HitRec;		/* generic hit file record */

typedef struct {				/* generic target file record */
	u_long	nhits;
	u_long	hitPtr;
	char		value[];
} TargetRec;


/*
*	Windows
*/

#define queryW	dialogKind
#define resW	userKind
#define entryW	userKind+1


/*
*	Window description structure
*/

typedef struct {
	DialogRecord	dRec;				/* dialog record for query windows; for entry and
												res windows we simply treat it as a
												WindowRecord */
	Boolean			inUse;			/* indicates whether this window record is 
												currently in use */
	ControlHandle	vScroll;			/* vertical scroll bar handle */
	ControlHandle	hScroll;			/* horizontal scroll bar handle */
	ControlHandle	ctrl1;			/* other controls */
	ControlHandle	ctrl2;
	ControlHandle	ctrl3;
	ControlHandle	ctrl4;
	FSSpec			theFSS;			/* File system spec of associated file */
	Handle			userHandle;		/* Handle to user data */
	Boolean			dirty;			/* is contents of window saved to disk ? */
	Boolean			inited;			/* Does a file already exist?	*/
} WDRec, *WDPtr;


/*
*	window-specific data structures
*/

#define MAXSUBQRY		5

typedef struct {
	short		field;								/* indexed field */
	Str255	text;									/* query text (simple ASCII string) */
} QueryFld;

typedef struct {
	long		magic;								/* magic number to identify queries */
	short		dbcode;								/* database */
	char		DBRelNum[DBRELNUMLEN+1];		/* release */
	short		gBooleanOp;							/* global boolean operator */
	QueryFld	subQuery[MAXSUBQRY];				/* subqueries */
} QueryRec, *QueryPtr, **QueryHdl;


/* results window */

typedef struct {
	u_long ename_rec;					/* offset into ename index */
	struct {								/* diverse flags */
		unsigned char selected:1;
	} flags;
} HitlistRec, *HitlistPtr,**HitlistHdl;


typedef struct {
	short				dbcode;			/* database which was searched for these hits */
	short				nhits;			/* no of hits */
	short				nsel;				/* no of selected hits */
	HitmapHdl		hitmapHdl;		/* pointer to hitmap (hit bitmap) */
	HitlistHdl 		hlHdl;			/* handle to hitlist */
	CString80Hdl	descBufHdl;		/* handle to description buffer */
	short				buftop;			/* hit no of top line in description buffer */
	QueryHdl			queryHdl;		/* handle to query record */
} ResultRec, *ResultPtr, **ResultHdl;



/* entry window */

typedef struct {
	FSSpec			theFSS;						/* File system spec of division file */
	long				bufpart[MAXBUFPARTS];	/* byte offsets of parts of entry */
	short				dbcode;						/* database used */
	short				nlines;						/* # of lines of entry */
	short				buftop;						/* top line in buffer */
	long				seqline_offset;			/* first sequence line (byte offset) */
	short				seqline_no;					/* first sequence line (byte offset) */
	CString80Hdl	lineBufHdl;					/* Handle to line buffer */
	XRefHdl			xrefHdl;						/* Handle to cross-reference buffer */
	short				firstSel;					/* first selected line in entry */
	short				lastSel;						/* last selected line in entry */
} EntryRec, *EntryRecPtr, **EntryRecHdl;