/*
 * Spyoo.c
 *
 * This is  Copyright 1992 by Jamie R. McCarthy.  All rights reserved.
 * Please see HeadlinesMain.c for distribution information.
 *
 * Spyoo.c is the guts of the routines.  All the important parsing and
 * expansion routines (readLine, process, setup, save, evaluate, lookup,
 * namecomp) were essentially taken straight out of Greg's spew code,
 * with a few modifications to handle reading from the resource
 * and so on.
 *
 * I apologize for the extremely messy code...
 *
 */



/******************************/

#include "Spyoo.h"

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "JMassMalloc.h"

/******************************/

OSErr setup(class *cp);
char *save(char *str);

class *lookup(char *str);
short namecomp(char *a, char *b);

/******************************/



	static int classComp(const register class *a, const register class *b);
	static int classComp(const register class *a, const register class *b)
	{
		if (a==b) return 0;
		return strcmp(a->name, b->name);
	}
	
	
OSErr readText(void)
{
	register class *cp;
	defn *dp;
	register defn **update;
	OSErr theOSErr;
	
	theOSErr = noErr;
	
	gNClasses = 0;
	gClass = malloc( (long) kMaxNClasses * sizeof(class) );
	if (gClass == NULL) {
		return memFullErr;
	}
	
	cp = gClass;
	readLine();
	if (gInLine[0] != '%') {
		return expectedClassDefnErr;
	}
	
	while (gInLine[1] != '%') {
		
		if (gNClasses == kMaxNClasses) {
			freeHeadlinesAllocations();
			return tooManyClassesErr;
		}
		
		theOSErr = setup(cp);		/* set up the class struct */
		if (theOSErr != noErr) {
			freeHeadlinesAllocations();
			return theOSErr;
		}
		
		readLine();
		if (gInLine[0] == '%') {
				// must have at least one instance of each class
			freeHeadlinesAllocations();
			return expectedClassInstanceErr;
		}
		
		update = &(cp->list);
		do {
			OSErr theOSErr;
			theOSErr = process(&dp);
			if (theOSErr != noErr) {
				freeHeadlinesAllocations();
				return theOSErr;
			}
			*update = dp;
			cp->weight += dp->cumul;
			dp->cumul = cp->weight;
			update = &(dp->next);
		} while (readLine(), gInLine[0] != '%');
		
		++gNClasses;
		++cp;
		*update = NULL;
		
	}
	
	qsort( (char*) gClass, gNClasses, sizeof(class), (__cmp_func) classComp );
	
	return noErr;
}



	static char *getLine(void);
	static char *getLine(void)
	{
		register short nChars = kMaxNCharsPerLine;
		register char *dest = &gInLine[0];
		register short ch;
		
		if (gInRsrcMark >= gInRsrcLength) {
			return NULL;
		}
		
		while (--nChars && gInRsrcMark < gInRsrcLength) {
			ch = (*gInRsrcHndl)[gInRsrcMark++];
			if (gInRsrcMark >= gInRsrcLength || (ch == '\r') || (ch == '\n')) {
				break;
			} else {
				*dest++ = ch;
			}
		}
		*dest = '\0';
		return dest;
	}
	
	
void readLine(void)
{
	register char *p;
	do {
		if (getLine() == NULL) {
				/* Create an EOF, in order to fall out of the do loop. */
			gInLine[0] = gInLine[1] = '%';
			gInLine[2] = '\0';
		};
		p = &gInLine[0];
		while (*p != BSLASH && *p != '\0' ) ++p;
		if (*p == BSLASH && *(p+1) == '*') *p = '\0';
	} while (gInLine[0] == '\0');
}



OSErr setup(register class *cp)
{
	char scratch[kMaxNCharsPerLine];
	register char *p;
	register char *p2;
	
	cp->weight = 0;							// set up cp's other parameters with defaults
	cp->list = NULL;
	cp->name = NULL;
	cp->tags = gNullTags;
	
	p = &gInLine[1];							// point to right after the % */
	while (*p == ' ') {
		++p;										// skip spaces
	}
	if (!isalnum(*p)) {
		return badClassHeaderErr;			// make sure it's an OK char to start with
	}
	p2 = scratch;
	do {
		*p2++ = *p++;							// read the name into the temp array
	} while (isalnum(*p));
	*p2 = '\0';
	
	cp->name = save(scratch);					// allocate a copy of the name; store it in cp
	if (cp->name == NULL) {
		return memFullErr;
	}
	
	while (TRUE) {										// read in tags, if any
		
		switch (*p++) {
			case '\0':									// end of line, we're done
				return noErr;
			case ' ':									// spaces are allowed
				break;
			case LBRACE:								// open brace--read in tags
				if (cp->tags != gNullTags ) {		// if we already have tags, error
					return badClassHeaderErr;
				}
				p2 = scratch;
				*p2++ = ' ';							// start with null tag
				while (*p!=RBRACE) {
					if( !isalnum(*p)) {				// if non-alphanumeric tag char, error
						return badClassHeaderErr;
					}
					*p2++ = *p++;						
				}
				++p;										// skip reading the close brace
				*p2 = 0;									// null-terminate the string
				cp->tags = save(scratch);			// allocate a copy, and save the tags
				if (cp->tags == NULL) {				// couldn't save, error
					return memFullErr;
				}
				break;
			default: {
				return badClassHeaderErr;			// unrecognized char, error
			}	break;
		}
		
	}
	
	return noErr;
}



OSErr process(defn **theDefn)
	/* Create a 'processed' version of gInLine, and return a pointer to
	 * the definition. The 'cumul' field is temporarily used to hold the
	 * assigned weight of the line.
	 */
{
	static char scratch[kMaxNBytesInDef];
	register char *p;
	register char *pout;
	register defn *dp;
	register short c;

	dp = (defn *) malloc( sizeof(defn) );
	if (dp == NULL) {
		return memFullErr;
	}
	dp->cumul = 0;
	dp->str = NULL;
	dp->next = NULL;
	
	p = &gInLine[0];
	pout = scratch;
	
	if (*p == '(') {									// get line weight
		while (*++p ==' ') ;							// skip spaces
		if (!isdigit(*p)) {
			return badLineWeightErr;				// weights must be digits
		}
		c = *p - '0';
		while (isdigit(*++p)) {
			c = c*10 + (*p - '0' );					// accumulate weight
		}
		while (*p == ' ') {
			++p;											// skip spaces
		}
		if (*p != ')') {
			return badLineWeightErr;				// no closing paren, error
		}
		++p;												// skip reading the close paren
		dp->cumul = c;									// set the weight
	} else {
		dp->cumul = 1;									// default weight is 1
	}
	
	while ((c = *p++) != '\0') {
		switch (c) {
			case BSLASH:
				*pout++ = BSLASH;
				if (isalnum(*p)) {
					do {
						*pout++ = *p++;
					} while (isalnum(*p));
					*pout++ = SLASH;
					if (*p == SLASH) {
						++p;
						if (!isalnum(*p) && *p != ' ' && *p != '&') {
							*pout++ = ' ';
						} else {
							*pout++ = *p++;
						}
					} else {
						*pout++ = ' ';
					}
				} else {
					*pout++ = *p;
					if (*p != '\0') {
						 ++p;
					} else {
						--pout;	/* delete spurious '\' */
						readLine();	/* get new line */
						p = &gInLine[0];	/* point to it */
					}
				}
				break;
			default:
				*pout++ = c;
		}
	}
	
	*pout = '\0';
	dp->str = save(scratch);
	if (dp->str == NULL) {
		return memFullErr;
	} else {
		*theDefn = dp;
		return noErr;
	}
}



char *save(char *str)
	/* Make a copy of the null-terminated string in str, and return that copy. */
{
	register char *p;
	p = malloc( (strlen(str)+1) * sizeof(char) );
	if (p == NULL) return NULL;
	return strcpy(p, str);
}



/******************************/



OSErr evaluate(char *s, short deftag)
	/*
	 * Given the name of a class, terminated by SLASH, evaluate() will
	 * (1) find its class descriptor, by calling lookup;  (2) pick a
	 * definition;  and (3) evaluate that definition, i.e. recursively
	 * expand the class descriptor.
	 */
{
	register class *cp;
	register defn *dp;
	register char *p;
	register short c;
	register Boolean writing;
	register short i, variant;
	short tagLevel;
	OSErr theOSErr;
	
	theOSErr = noErr;
	
	cp = lookup(s);
	if (cp == NULL) {
		return noMainClassErr;
	}
	
	c = ((char*) memchr(s, SLASH, kMaxNCharsPerLine)) [1];
	if (c != '&') deftag = c;
	p = memchr(cp->tags, deftag, kMaxNTags);
	
	if (p == NULL) {
			/* Unknown variant--just make it blank. */
		variant = 0;
		deftag = ' ';
	} else {
		variant = p - cp->tags;
	}
	
	i = jrLinearShort(&gJR, 0, cp->weight-1);
	
	dp = cp->list;
	
	while (dp->cumul <= i) {
		dp = dp->next;
	}
	
	tagLevel = 0;
	writing = TRUE;
	p = dp->str; // this is the string
	
	while (TRUE) {
		
		switch (c = *p++) {
			case '\0': {
				return noErr;
			}	break;
				
			case BSLASH: {
				if ( (c = *p++) == '\0' ) {
					return noErr; // even tho' you shouldn't end a line with a BSLASH
				} else if (c == '!') {
					 if (writing) {
					 	jPutChar('\r'); // "\!" = newline
					 }
				} else if (isalnum(c)) {
					if (writing) {
						evaluate(p-1, deftag); // recurse
					}
					while (*p != SLASH) ++p;
					p += 2;
				} else {
					if (writing) {
						jPutChar(c);
					}
				}
			}	break;
				
			case LBRACE: {
				if (tagLevel == 0) {
					tagLevel = 1;
					writing = (variant == 0);
				} else {
					if (writing) {
						jPutChar(LBRACE);
					}
				}
			}	break;
				
			case VBAR: {
				if (tagLevel != 0) {
					writing = (variant == tagLevel++);
				} else {
					jPutChar(VBAR);
				}
			}	break;
				
			case RBRACE: {
				if (tagLevel != 0) {
					tagLevel = 0;
					writing = TRUE;
				} else {
					jPutChar(RBRACE);
				}
			}	break;
				
			default: {
				if (writing) {
					jPutChar(c);
				}
			}	break;
				
		}
	}
	
	return noErr;
}



class *lookup(char *str) // delimited by SLASH, not '\0'
{
	short first, last, try, comp;
	
	first = 0;
	last = gNClasses-1;
	while (first <= last) {
		try = (first+last) >> 1;
		comp = namecomp(str, gClass[try].name);
		if (comp == 0) {
			return &gClass[try];
		}
		if (comp > 0) {
			first = try+1;
		} else {
			last = try-1;
		}
	}
	
	return NULL;
}



short namecomp(register char *a, register char *b)
	/* 'a' is delimited by a SLASH, 'b' by a NULL. */
{
	register short ac;
	while (TRUE) {
		ac = *a++;
		if (ac == SLASH) ac = '\0';
		if (ac < *b) return -1;
		if (ac > *b++) return 1;
		if (ac == '\0') return 0;
	}
}

