/*
 * playfour.c - routines for playing 4-tone chords.
 */

#include <quickdraw.h>
#include <toolutil.h>
#include <sound.h>

/*
 * semir[] - the "rate" value corresponding to each semitone.
 * (relative to Low Cb -- 2 Octaves and a semitone below middle C).
 */

static long semir[] = {
    0xB5C8L,

    0xC097L, 0xCC0BL, 0xD82DL, 0xE508L, 0xF2A6L, 0x10114L,
    0x1105DL, 0x12090L, 0x131B8L, 0x143E6L, 0x15729L, 0x16B90L,
    
    0x1812FL, 0x19816L, 0x1B05AL, 0x1CA10L, 0x1E54DL, 0x20228L,
    0x220BBL, 0x2411FL, 0x26370L, 0x287CCL, 0x2AE51L, 0x2D721L,
    
    /* Middle C */
    0x3025DL, 0x3302CL, 0x360B5L, 0x39420L, 0x3CA99L, 0x40450L,
    0x44176L, 0x4823EL, 0x4C6E1L, 0x50F98L, 0x55CA2L, 0x5AE41L,
    
    0x604BBL, 0x66059L, 0x6C169L, 0x7283FL, 0x79533L, 0x808A1L,
    0x882ECL, 0x9047DL, 0x98DC2L, 0xA1F30L, 0xAB945L, 0xB5C83L,
    
    0xC0976L, 0xCC0B1L, 0xD82D2L, 0xE507EL, 0xF2A65L, 0x101141L,
    0x1105D8L, 0x1208F9L, 0x131B83L, 0x143E61L, 0x15728AL, 0x16B906L,
    
    0x1812EBL, 0x198163L
};

#define MIDC_SEMI	25	/* semir[] index of middle C		*/

static Wave wave;		/* the waveform used by everybody	*/
static FTSoundRec sndrec;	/* sound rec used for everything	*/
static FTSynthRec synthrec;	/* synth rec used for everything	*/

/*
 * mus4init() - initialize the sound structures for 4-part music.
 */

mus4init()
{
    short i;
    char *cp;
    
    for (cp = &wave[0], i = 0; cp < &wave[256]; ++cp, ++i) {
	*cp = (char) i;
    }
    
    sndrec.sound1Phase = 0L;
    sndrec.sound2Phase = 0L;
    sndrec.sound3Phase = 0L;
    sndrec.sound4Phase = 0L;
    sndrec.sound1Wave = &wave[0];
    sndrec.sound2Wave = &wave[0];
    sndrec.sound3Wave = &wave[0];
    sndrec.sound4Wave = &wave[0];
    
    synthrec.mode = ftMode;
    synthrec.sndRec = &sndrec;
}

/*
 * mus4play() - play the given semitones for the given duration.
 */

mus4play(dur, s1, s2, s3, s4)
short dur;			/* duration in 60th's of a second	*/
short s1, s2, s3, s4;		/* semitones to play (0 == middle C)	*/
				/* 1000 == silent			*/
{
    static Fixed semirate();

    sndrec.duration = dur;
    sndrec.sound1Rate = semirate(s1);
    sndrec.sound2Rate = semirate(s2);
    sndrec.sound3Rate = semirate(s3);
    sndrec.sound4Rate = semirate(s4);
    
    StartSound((Ptr) &synthrec, sizeof(synthrec), (ProcPtr) -1);
}

/*
 * semirate() - given a semitone (0 == middle C, 1000 == silent),
 *  give the "rate" value to plug into the four-tone synthesizer.
 */

static
Fixed
semirate(s)
short s;		/* semitone to convert			*/
{
    Fixed rate;
    
    if (s == 1000) {
	rate = (Fixed) 0L;
    } else {
	rate = (Fixed) semir[s + MIDC_SEMI];
    }
    return(rate);
}
